<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/
        
require_once 'fun/assemblies.php';
require_once 'fun/common.php';
require_once 'util/assembly_flags.php';
define("ASSEMBLY_PAGE_SIZE", 18);
define("ASSEMBLY_ITEM_PAGE_SIZE", 18);
$default_keys = array(
    'assembly_id' => 0);
$keys = get_settings($default_keys);
$default_settings = array(
    'serial_number_filter' => '',
    'assembly_type_filter' => 0,
    'person_filter' => 0,
    'assembly_person_filter' => 0,
    'item_description_filter' => '',
    'page_number' => 1,
    'item_page_number' => 1,
    'p_page_number' => 1,
    'p_item_page_number' => 1,
    'work_in_progress' => '',
    'p_work_in_progress' => '',
    'iqn_item_id' => null);
$settings = get_settings($default_settings);
$max_items_pages = 0;
if(isset($_SERVER['HTTP_REFERER'])) {
    $referer = parse_url($_SERVER['HTTP_REFERER']);
    if($referer['path'] == '/ShopChart/assemblies.php') {
        if(isset($_POST['add_assembly'])) {
            $keys['assembly_id'] = add_assembly($_POST['new_assembly_type_id'], $_POST['new_serial_number']);
        } elseif($_SERVER['REQUEST_METHOD'] == 'POST') {
            update_attributes($keys['assembly_id'], $_POST);
        }
        $settings['page_number'] = get_page_number('page_number', 'previous_page', 'next_page');
        $settings['item_page_number'] = get_page_number('item_page_number', 'previous_item_page', 'next_item_page');
    }
}
$assembly_types = get_assembly_types(true);
$assemblies = get_assemblies($settings['serial_number_filter'], $settings['assembly_type_filter'],
    $settings['person_filter'], $settings['user']);
$people = get_assembly_people();
$max_pages = get_max_pages($assemblies, ASSEMBLY_PAGE_SIZE);
$settings['page_number'] = go_to_page($keys['assembly_id'], 'aAssemblyID', $assemblies, ASSEMBLY_PAGE_SIZE,
    $settings['page_number'], $max_pages, ($settings['p_page_number'] == $settings['page_number']));
print '
<!DOCTYPE html>
<html>
    <head>
        <style type="text/css">
            @import url(css/shop_chart.css);
            @import url(css/assemblies.css);
        </style>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <title>Assemblies</title>
    </head>
    <body>
        <form action="assemblies.php" method="post">';
$nav_links = '
                    <div class="nav_link">
                        <a href="assembly_types.php">Assembly Types</a>
                    </div>';
include 'top_nav.php';
if(isset($settings['iqn_item_id'])) {
    print iqn_notice($settings, $keys);
}
print '
            <div id="top_half">
                <table id="assemblies">';
if($max_pages > 1) {
    print '
                    <tr class="page_nav">
                        <td colspan="3">
                            <input type="submit" name="previous_page" value="Previous" />
                            <input type="text" name="page_number" value="' . $settings['page_number'] . '" /> of ' . $max_pages . '
                            <input type="submit" name="next_page" value="Next" />
                        </td>
                    </tr>';
}
print '
                    <tr class="field_header">
                        <th>Serial Number</th>
                        <th>Assembly Type</th>
                        <th>Owner</th>
                        <th>Work in</th>
                    </tr>
                    <tr class="field_header">
                        <th>
                            <input type="text" name="serial_number_filter" value="' .
    htmlspecialchars($settings['serial_number_filter']) . '" />
                        </th>
                        <th>
                            <select name="assembly_type_filter" onchange="this.form.submit();">
                                <option value="0"></option>';
foreach($assembly_types as $id => $description) {
    print '
                                <option value="' . $id . '"' . ($id == $settings['assembly_type_filter'] ?
        ' selected="true"' : '') . '>' . htmlspecialchars($description) . '</option>';
}
print '
                            </select>
                        </th>
                        <th>
                            <select name="person_filter" onchange="this.form.submit();">
                                <option value="0"></option>
                                <option value="-1"' . ($settings['person_filter'] == -1 ? ' selected="true"' :
    '') . '>No Person</option>';
foreach($people as $id => $full_name) {
    print '
                                <option value="' . $id . '"' . ($id == $settings['person_filter'] ?
        ' selected="true"' : '') . '>' . htmlspecialchars($full_name) . '</option>';
}
print '
                            </select>
                        </th>
                        <th>Progress</th>
                    </tr>';
$displayed = 0;
$assembly_types = get_assembly_types(false);
$totals = array('MonetaryTotalAmount' => '', 'TimeTotalAmount' => '', 'TotalWeight' => '');
while ($assembly = $assemblies->fetch_assoc()) {
    if($assembly['aAssemblyID'] <> null) {
        if (!isset($selected_assembly) && ($assembly['aAssemblyID'] == $keys['assembly_id'] || 
            $keys['assembly_id'] == 0)) {
            $keys['assembly_id'] = $assembly['aAssemblyID'];
            $selected_assembly = $assembly;
            $assembly_people = get_assembly_people($keys['assembly_id']);
            if(!isset($assembly_people[$settings['assembly_person_filter']])) {
                foreach($assembly_people as $id => $full_name) {
                    $settings['assembly_person_filter'] = $id;
                    break;
                }
            }
            $assembly_items = get_assembly_items($keys['assembly_id'], $settings['assembly_person_filter'],
                $settings['item_description_filter'], $totals);
            $max_items_pages = get_max_pages($assembly_items, ASSEMBLY_ITEM_PAGE_SIZE);
            if(isset($_REQUEST['iqn_item_id'])) {
                $settings['item_page_number'] = go_to_page($_REQUEST['iqn_item_id'], 'aifaItemID', $assembly_items,
                    ASSEMBLY_ITEM_PAGE_SIZE, $settings['item_page_number'], $max_items_pages,
                    ($settings['p_item_page_number'] == $settings['item_page_number']));
            } else {
                $settings['item_page_number'] = go_to_page(0, '', $assembly_items, ASSEMBLY_ITEM_PAGE_SIZE,
                    $settings['item_page_number'], $max_items_pages, false);
            }
            print '
                        <tr class="selected_row">
                            <td class="serial_number">
                                <input type="submit" class="delete" name="delete_assembly" value=" "/>
                                <input type="text" name="aSerialNumber" value="' .
                htmlspecialchars($assembly['aSerialNumber']) . '" />
                            </td>
                            <td class="assembly_type">
                                <select name="aAssemblyTypeID" onchange="this.form.submit();">';
            foreach($assembly_types as $assembly_type_id => $description) {
                print '
                                    <option value="' . $assembly_type_id . '"' . ($assembly_type_id ==
                    $assembly['aAssemblyTypeID'] ? ' selected="true"' : '') . '>' .
                    htmlspecialchars($description) . '</option>';
            }
            print '
                                </select>
                            </td>';
            if(count($assembly_people) >= 2) {
                print '
                            <td class="person">
                                <select name="assembly_person_filter" onchange="this.form.submit();">';
                foreach($assembly_people as $id => $full_name) {
                    print '
                                    <option value="' . $id . '"' . ($id == $settings['assembly_person_filter'] ?
                        ' selected="true"' : '') . '>' . htmlspecialchars($full_name) . '</option>';
                }
                print '
                                </select>
                            </td>';
            } else {
                print '
                            <td class="person">
                                ' . $assembly_people[$settings['assembly_person_filter']] . '
                            </td>';
            }
            print '
                            <td class="work_in_progress">
                                <input type="checkbox" name="work_in_progress"' . ($assembly['aFlags'] & AF_WORK_IN_PROGRESS ?
                ' checked="true"' : '') . ' onchange="this.form.submit();" />
                            </td>
                        </tr>';
        } else {
            if($displayed % 2) {
                print '
                        <tr class="odd_row">';
            } else {
                print '
                        <tr class="even_row">';
            }
            print '
                            <td class="serial_number">
                                <a href="assemblies.php?assembly_id=' . $assembly['aAssemblyID'] .
                get_params(remove_settings($settings, 'assembly_person_filter'), false) . '">
                                ' . htmlspecialchars($assembly['aSerialNumber']) . '
                                </a>
                            </td>
                            <td class="assembly_type">
                                ' . $assembly_types[$assembly['aAssemblyTypeID']] . '
                            </td>
                            <td class="person">
                            ' . ($assembly['appPersonID'] == 0 ? '' :
                htmlspecialchars($people[$assembly['appPersonID']])) . '
                            </td>
                            <td class="work_in_progress">
                                <input type="checkbox" name="work_in_progress"' . ($assembly['aFlags'] & AF_WORK_IN_PROGRESS ?
                ' checked="true"' : '') . ' disabled="true" onchange="this.form.submit();" />
                            </td>
                        </tr>';
        }
    }
    if(++$displayed >= ASSEMBLY_PAGE_SIZE) {
        break;
    }
}
print '
                    <tr class="footer">
                        <td class="serial_number">
                            <input type="text" name="new_serial_number" />
                        </td>
                        <td class="assembly_type">
                            <select name="new_assembly_type_id">';
foreach($assembly_types as $assembly_type_id => $description) {
    print '
                                <option value="' . $assembly_type_id . '">' . htmlspecialchars($description) .
        '</option>';
}
print '
                            </select>
                        </td>
                        <td>
                        </td>
                        <td>
                            <input type="submit" name="add_assembly" value="Add Assembly"' .
    (count($assembly_types) == 0 ? ' disabled="true"' : '') . ' />
                        </td>
                    </tr>
                </table>
                <div id="assembly_image">';
if(isset($selected_assembly)) {
    if($selected_assembly['aImageURL'] <> '') {
        if($selected_assembly['aImageLink'] <> '') {
            print '
                    <a href="' . htmlspecialchars($selected_assembly['aImageLink']) . '" target="_blank">
                        <img src="' . htmlspecialchars($selected_assembly['aImageURL']) . '" alt="' .
            htmlspecialchars($selected_assembly['aSerialNumber']) . '" />
                    </a>';
        } else {
         print '
                    <img src="' . htmlspecialchars($selected_assembly['aImageURL']) . '" alt="' .
            htmlspecialchars($selected_assembly['aSerialNumber']) . '" />';
        }
        print '
                    <br />';
    }
    print '
                    <label>Image URL</label> <input type="text" name="aImageURL" caption="Image URL" value="' .
        htmlspecialchars($selected_assembly['aImageURL']) . '" /><br />
                    <label>Image Link</label> <input type="text" name="aImageLink" caption="Image Link" value="' .
        htmlspecialchars($selected_assembly['aImageLink']) . '" /><br />
                    <label>Total Money Invested:</label> ' . money_format('%.2n', $totals['MonetaryTotalAmount']) . 
        '<br />
                    <label>Total Revenue:</label> Not yet available<br />
                    <label>Total Time Invested:</label> ' . timeize($totals['TimeTotalAmount']) . '<br />';
    if($totals['TimeTotalAmount'] == 0) {
        print '
                    <label>Effective Hourly Rate:</label>$0.00<br />';
    } else {
        print '
                    <label>Effective Hourly Rate:</label> ' . money_format('%.2n', ($totals['RevenueTotalAmount'] -
        $totals['MonetaryTotalAmount']) / ($totals['TimeTotalAmount'] / 3600)) . '<br />';
    }
} else{
    print '
                    <label>Image URL</label><input type="text" disabled="true" /><br />
                    <label>Image Link</label><input type="text" disabled="true" />';
}
    print '
                </div>
            </div>
            <div id="bottom_half">
                <table id="assembly_items">';
    if($max_items_pages > 1) {
        print '
                    <tr class="page_nav">
                        <td colspan="5">
                            <input type="submit" name="previous_item_page" value="Previous" />
                            <input type="text" name="item_page_number" value="' . $settings['item_page_number'] . '" />
                            of '. $max_items_pages . '
                            <input type="submit" name="next_item_page" value="Next" />
                        </td>
                    </tr>';
    }
    print '
                    <tr class="field_header">
                        <th>Description</th>
                        <th>Qty</th>
                        <th>Qty</th>
                        <th>Time</th>
                        <th>Money</th>
                    </tr>
                    <tr class="field_header">
                        <th>
                            <input type="text" name="item_description_filter" value="' .
    htmlspecialchars($settings['item_description_filter']) . '" />
                        </th>
                        <th>Needed</th>
                        <th>Included</th>
                        <th>Invested</th>
                        <th>Invested</th>
                    </tr>';
if(isset($assembly_items)) {
    $displayed = 0;
    while($assembly_item = $assembly_items->fetch_assoc()) {
        if($displayed % 2) {
            print '
                    <tr class="odd_row">';
        } else {
            print '
                    <tr class="even_row">';
        }
        print '
                        <td class="item_description">
                            ' . htmlspecialchars($assembly_item['iDescription']) . '
                        </td>
                        <td class="quantity">
                            ' . $assembly_item['atiQuantity'] . '
                        </td>
                        <td class="quantity">';
        if($assembly_item['aiHasHistory']) {
            print '
                            <a href="transactions.php?aii_assembly_id=' . $keys['assembly_id'] .
            '&aii_item_id=' . $assembly_item['aifaItemID'] . '&aii_person_id=' .
            $settings['assembly_person_filter'] . '">' . $assembly_item['aiQuantity'] . '</a>';
        } else {
            print '
                            ' . $assembly_item['aiQuantity'];
        }
        print '
                        </td>
                        <td class="time">
                            ' . timeize($assembly_item['aiTimeTotalAmount']) . '
                        </td>
                        <td class="money">
                            ' . money_format('%.2n', $assembly_item['aiMonetaryTotalAmount']) . '
                        </td>
                    </tr>';
        if(++$displayed >= ASSEMBLY_ITEM_PAGE_SIZE) {
            break;
        }
    }
}
print '
                </table>
            </div>';
if(isset($selected_assembly)) {
    print '
            <input type="hidden" name="assembly_id" value="' . $keys['assembly_id'] . '" />
            <input type="hidden" name="p_page_number" value="' . $settings['page_number'] . '" />
            <input type="hidden" name="p_item_page_number" value="' . $settings['item_page_number'] . '" />
            <input type="hidden" name="p_aSerialNumber" value="' .
        htmlspecialchars($selected_assembly['aSerialNumber']) . '" />
            <input type="hidden" name="p_aAssemblyTypeID" value="' . $selected_assembly['aAssemblyTypeID'] .
        '" />
            <input type="hidden" name="p_aImageURL" value="' .
        htmlspecialchars($selected_assembly['aImageURL']) . '" />
            <input type="hidden" name="p_aImageLink" value="' .
        htmlspecialchars($selected_assembly['aImageLink']) . '" />';
    if($selected_assembly['aFlags'] & AF_WORK_IN_PROGRESS) {
        print '
            <input type="hidden" name="p_work_in_progress" />';
    }
}
print '
        </form>
    </body>
</html>';

