<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once 'fun/assembly_types.php';
require_once 'fun/common.php';
define("ASSEMBLY_TYPE_PAGE_SIZE", 18);
define("ASSEMBLY_TYPE_ITEM_PAGE_SIZE", 18);
$default_keys = array(
    'assembly_type_id' => 0,
    'assembly_type_item_id' => 0);
$keys = get_settings($default_keys);
$default_settings = array(
    'description_filter' => '',
    'item_description_filter' => '',
    'p_page_number' => 1,
    'p_item_page_number' => 1,
    'new_item_description_filter' => '',
    'p_new_item_description_filter' => '');
$settings = get_settings($default_settings);
$max_items_pages = 0;
if(isset($_SERVER['HTTP_REFERER'])) {
    $referer = parse_url($_SERVER['HTTP_REFERER']);
    if($referer['path'] == '/ShopChart/assembly_types.php' || $referer['path'] == '/ShopChart/items.php') {
        if(isset($_POST['add_assembly_type'])) {
            $keys['assembly_type_id'] = add_assembly_type($_POST['new_assembly_type_description']);
            $keys['assembly_type_item_id'] = 0;
        } elseif(isset($_REQUEST['item_id'])) {
            $keys['assembly_type_item_id'] = $_REQUEST['item_id'];
            if($referer['path'] == '/ShopChart/items.php') {
                add_assembly_type_item($keys['assembly_type_id'], $keys['assembly_type_item_id']);
            }
        }
        if(isset($_REQUEST['assembly_id'])) {
            $keys['assembly_type_item_id'] = 0;
            add_assembly_type_assembly($keys['assembly_type_id'], $_REQUEST['assembly_id']);
        } elseif(isset($_POST['add_assembly_type_item'])) {
            header('Location: items.php?description_filter=' . 
                htmlspecialchars($_POST['new_item_description_filter']) .
                '&sender=assembly_types.php?assembly_type_id=' . $keys['assembly_type_id'] .
                get_params($settings, true));
        }
        if($_SERVER['REQUEST_METHOD'] == 'POST') {
            update_attributes($keys['assembly_type_id'], $keys['assembly_type_item_id'], $_POST);
        }
        $settings['page_number'] = get_page_number('page_number', 'previous_page', 'next_page');
        $settings['item_page_number'] = get_page_number('item_page_number', 'previous_item_page', 'next_item_page');
    }
}
$assembly_types = get_assembly_types($settings['description_filter']);
$max_pages = get_max_pages($assembly_types, ASSEMBLY_TYPE_PAGE_SIZE);
$settings['page_number'] = go_to_page($keys['assembly_type_id'], 'atAssemblyTypeID', $assembly_types,
    ASSEMBLY_TYPE_PAGE_SIZE, $settings['page_number'], $max_pages, ($settings['p_page_number'] ==
    $settings['page_number']));
print '
<!DOCTYPE html>
<html>
    <head>
        <style type="text/css">
            @import url(css/shop_chart.css);
            @import url(css/assembly_types.css);
        </style>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <title>Assembly Types</title>
    </head>
    <body>
        <form action="assembly_types.php" method="post">';
$nav_links = '
                    <div id="selected_nav_link">
                        <a href="assembly_types.php">Assembly Types</a>
                    </div>';
$help_link = 'doc/assembly_types.xhtml';
include 'top_nav.php';
print '
            <div id="top_half">
                <table id="assembly_types">';
if($max_pages > 1) {
    print '
                    <tr class="page_nav">
                        <td>
                            <input type="submit" name="previous_page" value="Previous" />
                            <input type="text" clas="page_number" name="page_number" value="' .
        $settings['page_number'] . '" />
                            of ' . $max_pages . '
                            <input type="submit" name="next_page" value="Next" />
                        </td>
                    </tr>';
}
print '
                    <tr class="field_header">
                        <th>Description</th>
                    </tr>
                    <tr class="field_header">
                        <th><input type="text" name="assembly_type_description_filter" value="' .
    htmlspecialchars($settings['description_filter']) . '" /></th>
                    </tr>';
$displayed = 0;
while ($assembly_type = $assembly_types->fetch_assoc()) {
    if($assembly_type['atAssemblyTypeID'] <> null) {
        if ($assembly_type['atAssemblyTypeID'] == $keys['assembly_type_id']) {
            $selected_assembly_type = $assembly_type;
            $assembly_type_items = get_assembly_type_items($keys['assembly_type_id'],
                $settings['item_description_filter']);
            $max_items_pages = get_max_pages($assembly_type_items, ASSEMBLY_TYPE_ITEM_PAGE_SIZE);
            $settings['item_page_number'] = go_to_page($keys['assembly_type_item_id'], 'atiItemID',
                $assembly_type_items, ASSEMBLY_TYPE_ITEM_PAGE_SIZE, $settings['item_page_number'],
                $max_items_pages, ($settings['p_item_page_number'] == $settings['item_page_number']));
            print '
                    <tr class="selected_row">
                        <td class="description"><input type="text" name="atDescription" value="' .
                htmlspecialchars($assembly_type['atDescription']) . '" /></td>
                    </tr>';
        } else {
            if($displayed % 2) {
                print '
                    <tr class="odd_row">';
            } else {
                print '
                    <tr class="even_row">';
            }
            print '
                        <td class="description"><a href="assembly_types.php?assembly_type_id=' .
                $assembly_type['atAssemblyTypeID'] . get_params($settings, false) . '">' .
                htmlspecialchars($assembly_type['atDescription']) . '</a></td>
                    </tr>';
        }
        if(++$displayed >= ASSEMBLY_TYPE_ITEM_PAGE_SIZE) {
            break;
        }
    }
}
print '
                    <tr class="footer">
                        <td>
                            <input type="text" name="new_assembly_type_description" />
                            <input type="submit" name="add_assembly_type" value="Add Assembly Type" />
                        </td>
                    </tr>
                </table>
            </div>
            <div id="bottom_half">
                <table id="assembly_type_items">';
if($max_items_pages > 1) {
    print '
                    <tr class="page_nav">
                        <td colspan="2">
                            <input type="submit" name="previous_item_page" value="Previous" />
                            <input type="text" name="item_page_number" value="' . $settings['item_page_number'] . '" />
                             of ' . $max_items_pages . '
                            <input type="submit" name="next_item_page" value="Next" />
                        </td>
                    </tr>';
}
print '
                    <tr class="field_header">
                        <th>Description</th>
                        <th>Qty</th>
                    </tr>
                    <tr class="field_header">
                        <th><input type="text" name="item_description_filter" value="' . 
    htmlspecialchars($settings['item_description_filter']) . '" /></th>
                        <th>Needed</th>
                    </tr>';
if(isset($assembly_type_items)) {
    $displayed = 0;
    while($assembly_type_item = $assembly_type_items->fetch_assoc()) {
        if($assembly_type_item['atiItemID'] == $keys['assembly_type_item_id']) {
            $selected_item = $assembly_type_item;
            print '
                <tr class="selected_row">
                    <td class="description">' . htmlspecialchars($assembly_type_item['iDescription']) . '</td>
                    <td><input type="text" name="atiQuantity" value="' . $assembly_type_item['atiQuantity'] .
                '" /></td>
                </tr>';
        } else {
            if($displayed % 2) {
                print '
                <tr class="odd_row">';
            } else {
                print '
                <tr class="even_row">';
            }
            print '
                    <td class="description"><a href="assembly_types.php?assembly_type_id=' .
                $keys['assembly_type_id'] . '&item_id=' . $assembly_type_item['atiItemID'] .
                get_params($settings, false) . '">' . htmlspecialchars($assembly_type_item['iDescription']) .
                '</a></td>
                    <td>' . $assembly_type_item['atiQuantity'] . '</td>
                </tr>';
        }
        if(++$displayed >= ASSEMBLY_TYPE_ITEM_PAGE_SIZE) {
            break;
        }
    }
    print '
                    <tr class="footer">
                        <td colspan="2">
                            <input type="text" name="new_item_description_filter" value="' .
        htmlspecialchars($settings['new_item_description_filter']) . '" />
                            <input type="submit" name="add_assembly_type_item" value="Add Item" />
                        </td>
                    </tr>';
} else {
    print '
                    <tr class="footer">
                        <td colspan="2">
                            <input type="text" disabled="true" />
                            <input type="submit" value="Add Item" disabled="true" />
                        </td>
                    </tr>';
}
print '
                </table>
            </div>';
if(isset($selected_assembly_type)) {
    print '
            <input type="hidden" name="p_page_number" value="' . $settings['page_number'] . '" />
            <input type="hidden" name="p_item_page_number" value="' . $settings['item_page_number'] . '" />
            <input type="hidden" name="assembly_type_id" value="' . $keys['assembly_type_id'] . '" />
            <input type="hidden" name="item_id" value="' . $keys['assembly_type_item_id'] . '" />
            <input type="hidden" name="p_atDescription" value="' .
        htmlspecialchars($selected_assembly_type['atDescription']) . '" />
            <input type="hidden" name="p_new_item_description_filter" value="' .
        htmlspecialchars($settings['new_item_description_filter']) . '" />';
    if(isset($selected_item)) {
        print '
            <input type="hidden" name="p_atiQuantity" value="' . $selected_item['atiQuantity'] . '" />';
    }
}
print '
        </form>
    </body>
</html>';

