<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once('mysqli.php');
require_once('util/transaction_type_flags.php');
require_once('sql/assemblies.php');

function add_assembly($assembly_type_id, $serial_number) {
    if(!isset($GLOBALS['administrator'])) return;
    global $mysqli;
    $mysqli->query("
INSERT INTO Assemblies (
    aAssemblyTypeID,
    aSerialNumber
) VALUES (
    $assembly_type_id,
    '" . $mysqli->real_escape_string($serial_number) . "'
)");
    $assembly_id = $mysqli->insert_id;
    if($assembly_id == 0) {
        $assembly_id = $mysqli->query("
SELECT
    aAssemblyID
FROM
    Assemblies
WHERE
    aSerialNumber = '" . $mysqli->real_escape_string($serial_number) . "'");
        if($assembly_id = $assembly_id->fetch_assoc()) {
            return $assembly_id['aAssemblyID'];
        } else {
            return 0;
        }
    } else {
        return $assembly_id;
    }
}

function get_assembly_attributes($attributes) {
    global $mysqli;
    $set = '';
    if(isset($attributes['p_aAssemblyTypeID']) && $attributes['p_aAssemblyTypeID'] <>
        $attributes['aAssemblyTypeID']) {
        $set .= "
    aAssemblyTypeID = " . $attributes['aAssemblyTypeID'] . ",";
    }
    if(isset($attributes['p_aSerialNumber']) && $attributes['p_aSerialNumber'] <>
        $attributes['aSerialNumber']) {
        $set .= "
    aSerialNumber = '" . $mysqli->real_escape_string($attributes['aSerialNumber']) . "',";
    }
    if(isset($attributes['p_aImageURL']) && $attributes['p_aImageURL'] <> $attributes['aImageURL']) {
        $set .= "
    aImageURL = '" . $mysqli->real_escape_string($attributes['aImageURL']) . "',";
    }
    if(isset($attributes['p_aImageLink']) && $attributes['p_aImageLink'] <> $attributes['aImageLink']) {
        $set .= "
    aImageLink = '" . $mysqli->real_escape_string($attributes['aImageLink']) . "',";
    }
    if(isset($attributes['p_work_in_progress']) <> isset($attributes['work_in_progress'])) {
        if(isset($attributes['work_in_progress'])) {
            $set .= "
    aFlags = aFlags | " . AF_WORK_IN_PROGRESS . ",";
        } else {
            $set .= "
    aFlags = aFlags ^ " . AF_WORK_IN_PROGRESS . ",";
        }
    }
    return $set;
}

function update_attributes($assembly_id, $attributes) {
    if(!isset($GLOBALS['administrator'])) return;
    global $mysqli;
    $set = get_assembly_attributes($attributes);
    if($set <> '') {
        $mysqli->query("
UPDATE
    Assemblies
SET" . substr($set, 0, strlen($set) - 1) . "
WHERE
    aAssemblyID = $assembly_id");
    }
}

function get_assembly_types($active_only) {
    global $mysqli;
    if($active_only) {
        $join = "INNER JOIN
        Assemblies
    ON atAssemblyTypeID = aAssemblyTypeID";
    } else {
        $join = "";
    }
    $assembly_types = $mysqli->query("
SELECT
    atAssemblyTypeID,
    atDescription
FROM
    AssemblyTypes $join");
    $response = array();
    while($assembly_type = $assembly_types->fetch_assoc()) {
        $response[$assembly_type['atAssemblyTypeID']] = $assembly_type['atDescription'];
    }
    return $response;
}

function get_people() {
    global $mysqli;
    $people = $mysqli->query("
SELECT DISTINCT
    pPersonID,
    pFirstName,
    pLastName
FROM
    (SELECT
        tToPersonID ttlPersonID,
        tlAssemblyID ttlAssemblyID,
        SUM(tlQuantity) ttlQuantity
    FROM
        TransactionLines INNER JOIN (
            Transactions INNER JOIN
                TransactionTypes
            ON tTransactionTypeID = ttTransactionTypeID
        ) ON tlTransactionID = tTransactionID
    WHERE
        ttFlags & " . TTF_TRANSFERS_INVENTORY . "
    GROUP BY
        tToPersonID,
        tlAssemblyID
    ) ttl RIGHT JOIN (
        (SELECT
            tFromPersonID ftlPersonID,
            tlAssemblyID ftlAssemblyID,
            SUM(tlQuantity) ftlQuantity
        FROM
            TransactionLines INNER JOIN (
                Transactions INNER JOIN
                    TransactionTypes
                ON tTransactionTypeID = ttTransactionTypeID
            ) ON tlTransactionID = tTransactionID
        WHERE
            ttFlags & " . TTF_TRANSFERS_INVENTORY . "
        GROUP BY
            tFromPersonID,
            tlAssemblyID
        ) ftl RIGHT JOIN
            People
        ON ftlPersonID = pPersonID
    ) ON ttlPersonID = pPersonID
WHERE
    COALESCE(ttlQuantity, 0) - COALESCE(ftlQuantity, 0) > 0
ORDER BY
    pLastName");
    $response = array();
    while($person = $people->fetch_assoc()) {
        $response[$person['pPersonID']] = ltrim($person['pFirstName'] . ' ' . $person['pLastName']);
    }
    return $response;
}

function get_assemblies($serial_number, $assembly_type_id, $person_id, $user) {
    global $mysqli;
    if($serial_number == '') {
        $where = "";
    } else {
        $where = " AND aSerialNumber LIKE '%" . $mysqli->real_escape_string($serial_number) . "%'";
    }
    if($assembly_type_id <> 0) {
        $where = $where . " AND aAssemblyTypeID = $assembly_type_id";
    }
    if($person_id == -1) {
        if(isset($_REQUEST['iqn_item_id'])) {
            make_eligible_assemblies($_REQUEST['iqn_item_id'], $user);
            $join = "
            shopcharttmp.EligibleAssemblies INNER JOIN";
            $on = "
            ON eaAssemblyID = aAssemblyID";
        } else {
            $join = "";
            $on = "";
        }
        $sql = "
SELECT
    0 appPersonID,
    aAssemblyID,
    aAssemblyTypeID,
    aSerialNumber,
    aImageLink,
    aImageURL,
    aFlags
FROM $join
    Assemblies $on
WHERE
    NOT EXISTS (
        SELECT
            *
        FROM
            TransactionLines
        WHERE
            tlAssemblyID = aAssemblyID
    )
    $where
ORDER BY
    aSerialNumber";
    } elseif($person_id == 0) {
        make_assembly_best_quantity('');
        if(isset($_REQUEST['iqn_item_id'])) {
            make_eligible_assemblies($_REQUEST['iqn_item_id'], $user);
            $join = "
            shopcharttmp.EligibleAssemblies INNER JOIN (";
            $on = "
            ) ON eaAssemblyID = aAssemblyID";
        } else {
            $join = "";
            $on = "";
        }
        $sql = "
SELECT
    appPersonID,
    aAssemblyID,
    aAssemblyTypeID,
    aSerialNumber,
    aImageLink,
    aImageURL,
    aFlags
FROM $join
    (SELECT
        pqAssemblyID appAssemblyID,
        MAX(pqPersonID) appPersonID
    FROM
        shopcharttmp.AssemblyBestQuantity INNER JOIN
            shopcharttmp.PersonQuantity
        ON abqAssemblyID = pqAssemblyID
        AND abqQuantity = pqQuantity
    GROUP BY
        pqAssemblyID
    ) app  RIGHT JOIN
        Assemblies
    ON appAssemblyID = aAssemblyID $on
" . ($where == "" ? "" : "WHERE
    " . substr($where, 4)) . "
ORDER BY
    aSerialNumber";
    } else {
        make_assembly_best_quantity('');
        make_person_assemblies($person_id, '');
        
        if(isset($_REQUEST['iqn_item_id'])) {
            make_eligible_assemblies($_REQUEST['iqn_item_id'], $user);
            $join = "
            shopcharttmp.EligibleAssemblies INNER JOIN (";
            $on = "
            ) ON eaAssemblyID = aAssemblyID";
        } else {
            $join = "";
            $on = "";
        }
        $sql = "
SELECT
    appPersonID,
    aAssemblyID,
    aAssemblyTypeID,
    aSerialNumber,
    aImageLink,
    aImageURL,
    aFlags
FROM $join
    (SELECT
        pqAssemblyID appAssemblyID,
        MAX(pqPersonID) appPersonID
    FROM
        shopcharttmp.AssemblyBestQuantity INNER JOIN
            shopcharttmp.PersonQuantity
        ON abqAssemblyID = pqAssemblyID
        AND abqQuantity = pqQuantity
    GROUP BY
        pqAssemblyID
    ) app  RIGHT JOIN (
        shopcharttmp.PersonAssemblies INNER JOIN
            Assemblies
        ON paAssemblyID = aAssemblyID
    ) ON appAssemblyID = aAssemblyID $on
" . ($where == "" ? "" : "WHERE
    " . substr($where, 4)) . "
ORDER BY
    aSerialNumber";
    }
    $response = $mysqli->query($sql);
    return $response;
}

function get_assembly_items($assembly_id, $person_id, $description, &$totals) {
    global $mysqli;
    if($description <> '') {
        $description = "WHERE
    iDescription LIKE '%" . $mysqli->real_escape_string($description) . "%'";
    }
    make_assembly_item_from_amount($assembly_id, $person_id, 'gai');
    $summary = $mysqli->query("
SELECT
    SUM(aifaMonetaryTotalAmount) MonetaryTotalAmount,
    SUM(aifaTimeTotalAmount) TimeTotalAmount,
    SUM(iWeight * aifaQuantity) TotalWeight
FROM
    shopcharttmp.gaiAssemblyItemFromAmount INNER JOIN
        Items
    ON aifaItemID = iItemID");
    $totals = $summary->fetch_assoc();
    $totals['RevenueTotalAmount'] = 0;
    return $mysqli->query("
SELECT
    aifaItemID,
    iDescription,
    MAX(QuantityNeeded) atiQuantity,
    MAX(COALESCE(aifaQuantity, 0)) aiQuantity,
    MAX(aifaMonetaryTotalAmount) aiMonetaryTotalAmount,
    MAX(aifaTimeTotalAmount) aiTimeTotalAmount,
    MAX(aifaHasHistory) aiHasHistory
FROM
    (SELECT
        aifaItemID,
        0 QuantityNeeded,
        aifaQuantity,
        aifaMonetaryTotalAmount,
        aifaTimeTotalAmount,
        aifaHasHistory
    FROM
        shopcharttmp.gaiAssemblyItemFromAmount
    UNION
    SELECT
        atiItemID,
        atiQuantity,
        NULL,
        NULL,
        NULL,
        NULL
    FROM
        Assemblies INNER JOIN
            AssemblyTypeItems
        ON aAssemblyTypeID = atiAssemblyTypeID
    WHERE
        aAssemblyID = $assembly_id
        AND aFlags & " . AF_WORK_IN_PROGRESS . "
    ) ai INNER JOIN
        Items
    ON aifaItemID = iItemID
$description
GROUP BY
    aifaItemID,
    iDescription
ORDER BY
    iDescription");
}

function get_assembly_people($assembly_id = 0) {
    global $mysqli;
    if($assembly_id == 0) {
        $assembly_id = '';
    } else {
        $assembly_id = "
    WHERE
        qiAssemblyID = $assembly_id";
    }
    $mysqli->query("
SET
    SQL_BIG_SELECTS = 1");
    $assembly_people = "
SELECT
    pPersonID,
    RTRIM(CONCAT(pFirstName, ' ', pLastName)) pFullName
FROM
    (SELECT
        tFromPersonID qoPersonID,
        tlAssemblyID qoAssemblyID,
        tlItemID qoItemID,
        SUM(tlQuantity) qoQuantity
    FROM
        TransactionLines INNER JOIN (
            Transactions INNER JOIN
                TransactionTypes
            ON tTransactionTypeID = ttTransactionTypeID
        ) ON tlTransactionID = tTransactionID
    WHERE
        ttFlags & " . TTF_TRANSFERS_INVENTORY . "
    GROUP BY
        tFromPersonID,
        tlAssemblyID,
        tlItemID
    ) qo RIGHT JOIN (
        (SELECT
            tToPersonID qiPersonID,
            tlAssemblyID qiAssemblyID,
            tlItemID qiItemID,
            SUM(tlQuantity) qiQuantity
        FROM
            TransactionLines INNER JOIN (
                Transactions INNER JOIN
                    TransactionTypes
                ON tTransactionTypeID = ttTransactionTypeID
            ) ON tlTransactionID = tTransactionID
        WHERE
            ttFlags & " . TTF_TRANSFERS_INVENTORY . "
        GROUP BY
            tToPersonID,
            tlAssemblyID,
            tlItemID
        ) qi INNER JOIN
            People
        ON qiPersonID = pPersonID
    ) ON qoPersonID = qiPersonID
    AND qoAssemblyID = qiAssemblyID
    AND qoItemID = qiItemID $assembly_id
GROUP BY
    pPersonID,
    RTRIM(CONCAT(pFirstName, ' ', pLastName))
ORDER BY
    qiQuantity - COALESCE(qoQuantity, 0) DESC,
    CONCAT(pLastName, pFirstName)";
    $assembly_people = $mysqli->query($assembly_people);
    $response = array();
    while($assembly_person = $assembly_people->fetch_assoc()) {
        $response[$assembly_person['pPersonID']] = $assembly_person['pFullName'];
    }
    return $response;
}

function iqn_notice($settings, $keys) {
    $item_description = get_item_description($settings['iqn_item_id']);
    return('
            <div class="special_criteria">Only showing Assemblies used to calculate the Qty Needed of the ' .
        htmlspecialchars($item_description) . '. <a href="assemblies.php?assembly_id=' .
        $keys['assembly_id'] . get_params(remove_settings($settings, 'iqn_item_id'), false) .
        '">Remove</a>
            <input type="hidden" name="iqn_item_id" value="' . $settings['iqn_item_id'] . '" /></div>');
}

function get_item_description($item_id) {
    global $mysqli;
    $item = $mysqli->query("
SELECT
    iDescription
FROM
    Items
WHERE
    iItemID = $item_id");
    $item = $item->fetch_assoc();
    return $item['iDescription'];
}