<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once 'mysqli.php';


function change_sort_field($sort_order, $field_number) {
    $direction = 'A';
    if(strpos($sort_order, (string)$field_number) !== false) {
        $new_sort_order = '';
        while(strlen($sort_order) > 1) {
            if(substr($sort_order, 0, 1) == (string)$field_number) {
                if(substr($sort_order, 1, 1) == 'A') {
                    $direction = 'D';
                }
            } else {
                $new_sort_order .= substr($sort_order, 0, 2);
            }
            if(strlen($sort_order) > 2) {
                $sort_order = substr($sort_order, 2);
            } else {
                $sort_order = '';
            }
        }
    } else {
        $new_sort_order = $sort_order;
    }
    return $field_number . $direction . $new_sort_order;
}

function get_stored_settings() {
    global $mysqli;
    $response = array();
    $stored_settings = $mysqli->query("
SELECT
    setName,
    setValue
FROM
    Settings");
    while($stored_setting = $stored_settings->fetch_assoc()) {
        $response[$stored_setting['setName']] = $stored_setting['setValue'];
    }
    return $response;
}

function remove_settings($settings) {
    $params = $settings;
    $remove_count = func_num_args();
    for($remove = 1; $remove < $remove_count; $remove++) {
        unset($params[func_get_arg($remove)]);
    }
    return $params;
}

function timeize($seconds) {
    $hours = floor($seconds / 3600);
    $mins = floor(($seconds - ($hours*3600)) / 60);
    $secs = floor($seconds % 60);
    return str_pad($hours, 2, '0', STR_PAD_LEFT) . ':' . str_pad($mins, 2, '0',
        STR_PAD_LEFT) . ':' . str_pad($secs, 2, '0', STR_PAD_LEFT);
}

function validate_page_number($page_number, $max_pages) {
    if($page_number == -1) {
        $page_number = $max_pages;
    } elseif($page_number > $max_pages) {
        $page_number = 1;
    }
    return $page_number;
}

function get_page_number($page_number_name, $previous_page_name, $next_page_name) {
    $page_number = 1;
    if(isset($_REQUEST[$page_number_name])) {
        if(is_numeric($_REQUEST[$page_number_name])) {
            if($_REQUEST[$page_number_name] > 0) {
                $page_number = $_REQUEST[$page_number_name];
            }
        }
    }
    if(isset($_POST[$previous_page_name])) {
        if($page_number > 1) {
            $page_number--;
        } else {
            $page_number = -1;
        }
    } elseif(isset($_POST[$next_page_name])) {
        $page_number++;
    }
    return $page_number;
}

function get_settings($settings) {
    $new_settings = get_stored_settings();
    if(!isset($new_settings['user'])) {
        $new_settings['user'] = 0;
    }
    foreach($settings as $name => $value) {
        if(isset($_REQUEST[$name])) {
            $new_settings[$name] = $_REQUEST[$name];
        } else {
            if(!is_null($settings[$name])) {
                $new_settings[$name] = $settings[$name];
            }
        }
    }
    return $new_settings;
}

function get_params($settings, $html_safe) {
    $params = '';
    foreach($settings as $name => $value) {
        if($html_safe) {
            $params .= '%26' . $name . '=' . htmlspecialchars($value);
        } else {
            $params .= '&' . $name . '=' . htmlspecialchars($value);
        }
    }
    return $params;
}