<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once('mysqli.php');
function get_transaction_stats($transaction_id) {
    global $mysqli;
    $stats = $mysqli->query("
SELECT
    tDate,
    RTRIM(CONCAT(fp.pFirstName, ' ', fp.pLastName)) FromPerson,
    RTRIM(CONCAT(tp.pFirstName, ' ', tp.pLastName)) ToPerson,
    ttDescription,
    COALESCE(lcLineItems, 0) LineItems
FROM
    (SELECT
        tlTransactionID lcTransactionID,
        COUNT(*) lcLineItems
    FROM
        TransactionLines
    WHERE
        tlTransactionID = $transaction_id
    GROUP BY
        tlTransactionID
    ) lc RIGHT JOIN (
        TransactionTypes INNER JOIN (
            People fp INNER JOIN (
                People tp INNER JOIN
                    Transactions
                ON tp.pPersonID = tToPersonID
            ) ON fp.pPersonID = tFromPersonID
        ) ON ttTransactionTypeID = tTransactionTypeID
    ) ON lcTransactionID = tTransactionID
WHERE
    tTransactionID = $transaction_id");
    if($stats = $stats->fetch_assoc()) {
        return 'Deleting a ' . $stats['ttDescription'] . ' Transaction from ' . $stats['tDate'] . ' between ' .
            htmlspecialchars($stats['FromPerson']) . ' and ' . htmlspecialchars($stats['ToPerson']) .
            '.<br />This transaction has ' . $stats['LineItems'] . ($stats['LineItems'] == 1 ? ' Line Item.' :
            ' Line Items.');
    }
}

function get_person_stats($person_id) {
    global $mysqli;
    $stats = $mysqli->query("
SELECT
    RTRIM(CONCAT(pFirstName, ' ', pLastName)) FullName,
    COALESCE(tcFromTransactions, 0) FromTransactions,
    COALESCE(tcToTransactions, 0) ToTransactions
FROM
    (SELECT
        $person_id tcPersonID,
        SUM(tFromPersonID = $person_id) tcFromTransactions,
        SUM(tToPersonID = $person_id) tcToTransactions
    FROM
        Transactions
    WHERE
        tFromPersonID = $person_id
        OR tToPersonID = $person_id
    ) tc RIGHT JOIN
        People
    ON tcPersonID = pPersonID
WHERE
    pPersonID = $person_id");
    if($stats = $stats->fetch_assoc()) {
        $full_name = htmlspecialchars($stats['FullName']);
        $person_stats = '
            ' . $full_name . ' was the From Person for ' . $stats['FromTransactions'] . ' transactions.<br />';
        if($stats['FromTransactions'] + $stats['ToTransactions'] > 0) {
            $people = get_people($person_id);
            $person_options = '';
            foreach($people as $id => $name) {
                $person_options .= '
                <option value="' . $id . '">' . htmlspecialchars($name) . '</option>';
            }
            if($stats['FromTransactions'] > 0) {
                $person_stats .= '
            Please choose a person to reassign as the From Person <select name="new_from_person_id">' .
                    $person_options . '
            </select><br />';
            }
        }
        $person_stats .= '
            ' . $full_name . ' was the To Person for ' . $stats['ToTransactions'] . ' transactions.<br />';
        if($stats['ToTransactions'] > 0) {
            $person_stats .= '
            Please choose a person to reassign as the To Person <select name="new_to_person_id">' .
                    $person_options . '
            </select><br />';
        }
        return $person_stats;
    }
}

function get_people($exclude_person_id) {
    global $mysqli;
    $people = $mysqli->query("
SELECT
    pPersonID,
    RTRIM(CONCAT(pFirstName, ' ', pLastName)) FullName
FROM
    People
WHERE
    pPersonID <> $exclude_person_id");
    $response = array();
    while($person = $people->fetch_assoc()) {
        $response[$person['pPersonID']] = $person['FullName'];
    }
    return $response;
}