<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once('mysqli.php');
require_once('util/transaction_type_flags.php');
require_once('sql/items.php');

function add_item($description) {
    if(!isset($GLOBALS['administrator'])) return;
    global $mysqli;
    $mysqli->query("
INSERT INTO Items (
    iDescription
) VALUES (
    '" . $mysqli->real_escape_string($description) . "'
)");
    $item_id = $mysqli->insert_id;
    if($item_id == 0) {
        $item_id = $mysqli->query("
SELECT
    iItemID
FROM
    Items
WHERE
    iDescription = '" . $mysqli->real_escape_string($description) . "'");
        if($item_id = $item_id->fetch_assoc()) {
            return $item_id['iItemID'];
        } else {
            return 0;
        }
    } else {
        return $item_id;
    }
}

function get_transfer_info($item_id, $to_person_id, $transaction_type_id) {
    global $mysqli;
    if($to_person_id == 0) {
        $where = '';
    } else {
        $where = "
    AND tToPersonID = $to_person_id";
    }
    if($transaction_type_id <> 0) {
        $where .= "
    AND tTransactionTypeID = $transaction_type_id";
    }
    $mysqli->query("
SET
    SQL_BIG_SELECTS = 1");
    make_item_percent($item_id, $where);
    return $mysqli->query("
SELECT
    htcTransactionID,
    TRIM(CONCAT(pFirstName, ' ', pLastName)) htcFromPersonName,
    htcTransactionTypeID,
    COALESCE(giUnitAmount, htcUnitAmount) tlUnitAmount,
    htcFromURL,
    htcDate
FROM
    (SELECT
        ipTransactionID giTransactionID,
        ipFromPersonID giFromPersonID,
        ipTransactionTypeID giTransactionTypeID,
        MAX(ipPercent * tlgTotalAmount) giUnitAmount
    FROM
        shopcharttmp.ItemPercent INNER JOIN
            TransactionLineGroups
        ON ipTransactionID = tlgTransactionID
        AND ipLineGroupID = tlgLineGroupID
    GROUP BY
        ipTransactionID,
        ipFromPersonID,
        ipTransactionTypeID
    ) gi RIGHT JOIN (
        People INNER JOIN
            (SELECT
                ltTransactionID htcTransactionID,
                ltDate htcDate,
                tlItemID htcItemID,
                tlFromURL htcFromURL,
                ltFromPersonID htcFromPersonID,
                ltTransactionTypeID htcTransactionTypeID,
                MAX(tlUnitAmount) htcUnitAmount
            FROM
                TransactionLines INNER JOIN
                    shopcharttmp.LastTransaction
                ON tlTransactionID = ltTransactionID
            WHERE
                tlItemID = $item_id
            GROUP BY
                ltTransactionID,
                ltDate,
                tlItemID,
                tlFromURL
            ) htc
        ON pPersonID = htcFromPersonID
    ) ON giTransactionID = htcTransactionID
ORDER BY
    htcDate DESC");
}

function get_transaction_types() {
    global $mysqli;
    $transaction_types = $mysqli->query("
SELECT
    ttTransactionTypeID,
    ttDescription
FROM
    TransactionTypes
WHERE
    ttFlags & " . TTF_TRANSFERS_INVENTORY);
    $response = array();
    while($transaction_type = $transaction_types->fetch_assoc()) {
        $response[$transaction_type['ttTransactionTypeID']] = $transaction_type['ttDescription'];
    }
    return $response;
}

function get_item_attributes($attributes) {
    global $mysqli;
    $set = '';
    if(isset($attributes['p_iDescription']) && $attributes['p_iDescription'] <> $attributes['iDescription']) {
        $set .= "
    iDescription = '" . $mysqli->real_escape_string($attributes['iDescription']) . "',";
    }
    if(isset($attributes['p_iImageURL']) && $attributes['p_iImageURL'] <> $attributes['iImageURL']) {
        $set .= "
    iImageURL = '" . $mysqli->real_escape_string($attributes['iImageURL']) . "',";
    }
    if(isset($attributes['p_iImageLink']) && $attributes['p_iImageLink'] <> $attributes['iImageLink']) {
        $set .= "
    iImageLink = '" . $mysqli->real_escape_string($attributes['iImageLink']) . "',";
    }
    if(isset($attributes['p_iWeight']) && $attributes['p_iWeight'] <> $attributes['iWeight']) {
        $set .= "
    iWeight = " . $mysqli->real_escape_string($attributes['iWeight']) . ",";
    }
    if(isset($attributes['p_iInformation']) && $attributes['p_iInformation'] <> $attributes['iInformation']) {
        $set .= "
    iInformation = '" . $mysqli->real_escape_string($attributes['iInformation']) . "',";
    }
    return $set;
}

function update_attributes($item_id, $attributes) {
    if(!isset($GLOBALS['administrator'])) return;
    global $mysqli;
    $set = get_item_attributes($attributes);
    if($set <> '') {
        $mysqli->query("
UPDATE
    Items
SET" . substr($set, 0, strlen($set) - 1) . "
WHERE
    iItemID = $item_id");
    }
}

function get_purchasers($item_id) {
    global $mysqli;
    $people = $mysqli->query("
SELECT DISTINCT
    pPersonID,
    pFirstName,
    pLastName
FROM
    People INNER JOIN (
        TransactionLines INNER JOIN (
            Transactions INNER JOIN
                TransactionTypes
            ON tTransactionTypeID = ttTransactionTypeID
        ) ON tlTransactionID = tTransactionID
    ) ON pPersonID = tToPersonID
WHERE
    ttFlags & " . TTF_TRANSFERS_INVENTORY . "
    AND tlItemID = $item_id");
    $response = array();
    while($person = $people->fetch_assoc()) {
        $response[$person['pPersonID']] = ltrim($person['pFirstName'] . ' ' . $person['pLastName']);
    }
    return $response;
}

function get_items($filter, $person_id, $special_criteria, $sort_order) {
    global $mysqli;
    make_qty_needed($person_id);
    $where = '';
    if($filter <> '') {
        $where .= "
    iDescription LIKE '%" . $mysqli->real_escape_string($filter) . "%' AND";
    }
    if(isset($special_criteria['models'])) {
        $where .= "
    COALESCE(iImageLink, '') <> '' AND";
    }
    if($where <> '') {
        $where = "WHERE" . substr($where, 0, strlen($where) - 4);
    }
    if(strlen($sort_order) > 1) {
        $order_by = "ORDER BY";
        while(strlen($sort_order) > 1) {
            switch(substr($sort_order, 0, 1)) {
                case '1':
                    $order_by .= "
        iDescription";
                    break;
                case '2':
                    $order_by .= "
        COALESCE(qnQuantity, 0)";
                    break;
                case '3':
                    $order_by .= "
        COALESCE(qohQtyIn - qohQtyOut, 0)";
                    break;
                case '4':
                    $order_by .= "
        CASE
            WHEN COALESCE(qnQuantity, 0) > COALESCE(qohQtyIn - qohQtyOut, 0) THEN
                COALESCE(qnQuantity, 0) - COALESCE(qohQtyIn - qohQtyOut, 0)
            ELSE
                0
        END";
            }
            if(substr($sort_order, 1, 1) == 'D') {
                $order_by .= " DESC,";
            } else {
                $order_by .= ",";
            }
            if(strlen($sort_order) > 2) {
                $sort_order = substr($sort_order, 2);
            } else {
                $sort_order = '';
            }
        }
        $order_by = substr($order_by, 0, strlen($order_by) - 1);
    } else {
        $order_by = "";
    }
        return $mysqli->query("
SELECT
    iItemID,
    iDescription,
    iImageURL,
    iImageLink,
    iInformation,
    iWeight,
    COALESCE(qnQuantity, 0) iQuantityNeeded,
    COALESCE(qohQtyIn - qohQtyOut, 0) iQuantityOnHand,
    qohItemID IS NOT NULL iHasHistory,
    CASE
        WHEN COALESCE(qnQuantity, 0) > COALESCE(qohQtyIn - qohQtyOut, 0) THEN
            COALESCE(qnQuantity, 0) - COALESCE(qohQtyIn - qohQtyOut, 0)
        ELSE
            0
    END iQuantityToOrder
FROM
    (SELECT
        tlItemID qohItemID,
        SUM(CASE WHEN
            tFromPersonID = $person_id
            AND (
                (
                    tlAssemblyID IS NULL
                    AND (
                        (
                            ttFlags & " . TTF_TRANSFERS_INVENTORY . "
                            AND tToPersonID <> $person_id
                        ) OR ttFlags & " . TTF_REMOVES_INVENTORY . "
                    )
                ) OR (
                    ttFlags & " . TTF_ADDS_INVENTORY_TO_ASSEMBLY . "
                    AND tlAssemblyID IS NOT NULL
                )
            ) THEN tlQuantity
        ELSE
            0
        END) qohQtyOut,
        SUM(CASE WHEN
            tToPersonID = $person_id
            AND (
                (
                    tlAssemblyID IS NULL
                    AND ttFlags & " . TTF_TRANSFERS_INVENTORY . "
                    AND tFromPersonID <> $person_id
                ) OR (
                    ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
                    AND tlAssemblyID IS NOT NULL
                )
            ) THEN tlQuantity
            ELSE 
                0
        END) qohQtyIn
    FROM
        TransactionLines INNER JOIN (
            Transactions INNER JOIN
                TransactionTypes
            ON tTransactionTypeID = ttTransactionTypeID
        ) ON tlTransactionID = tTransactionID
    WHERE
        tDate <= NOW()
        AND (
            (
                tlAssemblyID IS NULL
                AND (
                    tFromPersonID = $person_id
                    OR tToPersonID = $person_id
                ) AND (
                    ttFlags & " . TTF_REMOVES_INVENTORY . "
                    OR (
                        ttFlags & " . TTF_TRANSFERS_INVENTORY . "
                        AND tToPersonID <> tFromPersonID
                    )
                )
            ) OR (
                tlAssemblyID IS NOT NULL
                AND (
                    (
                        ttFlags & " . TTF_ADDS_INVENTORY_TO_ASSEMBLY . "
                        AND tFromPersonID = $person_id
                    ) OR (
                        ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
                        AND tToPersonID = $person_id
                    )
                )
            )
        )
    GROUP BY
        tlItemID
    ) qoh RIGHT JOIN (
        shopcharttmp.QtyNeeded RIGHT JOIN
            Items
        ON qnItemID = iItemID
    ) ON qohItemID = iItemID
$where
$order_by");
}

function get_assemblies() {
    global $mysqli;
    $assemblies = $mysqli->query("
SELECT
    aAssemblyID,
    aSerialNumber
FROM
    Assemblies
ORDER BY
    aSerialNumber");
    $response = array();
    while($assembly = $assemblies->fetch_assoc()) {
        $response[$assembly['aAssemblyID']] = $assembly['aSerialNumber'];
    }
    return $response;
}