<?php
require_once('util/transaction_type_flags.php');

function make_responsible_transactions($transaction_id, $line_id, $prefix) {
    global $mysqli;
    make_current_transaction_items($transaction_id, $line_id, $prefix);
    make_out_transactions($prefix);
    make_reference_transactions($prefix);
    make_past_transactions($prefix);
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtAssemblyID int,
	rtCurrentFlags int,
	rtDate datetime,
	rtFlags int,
	rtFromPersonID int,
	rtItemID int,
	rtLineGroupID int,
	rtQtyUsed decimal(11, 2),
	rtTransactionID int,
	rtTransactionTypeID int,
	PRIMARY KEY (
		rtTransactionID,
		rtItemID,
		rtLineGroupID,
		rtAssemblyID
	)
)");
    add_assy_transactions($prefix);
    make_non_assy_items($transaction_id, $line_id, $prefix);
    add_non_assy_transactions($prefix);
    make_salvage_items($prefix);
    make_salvage_out_transactions($prefix);
    make_salvage_reference_transactions($prefix);
    make_salvage_past_transactions($prefix);
    add_salvage_transactions($prefix);
}
    

function make_transaction_line_from_amount($transaction_id, $prefix) {
    global $mysqli;
    make_responsible_transactions($transaction_id, 0, $prefix);
    make_past_cost($prefix);
    make_item_average_cost($prefix);
    make_total_unit_amount($prefix);
    make_item_percent($prefix);
    make_grouped_items($prefix);
    make_transaction_item_cost($prefix);
    make_directly_responsible($prefix);
    make_general_invy_cost($prefix);
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "TransactionLineFromAmount (
    tlfaHasUnitAmount bit,
	tlfaLineID int PRIMARY KEY,
	tlfaUnitAmount decimal(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TransactionLineFromAmount (
	tlfaLineID,
    tlfaHasUnitAmount,
	tlfaUnitAmount
) SELECT
	tlLineID,
    drItemID IS NULL AND gicItemID IS NULL,
	((ctiQuantity - COALESCE(drQuantity, 0)) * COALESCE(gicUnitAmount, 0) + COALESCE(drTotalAmount, 0)) / ctiQuantity
FROM
	shopcharttmp." . $prefix . "DirectlyResponsible RIGHT JOIN (
		shopcharttmp." . $prefix . "GeneralInvyCost RIGHT JOIN (
			shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN
				TransactionLines
			ON ctiTransactionID = tlTransactionID
			AND ctiItemID = tlItemID
		) ON gicItemID = ctiItemID
	) ON drItemID = ctiItemID");
}

function make_current_transaction_items($transaction_id, $line_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "CurrentTransactionItems (
	ctiAssemblyID int,
	ctiDate datetime,
	ctiFlags int,
	ctiFromPersonID int,
	ctiItemID int,
	ctiQuantity decimal(11, 2),
	ctiTransactionID int,
	PRIMARY KEY (
		ctiTransactionID,
		ctiItemID,
		ctiAssemblyID
	)
)");
    if($line_id == 0) {
        $line_id = "";
    } else {
        $line_id = "
    AND tlLineID = $line_id";
    }
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "CurrentTransactionItems (
	ctiTransactionID,
	ctiDate,
	ctiFromPersonID,
	ctiFlags,
	ctiItemID,
	ctiAssemblyID,
	ctiQuantity
) SELECT
	tTransactionID,
	tDate,
	tFromPersonID,
	ttFlags,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	TransactionLines INNER JOIN (
		Transactions INNER JOIN
			TransactionTypes
		ON tTransactionTypeID = ttTransactionTypeID
	) ON tlTransactionID = tTransactionID
WHERE
	tTransactionID = $transaction_id $line_id
GROUP BY
	tTransactionID,
	tDate,
	tFromPersonID,
	ttFlags,
	tlItemID,
	tlAssemblyID");
}

function make_out_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "OutTransactions (
	otAssemblyID int,
	otDate datetime,
	otItemID int,
	otQuantity decimal(11, 2),
	otTransactionID int,
	PRIMARY KEY (
		otTransactionID,
		otItemID,
		otAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "OutTransactions (
	otTransactionID,
	otDate,
	otItemID,
	otAssemblyID,
	otQuantity
) SELECT
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON ctiFromPersonID = tFromPersonID
	AND ctiItemID = tlItemID
	AND ctiAssemblyID = tlAssemblyID
WHERE
	(
		tDate < ctiDate
		OR (
			tDate = ctiDate
			AND tTransactionID < ctiTransactionID
		)
	) AND (
		(
			ttFlags & " . TTF_TRANSFERS_INVENTORY . "
			AND tFromPersonID <> tToPersonID
		) OR ttFlags & " . (TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY) . "
	)
GROUP BY
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID");
}

function make_reference_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "ReferenceTransactions (
	rtDate datetime,
	rtTransactionID int PRIMARY KEY,
	INDEX rtiDate (
		rtDate
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ReferenceTransactions (
	rtTransactionID,
	rtDate
) SELECT DISTINCT
	tTransactionID,
	tDate
FROM
	shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN (
		TransactionLines INNER JOIN
			Transactions
		ON tlTransactionID = tTransactionID
	) ON ctiItemID = tlItemID
	AND ctiAssemblyID = tlAssemblyID
WHERE
	tFromPersonID = ctiFromPersonID
	OR tToPersonID = ctiFromPersonID");
}

function make_past_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "PastTransactions (
	ptAssemblyID int,
	ptItemID int,
	ptPersonID int,
	ptQuantity decimal(11, 2),
	ptTransactionID int,
	PRIMARY KEY (
		ptTransactionID,
		ptPersonID,
		ptItemID,
		ptAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PastTransactions (
	ptTransactionID,
	ptPersonID,
	ptItemID,
	ptAssemblyID,
	ptQuantity
) SELECT
	rtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID,
	SUM(
		CASE
			WHEN rtDate > tDate
				OR (
					rtDate = tDate
					AND rtTransactionID > tTransactionID
				) THEN CASE
				WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tlQuantity
				ELSE -tlQuantity
			END
			ELSE 0
		END
	)
FROM
	shopcharttmp." . $prefix . "ReferenceTransactions,
	shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON ctiItemID = tlItemID
	AND ctiAssemblyID = tlAssemblyID
WHERE
	ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY |
        TTF_ADDS_INVENTORY_TO_ASSEMBLY) . "
	AND tDate <= rtDate
	AND (
		tToPersonID = ctiFromPersonID
		OR tFromPersonID = ctiFromPersonID
	)
GROUP BY
	rtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID");
}

function add_assy_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtTransactionID,
	rtTransactionTypeID,
	rtFromPersonID,
	rtDate,
	rtFlags,
	rtCurrentFlags,
	rtItemID,
	rtLineGroupID,
	rtAssemblyID,
	rtQtyUsed
) SELECT
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	COALESCE(itLineGroupID, 0),
	itAssemblyID,
	CASE
		WHEN itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN
			CASE
				WHEN itRunningTotal < itQtyReqd THEN itQuantity
				ELSE itQtyReqd - (itRunningTotal - itQuantity)
			END - SUM(
				CASE
					WHEN otDate > itDate OR (otDate = itDate AND otTransactionID > itTransactionID) THEN otQuantity
					ELSE 0
				END
			)
		ELSE 
			CASE
				WHEN itQuantity < itQtyReqd THEN itQuantity
				ELSE itQtyReqd
			END
	END
FROM
	shopcharttmp." . $prefix . "OutTransactions RIGHT JOIN
		(SELECT
			tTransactionID itTransactionID,
			tTransactionTypeID itTransactionTypeID,
			tFromPersonID itFromPersonID,
			tDate itDate,
			ttFlags itFlags,
			ctiFlags itCurrentFlags,
			tlItemID itItemID,
			tlLineGroupID itLineGroupID,
			tlAssemblyID itAssemblyID,
			ctiQuantity itQtyReqd,
			SUM(tlQuantity) itQuantity,
			SUM(tlQuantity) + ptQuantity itRunningTotal
		FROM
			shopcharttmp." . $prefix . "PastTransactions RIGHT JOIN (
				shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN (
					TransactionLines INNER JOIN (
						Transactions INNER JOIN
							TransactionTypes
						ON tTransactionTypeID = ttTransactionTypeID
					) ON tlTransactionID = tTransactionID
				) ON ctiFromPersonID = tToPersonID
				AND ctiItemID = tlItemID
				AND ctiAssemblyID = tlAssemblyID
			) ON ptTransactionID = tTransactionID
			AND ptPersonID = tToPersonID
			AND ptItemID = tlItemID
			AND ptAssemblyID = tlAssemblyID
		WHERE
			NOT ttFlags & " . (TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY) . "
			AND (
				tDate < ctiDate
				OR (
					tDate = ctiDate
					AND tTransactionID < ctiTransactionID
				)
			) AND (
				(
					NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
					OR tFromPersonID <> tToPersonID
				)
			)
		GROUP BY
			tTransactionID,
			tTransactionTypeID,
			tFromPersonID,
			tDate,
			ttFlags,
			ctiFlags,
			tlItemID,
			tlLineGroupID,
			ctiQuantity,
			ptQuantity
		) it
	ON otItemID = itItemID
	AND otAssemblyID = itAssemblyID
GROUP BY
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	itLineGroupID,
	itAssemblyID,
	itQtyReqd,
	itQuantity,
	itRunningTotal
HAVING
	itRunningTotal - itQuantity - SUM(
		CASE
			WHEN otDate > itDate 
				OR (
					otDate = itDate
					AND otTransactionID > itTransactionID
				) THEN otQuantity
			ELSE 0
	END) < itQtyReqd
	OR NOT itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY));
}

function make_non_assy_items($transaction_id, $line_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "NonAssyItems (
	naiDate datetime,
	naiFlags int,
	naiFromPersonID int,
	naiItemID int PRIMARY KEY,
	naiQuantity decimal(11, 2),
	naiTransactionID int
)");
    if($line_id == 0) {
        $line_id = "";
    } else {
        $line_id = "
    AND tlLineID = $line_id";
    }
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "NonAssyItems (
	naiTransactionID,
	naiFromPersonID,
	naiDate,
	naiFlags,
	naiItemID,
	naiQuantity
) SELECT
	85,
	iaqFromPersonID,
	iaqDate,
	iaqFlags,
	iaqItemID,
	SUM(iaqQuantity - COALESCE(alqQuantity, 0))
FROM
	(SELECT
		rtItemID alqItemID,
		rtAssemblyID alqAssemblyID,
		SUM(rtQtyUsed) alqQuantity
	FROM
		shopcharttmp." . $prefix . "ResponsibleTransactions
	WHERE
		rtFlags & " . TTF_TRANSFERS_INVENTORY . "
	GROUP BY
		rtItemID,
		rtAssemblyID
	) alq RIGHT JOIN
		(SELECT
			tFromPersonID iaqFromPersonID,
			tDate iaqDate,
			ttFlags iaqFlags,
			tlItemID iaqItemID,
			tlAssemblyID iaqAssemblyID,
			SUM(tlQuantity) iaqQuantity
		FROM
			TransactionLines INNER JOIN (
				Transactions INNER JOIN
					TransactionTypes
				ON tTransactionTypeID = ttTransactionTypeID
			) ON tlTransactionID = tTransactionID
		WHERE
			tTransactionID = $transaction_id $line_id
		GROUP BY
			tFromPersonID,
			tDate,
			ttFlags,
			tlItemID,
			tlAssemblyID
		) iaq
	ON alqItemID = iaqItemID
	AND alqAssemblyID = iaqAssemblyID
GROUP BY
	iaqFlags,
	iaqItemID
HAVING
	SUM(iaqQuantity - COALESCE(alqQuantity, 0)) > 0");
}

function add_non_assy_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtTransactionID,
	rtTransactionTypeID,
	rtFromPersonID,
	rtDate,
	rtFlags,
	rtCurrentFlags,
	rtItemID,
	rtLineGroupID,
	rtAssemblyID,
	rtQtyUsed
) SELECT
	tTransactionID,
	tTransactionTypeID,
	tFromPersonID,
	tDate,
	ttFlags | " . TTF_GENERAL_INVENTORY . ",
	naiFlags,
	tlItemID,
	COALESCE(tlLineGroupID, 0),
	COALESCE(tlAssemblyID, 0),
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "NonAssyItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON naiItemID = tlItemID
WHERE
	(
		tDate < naiDate
		OR (
			tDate = naiDate
			AND tTransactionID < naiTransactionID
		)
	) AND NOT ttFlags & " . TTF_REMOVES_INVENTORY . "
	AND tToPersonID = naiFromPersonID
	AND (
		tlAssemblyID IS NULL
		OR ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
	)
GROUP BY
	tTransactionID,
	tTransactionTypeID,
	tFromPersonID,
	tDate,
	ttFlags | " . TTF_GENERAL_INVENTORY . ",
	naiFlags,
	tlItemID,
	COALESCE(tlLineGroupID, 0),
	COALESCE(tlAssemblyID, 0)");
}

function make_salvage_items($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "SalvageItems (
	siAssemblyID int,
	siDate datetime,
	siFromPersonID int,
	siFlags int,
	siItemID int,
	siQuantity decimal(11, 2),
	siTransactionID int,
	PRIMARY KEY (
		siTransactionID,
		siItemID,
		siAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvageItems (
	siTransactionID,
	siDate,
	siFromPersonID,
	siFlags,
	siItemID,
	siAssemblyID,
	siQuantity
) SELECT
	tTransactionID,
	tDate,
	tFromPersonID,
	naiFlags,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "NonAssyItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON naiFromPersonID = tToPersonID
    AND naiItemID = tlItemID
WHERE
	ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
	AND tlAssemblyID IS NOT NULL
GROUP BY
	tTransactionID,
	tDate,
	tFromPersonID,
	naiFlags,
	tlItemID,
	tlAssemblyID");
}

function make_salvage_out_transactions($prefix) {
    global $mysqli;
    $mysqli->query ("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "SalvageOutTransactions (
	sotAssemblyID int,
	sotDate datetime,
	sotItemID int,
	sotQuantity decimal(11, 2),
	sotTransactionID int,
	PRIMARY KEY (
		sotTransactionID,
		sotItemID,
		sotAssemblyID
	)
)") ;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvageOutTransactions (
	sotTransactionID,
	sotDate,
	sotItemID,
	sotAssemblyID,
	sotQuantity
) SELECT
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON siFromPersonID = tFromPersonID
	AND siItemID = tlItemID
	AND siAssemblyID = tlAssemblyID
WHERE
	(
		tDate < siDate
		OR (
			tDate = siDate
			AND tTransactionID < siTransactionID
		)
	) AND (
		(
			ttFlags & " . TTF_TRANSFERS_INVENTORY . "
			AND tFromPersonID <> tToPersonID
		) OR ttFlags & " . TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
	)
GROUP BY
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID");
}

function make_salvage_reference_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "SalvageReferenceTransactions (
	srtDate datetime,
	srtTransactionID int PRIMARY KEY,
	INDEX srtiDate (
		srtDate
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvageReferenceTransactions (
	srtTransactionID,
	srtDate
) SELECT DISTINCT
	tTransactionID,
	tDate
FROM
	shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
		TransactionLines INNER JOIN
			Transactions
		ON tlTransactionID = tTransactionID
	) ON siItemID = tlItemID
	AND siAssemblyID = tlAssemblyID
WHERE
	tFromPersonID = siFromPersonID
	OR tToPersonID = siFromPersonID");
}

function make_salvage_past_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "SalvagePastTransactions (
	sptAssemblyID int,
	sptItemID int,
	sptPersonID int,
	sptQuantity decimal(11, 2),
	sptTransactionID int,
	PRIMARY KEY (
		sptTransactionID,
		sptPersonID,
		sptItemID,
		sptAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvagePastTransactions (
	sptTransactionID,
	sptPersonID,
	sptItemID,
	sptAssemblyID,
	sptQuantity
) SELECT
	srtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID,
	SUM(
		CASE
			WHEN srtDate > tDate
				OR (
					srtDate = tDate
					AND srtTransactionID > tTransactionID
				) THEN CASE
				WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tlQuantity
				ELSE -tlQuantity
			END
			ELSE 0
		END
	)
FROM
	shopcharttmp." . $prefix . "SalvageReferenceTransactions,
	shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON siItemID = tlItemID
	AND siAssemblyID = tlAssemblyID
WHERE
	ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY |
        TTF_ADDS_INVENTORY_TO_ASSEMBLY) . "
	AND tDate <= srtDate
	AND (
		tToPersonID = siFromPersonID
		OR tFromPersonID = siFromPersonID
	)
GROUP BY
	srtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID");
}

function add_salvage_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtTransactionID,
	rtTransactionTypeID,
	rtFromPersonID,
	rtDate,
	rtFlags,
	rtCurrentFlags,
	rtItemID,
	rtLineGroupID,
	rtAssemblyID,
	rtQtyUsed
) SELECT
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	COALESCE(itLineGroupID, 0),
	itAssemblyID,
	CASE
		WHEN itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN
			CASE
				WHEN itRunningTotal < itQtyReqd THEN itQuantity
				ELSE itQtyReqd - (itRunningTotal - itQuantity)
			END - SUM(
				CASE
					WHEN sotDate > itDate OR (sotDate = itDate AND sotTransactionID > itTransactionID) THEN sotQuantity
					ELSE 0
				END
			)
		ELSE 
			CASE
				WHEN itQuantity < itQtyReqd THEN itQuantity
				ELSE itQtyReqd
			END
	END
FROM
	shopcharttmp." . $prefix . "SalvageOutTransactions RIGHT JOIN
		(SELECT
			tTransactionID itTransactionID,
			tTransactionTypeID itTransactionTypeID,
			tFromPersonID itFromPersonID,
			tDate itDate,
			ttFlags itFlags,
			siFlags itCurrentFlags,
			tlItemID itItemID,
			tlLineGroupID itLineGroupID,
			tlAssemblyID itAssemblyID,
			siQuantity itQtyReqd,
			SUM(tlQuantity) itQuantity,
			SUM(tlQuantity) + sptQuantity itRunningTotal
		FROM
			shopcharttmp." . $prefix . "SalvagePastTransactions RIGHT JOIN (
				shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
					TransactionLines INNER JOIN (
						Transactions INNER JOIN
							TransactionTypes
						ON tTransactionTypeID = ttTransactionTypeID
					) ON tlTransactionID = tTransactionID
				) ON siFromPersonID = tToPersonID
				AND siItemID = tlItemID
				AND siAssemblyID = tlAssemblyID
			) ON sptTransactionID = tTransactionID
			AND sptPersonID = tToPersonID
			AND sptItemID = tlItemID
			AND sptAssemblyID = tlAssemblyID
		WHERE
			NOT ttFlags & " . (TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY) . "
			AND (
				tDate < siDate
				OR (
					tDate = siDate
					AND tTransactionID < siTransactionID
				)
			) AND (
				(
					NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
					OR tFromPersonID <> tToPersonID
				)
			)
		GROUP BY
			tTransactionID,
			tTransactionTypeID,
			tDate,
			ttFlags,
			siFlags,
			tlItemID,
			tlLineGroupID,
			siQuantity,
			sptQuantity
		) it
	ON sotItemID = itItemID
	AND sotAssemblyID = itAssemblyID
GROUP BY
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	COALESCE(itLineGroupID, 0),
	itAssemblyID,
	itQtyReqd,
	itQuantity,
	itRunningTotal
HAVING
	itRunningTotal - itQuantity - SUM(
		CASE
			WHEN sotDate > itDate 
				OR (
					sotDate = itDate
					AND sotTransactionID > itTransactionID
				) THEN sotQuantity
			ELSE 0
	END) < itQtyReqd
	OR NOT itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY));
}

function make_past_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "PastCost (
    pcExtAmount decimal(20, 6),
    pcItemID int,
    pcTotalQuantity decimal(11, 2),
	pcTransactionTypeID int
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PastCost (
	pcTransactionTypeID,
    pcItemID,
    pcExtAmount,
    pcTotalQuantity
) SELECT
	tTransactionTypeID,
	ptl.tlItemID,
	SUM(
		CASE
			WHEN (rtCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") <> (ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") THEN
				CASE
					WHEN rtCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN ptl.tlUnitAmount / pMonetaryPerSecond
					ELSE ptl.tlUnitAmount * pMonetaryPerSecond
				END
			ELSE ptl.tlUnitAmount
		END * ptl.tlQuantity),
	SUM(ptl.tlQuantity)
FROM
	TransactionLines ctl INNER JOIN (
		(SELECT DISTINCT
			rtTransactionID,
			rtTransactionTypeID,
			rtCurrentFlags
		FROM
			shopcharttmp." . $prefix . "ResponsibleTransactions
		) rt INNER JOIN (
			TransactionLines ptl INNER JOIN (
				People INNER JOIN (
					Transactions INNER JOIN
						TransactionTypes
					ON tTransactionTypeID = ttTransactionTypeID
				) ON pPersonID = tFromPersonID
			) ON ptl.tlTransactionID = tTransactionID
		) ON rtTransactionTypeID = tTransactionTypeID
	) ON ctl.tlTransactionID = rtTransactionID
	AND ctl.tlItemID = ptl.tlItemID
WHERE
	ptl.tlLineGroupID IS NULL
GROUP BY
	tTransactionID,
	ptl.tlItemID");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PastCost (
	pcTransactionTypeID,
    pcItemID,
    pcExtAmount,
    pcTotalQuantity
) SELECT
	tTransactionTypeID,
	tlItemID,
	SUM(
		CASE
			WHEN (rtCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") <> (ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") THEN
				CASE
					WHEN rtCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN lgcTotalAmount / lgcLineGroupCount / pMonetaryPerSecond
					ELSE lgcTotalAmount / lgcLineGroupCount * pMonetaryPerSecond
				END
			ELSE lgcTotalAmount / lgcLineGroupCount
		END * tlQuantity),
	SUM(tlQuantity)
FROM
	(SELECT
		tlgTransactionID lgcTransactionID,
		tlgLineGroupID lgcLineGroupID,
		tlgTotalAmount lgcTotalAmount,
		SUM(tlQuantity) lgcLineGroupCount
	FROM
		TransactionLineGroups INNER JOIN
			TransactionLines
		ON tlgTransactionID = tlTransactionID
		AND tlgLineGroupID = tlLineGroupID
	GROUP BY
		tlgTransactionID,
		tlgLineGroupID,
		tlgTotalAmount
	) lgc INNER JOIN (
		TransactionLines INNER JOIN (
			People INNER JOIN (
				TransactionTypes INNER JOIN (
					Transactions INNER JOIN
						(SELECT DISTINCT
							rtTransactionTypeID,
							rtCurrentFlags
						FROM
							shopcharttmp." . $prefix . "ResponsibleTransactions
						) rt
					ON tTransactionTypeID = rtTransactionTypeID
				) ON ttTransactionTypeID = tTransactionTypeID
			) ON pPersonID = tFromPersonID
		) ON tlTransactionID = tTransactionID
	) ON lgcTransactionID = tlTransactionID
	AND lgcLineGroupID = tlLineGroupID
GROUP BY
	tTransactionTypeID,
	tlItemID");
}

function make_item_average_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "ItemAverageCost (
    iacItemID int,
	iacLineGroupID int,
    iacUnitAmount decimal(20, 6),
    PRIMARY KEY (
        iacItemID,
        iacLineGroupID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ItemAverageCost (
    iacItemID,
    iacUnitAmount
) SELECT
    tlItemID,
    SUM(pcExtAmount) / SUM(pcTotalQuantity)
FROM
    shopcharttmp." . $prefix . "PastCost INNER JOIN (
		TransactionLines INNER JOIN
			(SELECT DISTINCT
				rtTransactionID,
				rtLineGroupID
			FROM
				shopcharttmp." . $prefix . "ResponsibleTransactions
			) rt
		ON tlTransactionID = rtTransactionID
		AND tlLineGroupID = rtLineGroupID
    ) ON pcItemID = tlItemID
WHERE
    tlLineGroupID <> 0
GROUP BY
    tlItemID");
}

function make_total_unit_amount($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "TotalUnitAmount (
    tuaLineGroupID int,
	tuaTransactionID int,
    tuaUnitAmount decimal(20, 6),
	PRIMARY KEY (
		tuaTransactionID,
		tuaLineGroupID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TotalUnitAmount (
	tuaTransactionID,
    tuaLineGroupID,
    tuaUnitAmount
) SELECT
	tlTransactionID,
    tlLineGroupID,
    SUM(iacUnitAmount * tlQuantity)
FROM
    shopcharttmp." . $prefix . "ItemAverageCost INNER JOIN (
		TransactionLines INNER JOIN
			(SELECT DISTINCT
				rtTransactionID
			FROM
				shopcharttmp." . $prefix . "ResponsibleTransactions
			) rt
		ON tlTransactionID = rtTransactionID
    ) ON iacItemID = tlItemID
GROUP BY
	tlTransactionID,
    tlLineGroupID");
}

function make_item_percent($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "ItemPercent (
	ipItemID int,
	ipLineGroupID int,
	ipPercent decimal(20, 6),
	ipTransactionID int,
	PRIMARY KEY (
		ipTransactionID,
		ipLineGroupID,
		ipItemID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ItemPercent (
	ipTransactionID,
	ipLineGroupID,
	ipItemID,
	ipPercent
) SELECT
	tlTransactionID,
	tuaLineGroupID,
	tlItemID,
	iacUnitAmount / tuaUnitAmount
FROM
	shopcharttmp." . $prefix . "TotalUnitAmount INNER JOIN (
		shopcharttmp." . $prefix . "ItemAverageCost INNER JOIN (
			TransactionLines INNER JOIN (
				TransactionLineGroups INNER JOIN
					(SELECT DISTINCT
						rtTransactionID
					FROM
						shopcharttmp." . $prefix . "ResponsibleTransactions
					) rt
				ON tlgTransactionID = rtTransactionID
			) ON tlTransactionID = tlgTransactionID
			AND tlLineGroupID = tlgLineGroupID
		) ON iacItemID = tlItemID
	) ON tuaTransactionID = rtTransactionID
	AND tuaLineGroupID = tlgLineGroupID");
}

function make_grouped_items($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "GroupedItems (
	giLineGroupID int,
	giItemID int,
	giTransactionID int,
	giUnitAmount decimal(20, 6),
	PRIMARY KEY (
		giTransactionID,
		giLineGroupID,
		giItemID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "GroupedItems (
	giTransactionID,
	giLineGroupID,
	giItemID,
	giUnitAmount
) SELECT
	ipTransactionID giTransactionID,
	ipLineGroupID giLineGroupID,
	ipItemID giItemID,
	ipPercent * tlgTotalAmount giUnitAmount
FROM
	shopcharttmp." . $prefix . "ItemPercent INNER JOIN (
		TransactionLineGroups INNER JOIN
			(SELECT DISTINCT
				rtTransactionID
			FROM
				shopcharttmp." . $prefix . "ResponsibleTransactions
			) rt
		ON tlgTransactionID = rtTransactionID
	) ON ipTransactionID = rtTransactionID
	AND ipLineGroupID = tlgLineGroupID");
}

function make_transaction_item_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "TransactionItemCost (
	ticAssemblyID int,
	ticCurrentFlags int,
	ticFlags int,
	ticFromPersonID int,
	ticItemID int,
	ticQuantity decimal(11, 2),
	ticTransactionID int,
	ticUnitAmount decimal(20, 6),
	PRIMARY KEY (
		ticTransactionID,
		ticAssemblyID,
		ticItemID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TransactionItemCost (
	ticTransactionID,
	ticFromPersonID,
	ticCurrentFlags,
	ticFlags,
	ticAssemblyID,
	ticItemID,
	ticUnitAmount,
	ticQuantity
) SELECT
	rtTransactionID,
	rtFromPersonID,
	rtCurrentFlags,
	rtFlags,
	rtAssemblyID,
	rtItemID,
	AVG(COALESCE(giUnitAmount, ugiUnitAmount)),
	rtQtyUsed
FROM
    shopcharttmp." . $prefix . "GroupedItems RIGHT JOIN (
		(SELECT
			tlTransactionID ugiTransactionID,
			tlItemID ugiItemID,
			AVG(tlUnitAmount) ugiUnitAmount
		FROM
			TransactionLines
		WHERE
			tlLineGroupID IS NULL
		GROUP BY
			tlTransactionID,
			tlItemID
		) ugi RIGHT JOIN
			shopcharttmp." . $prefix . "ResponsibleTransactions
		ON ugiTransactionID = rtTransactionID
		AND ugiItemID = rtItemID
    ) ON giTransactionID = rtTransactionID
	AND giLineGroupID = rtLineGroupID
	AND giItemID = rtItemID
GROUP BY
	rtTransactionID,
	rtFromPersonID,
	rtCurrentFlags,
	rtFlags,
	rtItemID,
	rtQtyUsed");
}

function make_directly_responsible($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "DirectlyResponsible (
	drItemID int PRIMARY KEY,
	drQuantity decimal(11, 2),
	drTotalAmount decimal(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "DirectlyResponsible (
	drItemID,
	drQuantity,
	drTotalAmount
) SELECT
	ticItemID,
	SUM((ticFlags & " . TTF_TRANSFERS_INVENTORY . ") * ticQuantity),
	SUM(
		CASE
			WHEN (ticCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") <> (ticFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") THEN
				CASE
					WHEN ticCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN ticUnitAmount / pMonetaryPerSecond
					ELSE ticUnitAmount * pMonetaryPerSecond
				END
			ELSE ticUnitAmount
		END * ticQuantity)
FROM
	shopcharttmp." . $prefix . "TransactionItemCost INNER JOIN
		People
	ON ticFromPersonID = pPersonID
WHERE
	NOT ticFlags & " . TTF_GENERAL_INVENTORY . "
GROUP BY
	ticItemID");
}

function make_general_invy_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp." . $prefix . "GeneralInvyCost (
	gicItemID int PRIMARY KEY,
	gicUnitAmount decimal(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "GeneralInvyCost (
	gicItemID,
	gicUnitAmount
) SELECT
	ticItemID,
	SUM(
		CASE
			WHEN (ticCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") <> (ticFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") THEN
				CASE
					WHEN ticCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN ticUnitAmount / pMonetaryPerSecond
					ELSE ticUnitAmount * pMonetaryPerSecond
				END
			ELSE ticUnitAmount
		END * ticQuantity) / SUM(ticQuantity)
FROM
	shopcharttmp." . $prefix . "TransactionItemCost INNER JOIN
		People
	ON ticFromPersonID = pPersonID
WHERE
	ticFlags & " . TTF_GENERAL_INVENTORY . "
GROUP BY
	ticItemID");
}