<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/
     
require_once('fun/mysqli.php');
require_once('sql/transactions.php');

function add_transaction($transaction_type_id, $from_person_id, $to_person_id) {
    if(!isset($GLOBALS['administrator'])) return;
    global $mysqli;
    $mysqli->query("
INSERT INTO Transactions (
    tDate,
    tFromPersonID,
    tToPersonID,
    tTransactionTypeID
) VALUES (
    NOW(),
    $from_person_id,
    $to_person_id,
    $transaction_type_id
)");
    return $mysqli->insert_id;
}

function delete_transaction($transaction_id) {
    if(!isset($GLOBALS['administrator'])) return;
    global $mysqli;
    $mysqli->query("
DELETE
FROM
    Transactions
WHERE
    tTransactionID = $transaction_id");
}

function add_assembly($transaction_id, $assembly_id) {
    if(!isset($GLOBALS['administrator'])) return;
    global $mysqli;
    $mysqli->query("
INSERT INTO TransactionLines (
    tlTransactionID,
    tlLineID,
    tlItemID,
    tlFromURL,
    tlQuantity,
    tlUnitAmount,
    tlAssemblyID,
    tlLineGroupID
) SELECT
    $transaction_id,
    @curRank := @curRank + 1,
    atiItemID,
    tlFromURL,
    atiQuantity,
    COALESCE(tlUnitAmount, 0),
    $assembly_id,
    COALESCE(lgLineGroupID, 1)
FROM
    (SELECT
        @curRank := COALESCE((SELECT
            MAX(tlLineID)
        FROM
            TransactionLines
        WHERE
            tlTransactionID = $transaction_id), 0)
    ) cr,
    (SELECT
        MAX(tlgLineGroupID) lgLineGroupID
    FROM
        TransactionLineGroups
    WHERE
        tlgTransactionID = $transaction_id
    ) lg,
    (SELECT
        tlItemID,
        tlFromURL,
        tlUnitAmount
    FROM
        (SELECT
            tlItemID ltnItemID,
            MAX(tTransactionID) ltnTransactionID
        FROM
            (SELECT
                t.tFromPersonID ltdFromPersonID,
                t.tTransactionTypeID ltdTransactionTypeID,
                tlItemID ltdItemID,
                MAX(t.tDate) ltdDate
            FROM
                Transactions ft INNER JOIN (
                    TransactionLines INNER JOIN
                        Transactions t
                    ON tlTransactionID = t.tTransactionID
                ) ON ft.tFromPersonID = t.tFromPersonID
                AND ft.tTransactionTypeID = t.tTransactionTypeID
            WHERE
                ft.tTransactionID = $transaction_id
            GROUP BY
                t.tFromPersonID,
                t.tTransactionTypeID,
                tlItemID
            ) ltd INNER JOIN (
                TransactionLines INNER JOIN
                    Transactions
                ON tlTransactionID = tTransactionID
            ) ON ltdFromPersonID = tFromPersonID
            AND ltdTransactionTypeID = tTransactionTypeID
            AND ltdItemID = tlItemID
        GROUP BY
            tlItemID
        ) ltn INNER JOIN
            TransactionLines
        ON ltnTransactionID = tlTransactionID
        AND ltnItemID = tlItemID
    ) tl RIGHT JOIN
        (SELECT
            atiItemID,
            SUM(atiQuantity) atiQuantity
        FROM
            (SELECT
                atiItemID,
                atiQuantity
            FROM
                Assemblies INNER JOIN
                    AssemblyTypeItems
                ON aAssemblyTypeID = atiAssemblyTypeID
            WHERE
                aAssemblyID = $assembly_id
            UNION
            SELECT
                tlItemID,
                SUM(tlQuantity)
            FROM
                TransactionLines INNER JOIN (
                    Transactions INNER JOIN
                        TransactionTypes
                    ON tTransactionTypeID = ttTransactionTypeID
                ) ON tlTransactionID = tTransactionID
            WHERE
                tlAssemblyID = $assembly_id
                AND NOT ttFlags & 6
            ) ati
        GROUP BY
            atiItemID
        ) ati
    ON tlItemID = atiItemID");
    $mysqli->query("
INSERT INTO TransactionLineGroups (
    tlgTransactionID,
    tlgLineGroupID,
    tlgTotalAmount
) SELECT
    $transaction_id,
    MAX(tlLineGroupID),
    0
FROM
    TransactionLines
WHERE
    tlTransactionID = $transaction_id");
}

function get_item_cost($transaction_id, $line_id) {
    global $mysqli;
    $item_cost = $mysqli->query("
SELECT
    AVG(tlUnitAmount) UnitAmount
FROM
    (SELECT
        tlItemID ltnItemID,
        MAX(tTransactionID) ltnTransactionID
    FROM
        (SELECT
            t.tFromPersonID ltdFromPersonID,
            t.tTransactionTypeID ltdTransactionTypeID,
            tl.tlItemID ltdItemID,
            MAX(t.tDate) ltdDate
        FROM
            TransactionLines ftl INNER JOIN (
                Transactions ft INNER JOIN (
                    TransactionLines tl INNER JOIN
                        Transactions t
                    ON tl.tlTransactionID = t.tTransactionID
                ) ON ft.tFromPersonID = t.tFromPersonID
                AND ft.tTransactionTypeID = t.tTransactionTypeID
            ) ON ftl.tlTransactionID = ft.tTransactionID
            AND ftl.tlItemID = tl.tlItemID
        WHERE
            ft.tTransactionID = $transaction_id
            AND ftl.tlLineID = $line_id
        GROUP BY
            t.tFromPersonID,
            t.tTransactionTypeID,
            tl.tlItemID
        ) ltd INNER JOIN (
            TransactionLines INNER JOIN
                Transactions
            ON tlTransactionID = tTransactionID
        ) ON ltdFromPersonID = tFromPersonID
        AND ltdTransactionTypeID = tTransactionTypeID
        AND ltdItemID = tlItemID
    GROUP BY
        tlItemID
    ) ltn INNER JOIN
        TransactionLines
    ON ltnTransactionID = tlTransactionID
    AND ltnItemID = tlItemID");
    $item_cost = $item_cost->fetch_assoc();
    if($item_cost['UnitAmount'] == null) {
        return 0;
    } else {
        return $item_cost['UnitAmount'];
    }
}

function get_next_line_id($transaction_id) {
    global $mysqli;
    $next_line_id = $mysqli->query("
SELECT
    MAX(tlLineID) LastLineID
FROM
    TransactionLines
WHERE
    tlTransactionID = $transaction_id");
    if($next_line_id = $next_line_id->fetch_assoc()) {
        return $next_line_id['LastLineID'] + 1;
    } else {
        return 1;
    }
}

function add_transaction_line($transaction_id, $item_id) {
    if(!isset($GLOBALS['administrator'])) return;
    global $mysqli;
    $next_line_id = get_next_line_id($transaction_id);
    $mysqli->query("
INSERT INTO TransactionLines (
    tlTransactionID,
    tlLineID,
    tlItemID,
    tlFromURL,
    tlQuantity,
    tlUnitAmount,
    tlAssemblyID,
    tlLineGroupID
) SELECT
    $transaction_id,
    $next_line_id,
    $item_id,
    MAX(tlFromURL),
    1,
    MAX(tlUnitAmount),
    (SELECT
        lpaAssemblyID aaAssemblyID
    FROM
        (SELECT
            tlAssemblyID lpaAssemblyID,
            COUNT(*) lpaLines
        FROM
            TransactionLines
        WHERE
            tlTransactionID = $transaction_id
        GROUP BY
            tlAssemblyID
        ) lpa
    ORDER BY
        lpaLines DESC
    LIMIT 1
    ),
    (SELECT
        lpgLineGroupID agLineGroupID
    FROM
        (SELECT
            tlLineGroupID lpgLineGroupId,
            COUNT(*) lpgLines
        FROM
            TransactionLines
        WHERE
            tlTransactionID = $transaction_id
        GROUP BY
            tlLineGroupID
        ) lpg
    ORDER BY
        lpgLines DESC
    LIMIT 1
    )
FROM
    (SELECT
        tlItemID ltnItemID,
        MAX(tTransactionID) ltnTransactionID
    FROM
        (SELECT
            t.tFromPersonID ltdFromPersonID,
            t.tTransactionTypeID ltdTransactionTypeID,
            tlItemID ltdItemID,
            MAX(t.tDate) ltdDate
        FROM
            Transactions ft INNER JOIN (
                TransactionLines INNER JOIN
                    Transactions t
                ON tlTransactionID = t.tTransactionID
            ) ON ft.tFromPersonID = t.tFromPersonID
            AND ft.tTransactionTypeID = t.tTransactionTypeID
        WHERE
            ft.tTransactionID = $transaction_id
            AND tlItemID = $item_id
            AND (
                ft.tDate > t.tDate
                OR (
                    ft.tDate = t.tDate
                    AND ft.tTransactionID > t.tTransactionID
                )
            )
        GROUP BY
            t.tFromPersonID,
            t.tTransactionTypeID,
            tlItemID
        ) ltd INNER JOIN (
            TransactionLines INNER JOIN
                Transactions
            ON tlTransactionID = tTransactionID
        ) ON ltdFromPersonID = tFromPersonID
        AND ltdTransactionTypeID = tTransactionTypeID
        AND ltdItemID = tlItemID
    GROUP BY
        tlItemID
    ) ltn INNER JOIN
        TransactionLines
    ON ltnTransactionID = tlTransactionID
    AND ltnItemID = tlItemID");
    if($mysqli->affected_rows == 0) {
        $mysqli->query("
INSERT INTO TransactionLines (
    tlTransactionID,
    tlLineID,
    tlItemID,
    tlQuantity,
    tlUnitAmount,
    tlAssemblyID,
    tlLineGroupID
) SELECT
    $transaction_id,
    $next_line_id,
    $item_id,
    1,
    0,
    aaAssemblyID,
    agLineGroupID
FROM
    (SELECT
        lpgTransactionID agTransactionID,
        lpgLineGroupID agLineGroupID
    FROM
        (SELECT
            tlTransactionID lpgTransactionID,
            tlLineGroupID lpgLineGroupId,
            COUNT(*) lpgLines
        FROM
            TransactionLines
        WHERE
            tlTransactionID = $transaction_id
        GROUP BY
            tlTransactionID,
            tlLineGroupID
        ) lpg
    ORDER BY
        lpgLines DESC
    LIMIT 1
    ) ag RIGHT JOIN (
        (SELECT
            lpaTransactionID aaTransactionID,
            lpaAssemblyID aaAssemblyID
        FROM
            (SELECT
                tlTransactionID lpaTransactionID,
                tlAssemblyID lpaAssemblyID,
                COUNT(*) lpaLines
            FROM
                TransactionLines
            WHERE
                tlTransactionID = $transaction_id
            GROUP BY
                tlTransactionID,
                tlAssemblyID
            ) lpa
        ORDER BY
            lpaLines DESC
        LIMIT 1
        ) aa RIGHT JOIN
            Transactions
        ON aaTransactionID = tTransactionID
    ) ON agTransactionID = tTransactionID
WHERE
    tTransactionID = $transaction_id");
    }
    if($mysqli->errno <> 0) {
        throw new Exception($mysqli->error);
    }
    return $next_line_id;
}

function get_transaction_line_groups($transaction_id) {
    global $mysqli;
    $tig = $mysqli->query("
SELECT
    tlgLineGroupID,
    tlgTotalAmount
FROM
    TransactionLineGroups
WHERE
    tlgTransactionID = $transaction_id
ORDER BY
    tlgLineGroupID");
    $response = array();
    while($row = $tig->fetch_assoc()) {
        $response[$row['tlgLineGroupID']] = $row['tlgTotalAmount'];
    }
    return $response;
}

function time_to_seconds($time) {
    $time_parts = explode(':', $time);
    $multiplier = 1;
    $seconds = 0;
    foreach(array_reverse($time_parts) as $part) {
        if(is_numeric($part) == 0) return -1;
        $seconds += $part * $multiplier;
        $multiplier = $multiplier * 60;
    }
    return $seconds;
}

function get_transaction_attributes($attributes) {
    global $mysqli;
    $set = '';
    if(isset($attributes['p_tFromPersonID']) && $attributes['p_tFromPersonID'] <> $attributes['tFromPersonID']) {
        $set .= "
    tFromPersonID = " . $mysqli->real_escape_string($attributes['tFromPersonID']) . ",";
    }
    if(isset($attributes['p_tToPersonID']) && $attributes['p_tToPersonID'] <> $attributes['tToPersonID']) {
        $set .= "
    tToPersonID = " . $mysqli->real_escape_string($attributes['tToPersonID']) . ",";
    }
    if(isset($attributes['p_tTransactionTypeID']) && $attributes['p_tTransactionTypeID'] <>
        $attributes['tTransactionTypeID']) {
        $set .= "
    tTransactionTypeID = " . $mysqli->real_escape_string($attributes['tTransactionTypeID']) . ",";
    }
    if(isset($attributes['p_tDate'])) {
        $new_date = date_parse($attributes['tDate']);
        if($new_date['error_count'] > 0) {
            throw new Exception("Transaction Date must be a valid date.");
        }
        $old_date = date_parse($attributes['p_tDate']);
        if($old_date['year'] . $old_date['month'] . $old_date['day'] . $old_date['hour'] . $old_date['minute']
            . $old_date['second'] <> $new_date['year'] . $new_date['month'] . $new_date['day'] .
            $new_date['hour'] . $new_date['minute'] . $new_date['second']) {
            $set .= "
    tDate = '" . $new_date['year'] . '-' . $new_date['month'] . '-' .$new_date['day'] . ' ' . $new_date['hour']
        . ':' . $new_date['minute'] . ':' . $new_date['second'] . "',";
        }
    }
    if(isset($attributes['p_tShipping']) && $attributes['p_tShipping'] <> $attributes['tShipping']) {
        if(!is_numeric($attributes['tShipping'])) {
            throw new Exception("Shipping amount must be a numeric value.");
        }
        $set .= "
    tShipping = " . $attributes['tShipping'] . ",";
    }
    if(isset($attributes['p_tTax']) && $attributes['p_tTax'] <> $attributes['tTax']) {
        if(!is_numeric($attributes['tTax'])) {
            throw new Exception("Tax amount must be a numeric value.");
        }
        $set .="
    tTax = " . $attributes['tTax'] . ",";
    }
    return $set;
}

function get_transaction_line($transaction_id, $line_id) {
    global $mysqli;
    $transaction_line = $mysqli->query("
SELECT
    tTransactionTypeID,
    iDescription,
    tDate,
    tFromPersonID,
    tToPersonID
FROM
    Items INNER JOIN (
        TransactionLines INNER JOIN
            Transactions
        ON tlTransactionID = tTransactionID
    ) ON iItemID = tlItemID
WHERE
    tTransactionID = $transaction_id
    AND tlLineID = $line_id");
    if($transaction_line = $transaction_line->fetch_assoc()) {
        return $transaction_line;
    } else {
        return array();
    }
}

function get_transaction_line_attributes($transaction_id, $line_id, $attributes) {
    global $mysqli;
    if(isset($attributes['p_tlQuantity']) && $attributes['p_tlQuantity'] <> $attributes['tlQuantity'])  {
        if(!is_numeric($attributes['tlQuantity'])) {
            throw new Exception("Quantity must be a numeric value.");
        }
        if($attributes['tlQuantity'] <= 0) {
            return 'DELETE';
        } else {
            $set = "
    tlQuantity = " . $attributes['tlQuantity'] . ",";
        }
    } else {
        $set = '';
    }
    if(isset($attributes['p_tlLineGroupID']) && $attributes['p_tlLineGroupID'] <> $attributes['tlLineGroupID']) {
        if($attributes['tlLineGroupID'] == 0) {
            $set .= "
    tlLineGroupID = NULL,";
        } else {
            $set .= "
    tlLineGroupID = " . $mysqli->real_escape_string($attributes['tlLineGroupID']) . ",
    tlUnitAmount = 0,";
        }
    }
    if((isset($attributes['tlLineGroupID'])  && $attributes['tlLineGroupID'] == 0) ||
        !isset($attributes['tlLineGroupID'])) {
        if(isset($attributes['line_tfs'])) {
            $set .= "
    tlUnitAmount = CASE
        WHEN CURTIME() < TIME(tDate) THEN tlUnitAmount
        ELSE TIME_TO_SEC(TIMEDIFF(CURTIME(), TIME(tDate))) / tlQuantity
    END,";
        } elseif(isset($attributes['p_tlExtAmount']) && isset($attributes['tlExtAmount']) &&
            $attributes['p_tlExtAmount'] <> $attributes['tlExtAmount']) {
            $seconds = time_to_seconds($attributes['tlExtAmount']);
            if($seconds == -1) {
                throw new Exception("To Ext Amt must be a time value either in the standard hh:mm:ss format or a number of seconds.");
            }
            $set .= "
    tlUnitAmount = " . ($seconds / $attributes['tlQuantity']) . ",";
        } elseif(isset($attributes['p_tlUnitAmount']) && isset($attributes['tlUnitAmount']) &&
            $attributes['p_tlUnitAmount'] <> $attributes['tlUnitAmount']) {
            if(!is_numeric($attributes['tlUnitAmount'])) {
                throw new Exception("To Unit Amt must be a numeric value.");
            }
            $set .= "
    tlUnitAmount = " . $attributes['tlUnitAmount'] . ",";
        } else {
            $set .= "
    tlUnitAmount = " . get_item_cost($transaction_id, $line_id) . ",";
        }
    }
    if(isset($attributes['p_tlAssemblyID']) && $attributes['p_tlAssemblyID'] <> $attributes['tlAssemblyID']) {
        if($attributes['tlAssemblyID'] == 0) {
            $set .= "
    tlAssemblyID = NULL,";
        } else {
            $set .= "
    tlAssemblyID = " . $mysqli->real_escape_string($attributes['tlAssemblyID']) . ",";
        }
    }
    if(isset($attributes['p_tlFromURL']) && $attributes['p_tlFromURL'] <> $attributes['tlFromURL']) {
        $set .= "
    tlFromURL = '" . $mysqli->real_escape_string($attributes['tlFromURL']) . "',";
    }
    if($set == "") {
        return "";
    } else {
        return substr($set, 0, strlen($set) - 1);
    }
}

function update_attributes($transaction_id, $line_id, $line_group_id, $attributes) {
    if(!isset($GLOBALS['administrator'])) return;
    global $mysqli;
    $set = get_transaction_attributes($attributes);
    if($set <> "") {
        $mysqli->query("
UPDATE
    Transactions
SET " . substr($set, 0, strlen($set) - 1) . "
WHERE
    tTransactionID = $transaction_id");
    }
    if($line_id <> 0) {
        $set = get_transaction_line_attributes($transaction_id, $line_id, $attributes);
        if($set <> '') {
            if($set == 'DELETE') {
                $mysqli->query("
DELETE
FROM
    TransactionLines
WHERE
    tlTransactionID = $transaction_id
    AND tlLineID = $line_id");
                clear_dead_line_groups($transaction_id);
            } else {
                if(isset($attributes['tlLineGroupID'])) {
                    if($attributes['tlLineGroupID'] == 0) {
                        clear_dead_line_groups($transaction_id);
                    } else {
                    $mysqli->query("
INSERT IGNORE INTO TransactionLineGroups (
    tlgTransactionID,
    tlgLineGroupID,
    tlgTotalAmount
) VALUES (
    $transaction_id,
    " . $attributes['tlLineGroupID'] . ",
    0
)");
                    }
                }
                $mysqli->query("
UPDATE
    TransactionLines INNER JOIN
        Transactions
    ON tlTransactionID = tTransactionID
SET $set
WHERE
    tTransactionID = $transaction_id
    AND tlLineID = $line_id");
            }
        }
    }
    if(isset($attributes['tlgTotalAmount'])) {
        $seconds = 0;
        $total_amount = 0;
        $update = "
UPDATE
    TransactionLineGroups,";
        if(isset($attributes['line_group_tfs'])) {
            $seconds = "CASE
            WHEN CURTIME() < TIME(tDate) THEN tlgTotalAmount
            ELSE TIME_TO_SEC(TIMEDIFF(CURTIME(), TIME(tDate)))
        END";
            $update = "
UPDATE
    Transactions INNER JOIN
        TransactionLineGroups
    ON tTransactionID = tlgTransactionID,";
        } elseif(is_numeric($attributes['tlgTotalAmount'])) {
            $total_amount = $attributes['tlgTotalAmount'];
        } else {
            $seconds = time_to_seconds($attributes['tlgTotalAmount']);
        }
        $mysqli->query($update . "
    (SELECT
        ttFlags
    FROM
        TransactionTypes
    WHERE
        ttTransactionTypeID = " . $attributes['tTransactionTypeID'] . "
    ) tt
SET
    tlgTotalAmount = CASE
        WHEN ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN $seconds
        ELSE $total_amount
    END
WHERE
    tlgTransactionID = $transaction_id
    AND tlgLineGroupID = $line_group_id");
    }
}

function get_transaction_types() {
    global $mysqli;
    $transaction_types = $mysqli->query("
SELECT DISTINCT
    ttTransactionTypeID,
    ttDescription,
    tTransactionTypeID
FROM
    Transactions RIGHT JOIN
        TransactionTypes
    ON tTransactionTypeID = ttTransactionTypeID");
    $response = array();
    while($transaction_type = $transaction_types->fetch_assoc()) {
        $response[$transaction_type['ttTransactionTypeID']] = array(
            'description' => $transaction_type['ttDescription'],
            'active' => ($transaction_type['tTransactionTypeID'] <> null));
    }
    return $response;
}

function clear_dead_line_groups($transaction_id) {
    global $mysqli;
    $mysqli->query("
DELETE
FROM
    TransactionLineGroups
WHERE
    tlgTransactionID = $transaction_id
    AND NOT EXISTS (
        SELECT
            *
        FROM
            TransactionLines
        WHERE
            tlTransactionID = $transaction_id
            AND tlLineGroupID = tlgLineGroupID
    )");
}

function get_assembly_item($assembly_id, $item_id) {
    global $mysqli;
    $assembly_item = $mysqli->query("
SELECT
    iDescription,
    atDescription,
    aSerialNumber
FROM
    Items,
    Assemblies INNER JOIN
        AssemblyTypes
    ON aAssemblyTypeID = atAssemblyTypeID
WHERE
    iItemID = $item_id
    AND aAssemblyID = $assembly_id");
    if($assembly_item = $assembly_item->fetch_assoc()) {
        return $assembly_item;
    } else {
        return array();
    }
}

function aii_notice($settings, $people, $keys) {
    $assembly_item = get_assembly_item($settings['aii_assembly_id'], $settings['aii_item_id']);
    return('
            <div class="special_criteria">
                Only showing Transactions used to calculate the Qty Included of the ' .
        htmlspecialchars($assembly_item['iDescription']) . ' of the ' .
        htmlspecialchars($assembly_item['atDescription']) . ' (' .
        htmlspecialchars($assembly_item['aSerialNumber']) . ') owned by ' .
        htmlspecialchars($people[$settings['aii_person_id']]['full_name']) .
        ' <a href="transactions.php?transaction_id=' . $keys['transaction_id'] . '&line_id=' . $keys['line_id'] .
        '&line_group_id=' . $keys['line_group_id'] . get_params(remove_settings($settings, 'aii_assembly_id',
        'aii_person_id', 'aii_item_id'), false) . '">Remove</a>
                <input type="hidden" name="aii_assembly_id" value="' . $settings['aii_assembly_id'] . '" />
                <input type="hidden" name="aii_person_id" value="' . $settings['aii_person_id'] . '" />
                <input type="hidden" name="aii_item_id" value="' . $settings['aii_item_id'] . '" />
            </div>');
}

function fua_notice($settings, $transaction_types, $people, $keys) {
    $transaction_line = get_transaction_line($settings['fua_transaction_id'], $settings['fua_line_id']);
    return('
            <div class="special_criteria">
                Only showing Transactions used to calculate the From Unit Amount
of the ' . htmlspecialchars($transaction_types[$transaction_line['tTransactionTypeID']]['description']) .
' of ' . htmlspecialchars($transaction_line['iDescription']) . ' on ' . $transaction_line['tDate'] . ' from ' .
htmlspecialchars($people[$transaction_line['tFromPersonID']]['full_name']) . ' to ' .
htmlspecialchars($people[$transaction_line['tToPersonID']]['full_name']) .
' <a href="transactions.php?transaction_id=' . $keys['transaction_id'] . '&line_id=' . $keys['line_id'] .
'&line_group_id=' . $keys['line_group_id'] . get_params(remove_settings($settings, 'fua_transaction_id',
        'fua_line_id'), false) . '">Remove</a>
                <input type="hidden" name="fua_transaction_id" value="' . $settings['fua_transaction_id'] . '" />
                <input type="hidden" name="fua_line_id" value="' . $settings['fua_line_id'] . '" />
            </div>');
}

function qoh_notice($settings, $keys) {
    $item_description = get_item_description($settings['qoh_item_id']);
    return('
            <div class="special_criteria">Only showing Transactions used to calculate the Qty on Hand of the ' .
        htmlspecialchars($item_description) . ' <a href="transactions.php?transaction_id=' .
        $keys['transaction_id'] . '&line_id=' . $keys['line_id'] . '&line_group_id=' . $keys['line_group_id'] .
        get_params(remove_settings($settings, 'qoh_item_id'), false) . '">Remove</a>
            <input type="hidden" name="qoh_item_id" value="' . $settings['qoh_item_id'] . '" /></div>');
}

function get_transactions($transaction_type_id, $from_person_id, $to_person_id, $assembly_id, $special_criteria) {
    global $mysqli;
    if($transaction_type_id == 0) {
        $where = "";
    } else {
        $where = "     tTransactionTypeID = $transaction_type_id";
    }
    if($from_person_id <> 0) {
        $where = $where . " AND tFromPersonID = $from_person_id";
    }
    if($to_person_id <> 0) {
        $where = $where . " AND tToPersonID = $to_person_id";
    }
    if($assembly_id <> 0) {
        $where = $where . " AND paiAssemblyID = $assembly_id";
    }
    if($where <> '') {
        $where = "WHERE " . substr($where, 4);
    }
    $mysqli->query("
SET
    SQL_BIG_SELECTS = 1");
    make_transaction_assembly('gt');
    make_highest_item_count('gt');
    if(isset($special_criteria['fua_transaction_id'])) {
        make_responsible_transactions($special_criteria['fua_transaction_id'], $special_criteria['fua_line_id'], 'gt');
        return get_special_transactions($where, 'gt');
    } elseif(isset($special_criteria['aii_assembly_id'])) {
        make_assembly_item_included_transactions($special_criteria['aii_assembly_id'], $special_criteria['aii_item_id'],
            $special_criteria['aii_person_id'], 'gt');
        return get_special_transactions($where, 'gt');
    } elseif(isset($special_criteria['qoh_item_id'])) {
        make_qty_on_hand_transactions($special_criteria['qoh_item_id'], $special_criteria['user'], 'gt');
        return get_special_transactions($where, 'gt');
    } else {
        return get_all_transactions($where, 'gt');
    }
}

function get_assemblies() {
    global $mysqli;
    $assemblies = $mysqli->query("
SELECT
    aAssemblyID,
    aSerialNumber
FROM
    Assemblies
ORDER BY
    aSerialNumber");
    $response = array();
    $response["0"] = '';
    while($assembly = $assemblies->fetch_assoc()) {
        $response[$assembly['aAssemblyID']] = $assembly['aSerialNumber'];
    }
    return $response;
}

function get_people() {
    global $mysqli;
    $people = $mysqli->query("
SELECT DISTINCT
    pPersonID,
    pFirstName,
    pLastName,
    ft.tFromPersonID,
    tt.tToPersonID
FROM
    Transactions ft RIGHT JOIN (
        Transactions tt RIGHT JOIN
            People
        ON tt.tToPersonID = pPersonID
    ) ON ft.tFromPersonID = pPersonID
ORDER BY
    CONCAT(pFirstName, pLastName)");
    $response = array();
    while($person = $people->fetch_assoc()) {
        $response[$person['pPersonID']] = array(
            "full_name" => ltrim($person['pFirstName'] . ' ' . $person['pLastName']),
            "from" => ($person['tFromPersonID'] <> null),
            "to" => ($person['tToPersonID'] <> null));
    }
    return $response;
}

function get_first_item_line_id($transaction_id, $item_id) {
    global $mysqli;
    $first_line_id = $mysqli->query("
SELECT
    MIN(tlLineID) FirstItemLineID
FROM
    TransactionLines
WHERE
    tlTransactionID = $transaction_id
    AND tlItemID = $item_id");
    if($line_id = $first_line_id->fetch_assoc()) {
        return $line_id['FirstItemLineID'];
    } else {
        return 0;
    }
};

function get_transaction_lines($transaction_id, $description_filter, $group_filter, $assembly_filter) {
    global $mysqli;
    if($description_filter == '') {
        $filter = '';
    } else {
        $filter = "AND iDescription LIKE '%" . $mysqli->real_escape_string($description_filter) . "%'";
    }
    if($group_filter <> 0) {
        $filter .= "
    AND tlLineGroupID = $group_filter";
    }
    if($assembly_filter <> 0) {
        $filter .= "
    AND tlAssemblyID = $assembly_filter";
    }
    make_transaction_past_cost($transaction_id, 'tl');
    make_transaction_item_average_cost($transaction_id, 'tl');
    make_transaction_total_unit_amount($transaction_id, 'tl');
    make_transaction_line_from_amount($transaction_id, 'tlfa');
    return $mysqli->query("
SELECT
    tlLineID,
    tlItemID,
    iDescription,
    COALESCE(giUnitAmount, tlUnitAmount) tlUnitAmount,
    tlfaHasUnitAmount,
    tlfaUnitAmount,
    tlQuantity,
    COALESCE(tlAssemblyID, 0) tlAssemblyID,
    tlFromURL,
    tlLineGroupID
FROM
    (SELECT
        ipItemID giItemID,
        MAX(ipPercent * tlgTotalAmount) giUnitAmount
    FROM
        (SELECT
            tlItemID ipItemID,
            ttuaLineGroupID ipLineGroupID,
            tiacUnitAmount / ttuaUnitAmount ipPercent
        FROM
            shopcharttmp.tlTransactionTotalUnitAmount INNER JOIN (
                shopcharttmp.tlTransactionItemAverageCost INNER JOIN (
                    TransactionLines INNER JOIN
                        TransactionLineGroups
                    ON tlTransactionID = tlgTransactionID
                    AND tlLineGroupID = tlgLineGroupID
                ) ON tiacItemID = tlItemID
            ) ON ttuaLineGroupID = tlgLineGroupID
        WHERE
            tlgTransactionID = $transaction_id
        ) ip INNER JOIN
            TransactionLineGroups
        ON ipLineGroupID = tlgLineGroupID
    WHERE
        tlgTransactionID = $transaction_id
    GROUP BY
        ipItemID
    ) gi RIGHT JOIN (
        shopcharttmp.tlfaTransactionLineFromAmount RIGHT JOIN (
            TransactionLines INNER JOIN
                Items
            ON tlItemID = iItemID
        ) ON tlfaLineID = tlLineID
    ) ON giItemID = tlItemID
WHERE
    tlTransactionID = $transaction_id
    $filter
ORDER BY
    iDescription");
}