<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once 'fun/people.php';
require_once 'fun/common.php';
require_once 'util/person_flags.php';
define("PERSON_PAGE_SIZE", 18);
$default_keys = array(
    'person_id' => 0);
$keys = get_settings($default_keys);
$default_settings = array(
    'first_name_filter' => '',
    'last_name_filter' => '',
    'page_number' => 1,
    'p_page_number' => 1,
    'pPublic' => 0);
$settings = get_settings($default_settings);
if(isset($_REQUEST['person_id'])) {
    $keys['person_id'] = $_REQUEST['person_id'];
} else {
    $keys['person_id'] = 0;
}
if(isset($_POST['confirm_delete'])) {
    if(isset($_POST['new_from_person_id'])) {
        $new_from_person_id = $_POST['new_from_person_id'];
    } else {
        $new_from_person_id = 0;
    }
    if(isset($_POST['new_to_person_id'])) {
        $new_to_person_id = $_POST['new_to_person_id'];
    } else {
        $new_to_person_id = 0;
    }
    delete_person($keys['person_id'], $new_from_person_id, $new_to_person_id);
}
$referer = parse_url($_SERVER['HTTP_REFERER']);
if($referer['path'] == '/ShopChart/people.php') {
    if(isset($_POST['add_person'])) {
        $keys['person_id'] = add_person($_POST['new_first_name'], $_POST['new_last_name']);
    } elseif(isset($_POST['delete_person'])) {
        header('Location: delete.php?sender=people.php&person_id=' . $keys['person_id'] . '&first_name_filter=' .
            htmlspecialchars($settings['first_name_filter']) . '&last_name_filter=' . htmlspecialchars($settings['last_name_filter']));
    } elseif($_SERVER['REQUEST_METHOD'] == 'POST') {
        try {
            update_attributes($keys['person_id'], $_POST);
        } catch(Exception $e) {
            $error = $e->getMessage();
        }
    }
    $settings['page_number'] = get_page_number('page_number', 'previous_page', 'next_page');
}
$people = get_people($settings['first_name_filter'], $settings['last_name_filter']);
$max_pages = get_max_pages($people, PERSON_PAGE_SIZE);
$settings['page_number'] = go_to_page($keys['person_id'], 'pPersonID', $people, PERSON_PAGE_SIZE,
    $settings['page_number'], $max_pages, ($settings['p_page_number'] == $settings['page_number']));
print '
<!DOCTYPE html>
<html>
    <head>
        <style type="text/css">
            @import url(css/shop_chart.css);
            @import url(css/people.css);
        </style>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <title>People</title>
    </head>
    <body>        
        <form action="people.php" method="post">';
include 'top_nav.php';
print '
            <table id="people">
                <caption>People</caption>';
if($max_pages > 1) {
    print '
                <tr class="page_nav">
                    <td colspan="4">
                        <input type="submit" name="previous_page" value="Previous" />
                        <input type="text" name="page_number" value="' . $settings['page_number'] . '" /> of ' .
        $max_pages . '
                        <input type="submit" name="next_page" value="Next" />
                    </td>
                </tr>';
}
print '
                <tr class="field_header">
                    <th>First Name</th>
                    <th>Last Name</th>
                    <th>Hourly</th>
                    <th>Public</th>
                </tr>
                <tr class="field_header">
                    <th class="name">
                        <input type="text" name="first_name_filter" value="' .
    htmlspecialchars($settings['first_name_filter']) . '" />
                    </th>
                    <th class="name">
                        <input type="text" name="last_name_filter" value="' .
    htmlspecialchars($settings['last_name_filter']) . '" />
                    </th>
                    <th>Rate</th>
                    <th></th>
                </tr>';
$displayed = 0;
while ($person = $people->fetch_assoc()) {
    if($person['pPersonID'] <> null) {
        if ($person['pPersonID'] == $keys['person_id']) {
//            $person_locations = get_person_locations($keys['person_id']);
            $selected_person = $person;
            print '
                    <tr class="selected_row">
                        <td class="name">
                            <input type="text" name="pFirstName" value="' .
                htmlspecialchars($person['pFirstName']) . '" />
                        </td>
                        <td class="name">
                            <input type="text" name="pLastName" value="' .
                htmlspecialchars($person['pLastName']) . '" />
                        </td>
                        <td class="money">
                            <input type="text" name="pMonetaryPerSecond" value="' . money_format('%.2n',
                $person['pMonetaryPerSecond'] * 3600) . '" />
                        </td>
                        <td class="public">
                            <input type="checkbox" name="pPublic"' . ($person['pFlags'] & PF_PUBLIC ?
                ' checked="true"' : '') . ' onchange="this.form.submit();" />
                        </td>
                    </tr>';
        } else {
            if($displayed % 2) {
                print '
                    <tr class="odd_row">';
            } else {
                print '
                    <tr class="even_row">';
            }
            print '
                        <td class="name">
                            <a href="people.php?person_id=' . htmlspecialchars($person['pPersonID']) .
                get_params($settings, false) . '">' . htmlspecialchars($person['pFirstName']) . '</a>
                        </td>
                        <td class="name">
                            ' . htmlspecialchars($person['pLastName']) . '
                        </td>
                        <td class="money">
                            ' . money_format('%.2n', $person['pMonetaryPerSecond'] * 3600) . '
                        </td>
                        <td class="public">
                            <input type="checkbox"' . ($person['pFlags'] & PF_PUBLIC ? ' checked="true"' : '') .
                ' disabled="true" />
                    </tr>';
        }
        if(++$displayed >= PERSON_PAGE_SIZE) {
            break;
        }
    }
}
print '
                <tr class="footer">
                    <td class="name">
                        <input type="text" name="new_first_name" />
                    </td>
                    <td class="name">
                        <input type="text" name="new_last_name" />
                    </td>
                    <td class="money">
                        <input type="text" name="new_monetary_per_second" />
                    </td>
                    <td class="public">
                        <input type="submit" name="add_person" value="Add Person" />
                    </td>
                </tr>
            </table>';
//print '
//            <table border="1">
//                <tr style="text-align:center">
//                    <td colspan="0">Person Locations</td>
//                </tr>
//                <tr>
//                    <th>Street</th>
//                    <th>Street2</th>
//                    <th>City</th>
//                    <th>State</th>
//                    <th>Postal Code</th>
//                    <th>Primary</th>
//                </tr>';
//if(isset($person_locations)) {
//    while($person_location = $person_locations->fetch_assoc()) {
//        print '
//            <tr>
//                <td>' . $person_location['lStreet'] . '</td>
//                <td>' . $person_location['lStreet2'] . '</td>
//                <td>' . $person_location['lCity'] . '</td>
//                <td>' . $person_location['lState'] . '</td>
//                <td>' . $person_location['lPostalCode'] . '</td>
//                <td><input type="radio" name="plPrimary" value="' . $person_location['plLocationID'] . '"' .
//            ($person_location['plPrimary'] == 1 ? ' checked="true"' : '') . ' /></td>
//            </tr>';
//    }
//    print '
//                <tr>
//                    <td><input type="text" name="new_location_filter" /></td>
//                    <td colspan="0"><input type="submit" name="add_person_location" value="Add Location" /></td>
//                </tr>';
//} else {
//    print '
//                <tr>
//                    <td><input type="text" disabled="true" /></td>
//                    <td colspan="0"><input type="submit" value="Add Location" disabled="true" /></td>
//                </tr>';
//}
//print '
//            </table>';
if(isset($selected_person)) {
    print '
            <input type="hidden" name="person_id" value="' . $keys['person_id'] . '" />
            <input type="hidden" name="p_pFirstName" value="' . htmlspecialchars($selected_person['pFirstName'])
        . '" />
            <input type="hidden" name="p_pLastName" value="' . htmlspecialchars($selected_person['pLastName']) .
        '" />
            <input type="hidden" name="p_pMonetaryPerSecond" value="' .
        money_format('%.2n', $selected_person['pMonetaryPerSecond'] * 3600) . '" />';
    if($selected_person['pFlags'] & PF_PUBLIC) {
        print '
            <input type="hidden" name="p_pPublic" />';
    }
}
print '
        </form>
    </body>
</html>';

