<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/
require_once("util/transaction_type_flags.php");
define("TEMPORARY", "TEMPORARY");

function make_net_quantity($prefix) {
    global $mysqli;
    make_person_qty_to($prefix);
    make_person_qty_from($prefix);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "NetQuantity (
    nqAssemblyID int,
    nqDate datetime,
    nqQuantity decimal(11, 2),
    PRIMARY KEY (
        nqAssemblyID,
        nqDate
    )
)");
        $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "NetQuantity (
    nqAssemblyID,
    nqDate,
    nqQuantity
) SELECT
    pqtAssemblyID,
    pqtDate,
    MAX(pqtQuantity - COALESCE(pqfQuantity, 0))
FROM
    shopcharttmp." . $prefix . "PersonQtyFrom RIGHT JOIN
        shopcharttmp." . $prefix . "PersonQtyTo
    ON pqfPersonID = pqtPersonID
    AND pqfAssemblyID = pqtAssemblyID
GROUP BY
    pqtAssemblyID,
    pqtDate");
}

function make_assembly_best_quantity($prefix) {
    global $mysqli;
    make_person_quantity($prefix);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "AssemblyBestQuantity (
    abqAssemblyID int PRIMARY KEY,
    abqQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "AssemblyBestQuantity (
    abqAssemblyID,
    abqQuantity
) SELECT
    pqAssemblyID,
    MAX(pqQuantity)
FROM
    shopcharttmp." . $prefix . "PersonQuantity
GROUP BY
    pqAssemblyID");
}

function make_item_net_quantity($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ItemNetQuantity (
    inqItemID int PRIMARY KEY,
    inqQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ItemNetQuantity (
    inqItemID,
    inqQuantity
) SELECT
    iqtItemID,
    SUM(iqtQuantity - COALESCE(iqfQuantity, 0))
FROM
    shopcharttmp." . $prefix . "ItemQtyFrom RIGHT JOIN
        shopcharttmp." . $prefix . "ItemQtyTo
    ON iqfItemID = iqtItemID
GROUP BY
    iqtItemID");
}

/*
 * Average Unit Amount by ItemID and TransactionTypeID for all Grouped Items on ResponsibleTransactions. For 
 * calculating the relative value of Grouped Items on ResponsibleTransactions.
 */
function make_item_average_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ItemAverageCost (
    iacItemID int,
    iacMonetaryUnitAmount decimal(20, 6),
    iacTimeUnitAmount decimal(20, 6),
    iacTransactionTypeID int,
    PRIMARY KEY (
        iacItemID,
        iacTransactionTypeID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ItemAverageCost (
    iacItemID,
    iacTransactionTypeID,
    iacMonetaryUnitAmount,
    iacTimeUnitAmount
) SELECT
    tlItemID,
    rtTransactionTypeID,
    SUM(pcMonetaryExtAmount) / SUM(pcMonetaryQuantity),
    SUM(pcTimeExtAmount) / SUM(pcTimeQuantity)
FROM
    shopcharttmp." . $prefix . "PastCost INNER JOIN (
		TransactionLines INNER JOIN
			(SELECT DISTINCT
				rtTransactionID,
                rtTransactionTypeID,
				rtLineGroupID
			FROM
				shopcharttmp." . $prefix . "ResponsibleTransactions
			) rt
		ON tlTransactionID = rtTransactionID
		AND tlLineGroupID = rtLineGroupID
    ) ON pcItemID = tlItemID
    AND pcTransactionTypeID = rtTransactionTypeID
WHERE
    tlLineGroupID <> 0
GROUP BY
    tlItemID,
    rtTransactionTypeID");
}

/*
 * Total Unit Amount by TransactionID and LineGroupID for all Grouped Items on ResponsibleTransactions. For
 * comparing against the individual Unit Amounts of each Item in each Group to find their global relative value.
 */
function make_total_unit_amount($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "TotalUnitAmount (
    tuaMonetaryUnitAmount decimal(20, 6),
    tuaLineGroupID int,
    tuaTimeUnitAmount decimal(20, 6),
	tuaTransactionID int,
	PRIMARY KEY (
		tuaTransactionID,
		tuaLineGroupID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TotalUnitAmount (
	tuaTransactionID,
    tuaLineGroupID,
    tuaMonetaryUnitAmount,
    tuaTimeUnitAmount
) SELECT
	tlTransactionID,
    tlLineGroupID,
    SUM(iacMonetaryUnitAmount * tlQuantity),
    SUM(iacTimeUnitAmount * tlQuantity)
FROM
    shopcharttmp." . $prefix . "ItemAverageCost INNER JOIN (
		TransactionLines INNER JOIN
			(SELECT DISTINCT
				rtTransactionID,
                rtTransactionTypeID
			FROM
				shopcharttmp." . $prefix . "ResponsibleTransactions
			) rt
		ON tlTransactionID = rtTransactionID
    ) ON iacItemID = tlItemID
    AND iacTransactionTypeID = rtTransactionTypeID
GROUP BY
	tlTransactionID,
    tlLineGroupID");
}


function make_item_percent($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ItemPercent (
	ipItemID int,
	ipLineGroupID int,
	ipMonetaryPercent decimal(20, 6),
    ipTimePercent decimal(20, 6),
	ipTransactionID int,
	PRIMARY KEY (
		ipTransactionID,
		ipLineGroupID,
		ipItemID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ItemPercent (
    ipTransactionID,
    ipLineGroupID,
    ipItemID,
    ipMonetaryPercent,
    ipTimePercent
) SELECT
    tlTransactionID,
    tuaLineGroupID,
    tlItemID,
    AVG(iacMonetaryUnitAmount / tuaMonetaryUnitAmount),
    AVG(iacTimeUnitAmount / tuaTimeUnitAmount)
FROM
    shopcharttmp." . $prefix . "TotalUnitAmount INNER JOIN (
        shopcharttmp." . $prefix . "ItemAverageCost INNER JOIN (
            TransactionLines INNER JOIN (
                TransactionLineGroups INNER JOIN
                    (SELECT DISTINCT
                        rtTransactionID,
                        rtTransactionTypeID
                    FROM
                        shopcharttmp." . $prefix . "ResponsibleTransactions
                    ) rt
                ON tlgTransactionID = rtTransactionID
            ) ON tlTransactionID = tlgTransactionID
            AND tlLineGroupID = tlgLineGroupID
        ) ON iacItemID = tlItemID
        AND iacTransactionTypeID = rtTransactionTypeID
    ) ON tuaTransactionID = rtTransactionID
    AND tuaLineGroupID = tlgLineGroupID
GROUP BY
    tlTransactionID,
    tuaLineGroupID,
    tlItemID");
}

function make_grouped_items($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "GroupedItems (
	giItemID int,
	giLineGroupID int,
	giTransactionID int,
	giUnitAmount decimal(20, 6),
	PRIMARY KEY (
		giTransactionID,
		giLineGroupID,
		giItemID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "GroupedItems (
	giTransactionID,
	giLineGroupID,
	giItemID,
	giUnitAmount
) SELECT
	ipTransactionID,
	ipLineGroupID,
	ipItemID,
    CASE
        WHEN rtFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN ipTimePercent
        ELSE ipMonetaryPercent
    END * tlgTotalAmount
FROM
	shopcharttmp." . $prefix . "ItemPercent INNER JOIN (
		TransactionLineGroups INNER JOIN
			(SELECT DISTINCT
				rtTransactionID,
                rtFlags
			FROM
				shopcharttmp." . $prefix . "ResponsibleTransactions
			) rt
		ON tlgTransactionID = rtTransactionID
	) ON ipTransactionID = rtTransactionID
	AND ipLineGroupID = tlgLineGroupID");
}

function make_transaction_item_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "TransactionItemCost (
    ticAssemblyID int,
    ticFlags int,
    ticFromPersonID int,
    ticItemID int,
    ticQuantity decimal(11, 2),
    ticTransactionID int,
    ticUnitAmount decimal(20, 6),
    PRIMARY KEY (
        ticTransactionID,
        ticAssemblyID,
        ticItemID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TransactionItemCost (
    ticTransactionID,
    ticFromPersonID,
    ticFlags,
    ticAssemblyID,
    ticItemID,
    ticUnitAmount,
    ticQuantity
) SELECT
    rtTransactionID,
    rtFromPersonID,
    rtFlags,
    rtAssemblyID,
    rtItemID,
    AVG(COALESCE(giUnitAmount, ugiUnitAmount, 0)),
    SUM(rtQuantity)
FROM
    shopcharttmp." . $prefix . "GroupedItems RIGHT JOIN (
        (SELECT
            tlTransactionID ugiTransactionID,
            tlItemID ugiItemID,
            AVG(tlUnitAmount) ugiUnitAmount
        FROM
            TransactionLines
        WHERE
            tlLineGroupID IS NULL
        GROUP BY
            tlTransactionID,
            tlItemID
        ) ugi RIGHT JOIN
            shopcharttmp." . $prefix . "ResponsibleTransactions
        ON ugiTransactionID = rtTransactionID
        AND ugiItemID = rtItemID
    ) ON giTransactionID = rtTransactionID
    AND giLineGroupID = rtLineGroupID
    AND giItemID = rtItemID
GROUP BY
    rtTransactionID,
    rtFromPersonID,
    rtFlags,
    rtItemID");
}

function make_directly_responsible($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "DirectlyResponsible (
	drItemID int PRIMARY KEY,
	drMonetaryTotalAmount decimal(20, 6),
	drQuantity decimal(11, 2),
    drTimeTotalAmount decimal(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "DirectlyResponsible (
	drItemID,
	drQuantity,
	drMonetaryTotalAmount,
    drTimeTotalAmount
) SELECT
	ticItemID,
	SUM(((ticFlags & " . TTF_TRANSFERS_INVENTORY . ") <> 0) * ticQuantity),
    SUM(((ticFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") = 0) * ticUnitAmount * ticQuantity),
    SUM(((ticFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") <> 0) * ticUnitAmount * ticQuantity)
FROM
	shopcharttmp." . $prefix . "TransactionItemCost INNER JOIN
		People
	ON ticFromPersonID = pPersonID
WHERE
	NOT ticFlags & " . TTF_GENERAL_INVENTORY . "
GROUP BY
	ticItemID");
}

//function make_general_invy_cost($prefix) {
//    global $mysqli;
//    $mysqli->query("
//CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "GeneralInvyCost (
//	gicItemID int PRIMARY KEY,
//	gicMonetaryUnitAmount decimal(20, 6),
//    gicTimeUnitAmount decimal(20, 6)
//)");
//    $mysqli->query("
//INSERT INTO shopcharttmp." . $prefix . "GeneralInvyCost (
//	gicItemID,
//	gicMonetaryUnitAmount,
//    gicTimeUnitAmount
//) SELECT
//	ticItemID,
//    SUM(NOT(ticFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") * ticUnitAmount * ticQuantity),
//    SUM((ticFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") * ticUnitAmount * ticQuantity)
//FROM
//	shopcharttmp." . $prefix . "TransactionItemCost INNER JOIN
//		People
//	ON ticFromPersonID = pPersonID
//WHERE
//	ticFlags & " . TTF_GENERAL_INVENTORY . "
//GROUP BY
//	ticItemID");
//}

/*Current Monetary and Time values for the specified Person for items currently included in the specified 
 * Person's specified Assembly.
 */
function make_assembly_item_from_amount($assembly_id, $person_id, $prefix) {
    global $mysqli;
    make_responsible_transactions($assembly_id, $person_id, $prefix);
    make_past_cost($prefix);
    make_item_average_cost($prefix);
    make_total_unit_amount($prefix);
    make_item_percent($prefix);
    make_grouped_items($prefix);
    make_transaction_item_cost($prefix);
    make_directly_responsible($prefix);
//    make_general_invy_cost($prefix);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "AssemblyItemFromAmount (
    aifaHasHistory bit,
    aifaItemID int PRIMARY KEY,
    aifaMonetaryTotalAmount decimal(20, 6),
    aifaQuantity decimal(11, 2),
    aifaTimeTotalAmount decimal(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "AssemblyItemFromAmount (
    aifaItemID,
    aifaQuantity,
    aifaHasHistory,
    aifaMonetaryTotalAmount,
    aifaTimeTotalAmount
) SELECT
    caiItemID,
    caiQuantity,
    drItemID IS NOT NULL,
    drMonetaryTotalAmount,
    drTimeTotalAmount
FROM
    shopcharttmp." . $prefix . "DirectlyResponsible RIGHT JOIN
        shopcharttmp." . $prefix . "CurrentAssemblyItems
    ON drItemID = caiItemID");
}

/*All Transactions used in calculating the specified Person's From Amount of the Items in the specified Assembly.
Includes amounts directly related to the Assembly and amounts used to calculate the value of general inventory
that was used.*/
function make_responsible_transactions($assembly_id, $person_id, $prefix) {
    global $mysqli;
    make_current_assembly_items($assembly_id, $person_id, $prefix);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ResponsibleTransactions (
    rtAssemblyID int,
    rtCurrentFlags int,
    rtDate datetime,
    rtFlags int,
    rtFromPersonID int,
    rtItemID int,
    rtLineGroupID int,
    rtQuantity decimal(11, 2),
    rtTransactionID int,
    rtTransactionTypeID int,
    PRIMARY KEY (
            rtTransactionID,
            rtItemID,
            rtLineGroupID,
    rtAssemblyID
    )
)");
    add_assy_transactions($assembly_id, $person_id, $prefix);
//    make_non_assy_items($prefix);
//    add_non_assy_transactions($person_id, $prefix);
//    make_salvage_items($person_id, $prefix);
//    make_salvage_out_transactions($prefix);
//    make_salvage_reference_transactions($prefix);
//    make_salvage_past_transactions($prefix);
//    add_salvage_transactions($prefix);
}

/*All Items currently included in the specified Person's specified Assembly. The date is for calculating the
Items' current values later.
 */
function make_current_assembly_items($assembly_id, $person_id, $prefix) {
    global $mysqli;
    make_item_qty_to($person_id, $assembly_id, $prefix);
    make_item_qty_from($person_id, $assembly_id, $prefix);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "CurrentAssemblyItems (
	caiDate datetime,
	caiItemID int PRIMARY KEY,
	caiQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "CurrentAssemblyItems (
	caiDate,
	caiItemID,
	caiQuantity
) SELECT
    NOW(),
    iqtItemID,
    SUM(iqtQuantity - COALESCE(iqfQuantity, 0))
FROM
    shopcharttmp." . $prefix . "ItemQtyFrom RIGHT JOIN
        shopcharttmp." . $prefix . "ItemQtyTo
    ON iqfItemID = iqtItemID
GROUP BY
    iqtItemID");
}

function add_assy_transactions($assembly_id, $person_id, $prefix) {
    global $mysqli;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtTransactionID,
	rtTransactionTypeID,
	rtFromPersonID,
	rtDate,
	rtFlags,
	rtCurrentFlags,
	rtItemID,
	rtLineGroupID,
    rtAssemblyID,
	rtQuantity
) SELECT
    tTransactionID,
    tTransactionTypeID,
    tFromPersonID,
    tDate,
    ttFlags,
    0,
    tlItemID,
    COALESCE(tlLineGroupID, 0),
    $assembly_id,
    SUM(tlQuantity)
FROM
        shopcharttmp." . $prefix . "CurrentAssemblyItems INNER JOIN (
            TransactionLines INNER JOIN (
                Transactions INNER JOIN
                    TransactionTypes
                ON tTransactionTypeID = ttTransactionTypeID
            ) ON tlTransactionID = tTransactionID
        ) ON caiItemID = tlItemID
WHERE
    tToPersonID = $person_id
    AND tlAssemblyID = $assembly_id
    AND NOT ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
    AND tDate < caiDate
    AND (
        NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
        OR tFromPersonID <> tToPersonID
    )
GROUP BY
    tTransactionID,
    tTransactionTypeID,
    tFromPersonID,
    tDate,
    ttFlags,
    tlItemID,
    tlLineGroupID");
}

function make_non_assy_items($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "NonAssyItems (
	naiDate datetime,
	naiItemID int PRIMARY KEY,
	naiQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "NonAssyItems (
	naiDate,
	naiItemID,
	naiQuantity
) SELECT
	NOW(),
	caiItemID,
	SUM(caiQuantity - COALESCE(alqQuantity, 0))
FROM
	(SELECT
		rtItemID alqItemID,
		SUM(rtQtyUsed) alqQuantity
	FROM
		shopcharttmp." . $prefix . "ResponsibleTransactions
	WHERE
		rtFlags & " . TTF_TRANSFERS_INVENTORY . "
	GROUP BY
		rtItemID
	) alq RIGHT JOIN
        shopcharttmp." . $prefix . "CurrentAssemblyItems
    ON alqItemID = caiItemID
GROUP BY
	iaqItemID
HAVING
	SUM(caiQuantity - COALESCE(alqQuantity, 0)) > 0");
}

function add_non_assy_transactions($person_id, $prefix) {
    global $mysqli;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtTransactionID,
	rtTransactionTypeID,
	rtFromPersonID,
	rtDate,
	rtFlags,
	rtCurrentFlags,
	rtItemID,
	rtLineGroupID,
    rtAssemblyID,
	rtQtyUsed
) SELECT
	tTransactionID,
	tTransactionTypeID,
	tFromPersonID,
	tDate,
	ttFlags | " . TTF_GENERAL_INVENTORY . ",
	tlItemID,
	COALESCE(tlLineGroupID, 0),
	COALESCE(tlAssemblyID, 0),
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "NonAssyItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON naiItemID = tlItemID
WHERE
	tDate < naiDate
	AND NOT ttFlags & " . TTF_REMOVES_INVENTORY . "
	AND tToPersonID = $person_id
	AND (
		tlAssemblyID IS NULL
		OR ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
	)
GROUP BY
	tTransactionID,
	tTransactionTypeID,
	tFromPersonID,
	tDate,
	ttFlags | " . TTF_GENERAL_INVENTORY . ",
	tlItemID,
	COALESCE(tlLineGroupID, 0),
	COALESCE(tlAssemblyID, 0)");
}

function make_salvage_items($person_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "SalvageItems (
	siAssemblyID int,
	siDate datetime,
	siFromPersonID int,
	siItemID int,
	siQuantity decimal(11, 2),
	siTransactionID int,
	PRIMARY KEY (
		siTransactionID,
		siItemID,
		siAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvageItems (
	siTransactionID,
	siDate,
	siFromPersonID,
	siItemID,
	siAssemblyID,
	siQuantity
) SELECT
	tTransactionID,
	tDate,
	tFromPersonID,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "NonAssyItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON naiItemID = tlItemID
WHERE
    tToPersonID = $person_id
	AND ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
	AND tlAssemblyID IS NOT NULL
GROUP BY
	tTransactionID,
	tDate,
	tFromPersonID,
	tlItemID,
	tlAssemblyID");
}

function make_salvage_out_transactions($prefix) {
    global $mysqli;
    $mysqli->query ("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "SalvageOutTransactions (
	sotAssemblyID int,
	sotDate datetime,
	sotItemID int,
	sotQuantity decimal(11, 2),
	sotTransactionID int,
	PRIMARY KEY (
		sotTransactionID,
		sotItemID,
		sotAssemblyID
	)
)") ;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvageOutTransactions (
	sotTransactionID,
	sotDate,
	sotItemID,
	sotAssemblyID,
	sotQuantity
) SELECT
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON siFromPersonID = tFromPersonID
	AND siItemID = tlItemID
	AND siAssemblyID = tlAssemblyID
WHERE
	(
		tDate < siDate
		OR (
			tDate = siDate
			AND tTransactionID < siTransactionID
		)
	) AND (
		(
			ttFlags & " . TTF_TRANSFERS_INVENTORY . "
			AND tFromPersonID <> tToPersonID
		) OR ttFlags & " . TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
	)
GROUP BY
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID");
}

function make_salvage_reference_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "SalvageReferenceTransactions (
	srtDate datetime,
	srtTransactionID int PRIMARY KEY,
	INDEX srtiDate (
		srtDate
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvageReferenceTransactions (
	srtTransactionID,
	srtDate
) SELECT DISTINCT
	tTransactionID,
	tDate
FROM
	shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
		TransactionLines INNER JOIN
			Transactions
		ON tlTransactionID = tTransactionID
	) ON siItemID = tlItemID
	AND siAssemblyID = tlAssemblyID
WHERE
	tFromPersonID = siFromPersonID
	OR tToPersonID = siFromPersonID");
}

function make_salvage_past_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "SalvagePastTransactions (
	sptAssemblyID int,
	sptItemID int,
	sptPersonID int,
	sptQuantity decimal(11, 2),
	sptTransactionID int,
	PRIMARY KEY (
		sptTransactionID,
		sptPersonID,
		sptItemID,
		sptAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvagePastTransactions (
	sptTransactionID,
	sptPersonID,
	sptItemID,
	sptAssemblyID,
	sptQuantity
) SELECT
	srtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID,
	SUM(
		CASE
			WHEN srtDate > tDate
				OR (
					srtDate = tDate
					AND srtTransactionID > tTransactionID
				) THEN CASE
				WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tlQuantity
				ELSE -tlQuantity
			END
			ELSE 0
		END
	)
FROM
	shopcharttmp." . $prefix . "SalvageReferenceTransactions,
	shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON siItemID = tlItemID
	AND siAssemblyID = tlAssemblyID
WHERE
	ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY |
        TTF_ADDS_INVENTORY_TO_ASSEMBLY) . "
	AND tDate <= srtDate
	AND (
		tToPersonID = siFromPersonID
		OR tFromPersonID = siFromPersonID
	)
GROUP BY
	srtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID");
}

function add_salvage_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtTransactionID,
	rtTransactionTypeID,
	rtFromPersonID,
	rtDate,
	rtFlags,
	rtCurrentFlags,
	rtItemID,
	rtLineGroupID,
    rtAssemblyID,
	rtQtyUsed
) SELECT
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	COALESCE(itLineGroupID, 0),
    itAssemblyID,
	CASE
		WHEN itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN
			CASE
				WHEN itRunningTotal < itQtyReqd THEN itQuantity
				ELSE itQtyReqd - (itRunningTotal - itQuantity)
			END - SUM(
				CASE
					WHEN sotDate > itDate OR (sotDate = itDate AND sotTransactionID > itTransactionID) THEN sotQuantity
					ELSE 0
				END
			)
		ELSE 
			CASE
				WHEN itQuantity < itQtyReqd THEN itQuantity
				ELSE itQtyReqd
			END
	END
FROM
	shopcharttmp." . $prefix . "SalvageOutTransactions RIGHT JOIN
		(SELECT
			tTransactionID itTransactionID,
			tTransactionTypeID itTransactionTypeID,
			tFromPersonID itFromPersonID,
			tDate itDate,
			ttFlags itFlags,
			siFlags itCurrentFlags,
			tlItemID itItemID,
			tlLineGroupID itLineGroupID,
			tlAssemblyID itAssemblyID,
			siQuantity itQtyReqd,
			SUM(tlQuantity) itQuantity,
			SUM(tlQuantity) + sptQuantity itRunningTotal
		FROM
			shopcharttmp." . $prefix . "SalvagePastTransactions RIGHT JOIN (
				shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
					TransactionLines INNER JOIN (
						Transactions INNER JOIN
							TransactionTypes
						ON tTransactionTypeID = ttTransactionTypeID
					) ON tlTransactionID = tTransactionID
				) ON siFromPersonID = tToPersonID
				AND siItemID = tlItemID
				AND siAssemblyID = tlAssemblyID
			) ON sptTransactionID = tTransactionID
			AND sptPersonID = tToPersonID
			AND sptItemID = tlItemID
			AND sptAssemblyID = tlAssemblyID
		WHERE
			NOT ttFlags & " . (TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY) . "
			AND (
				tDate < siDate
				OR (
					tDate = siDate
					AND tTransactionID < siTransactionID
				)
			) AND (
				(
					NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
					OR tFromPersonID <> tToPersonID
				)
			)
		GROUP BY
			tTransactionID,
			tTransactionTypeID,
			tDate,
			ttFlags,
			siFlags,
			tlItemID,
			tlLineGroupID,
            tlAssemblyID,
			siQuantity,
			sptQuantity
		) it
	ON sotItemID = itItemID
	AND sotAssemblyID = itAssemblyID
GROUP BY
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	COALESCE(itLineGroupID, 0),
	itAssemblyID,
	itQtyReqd,
	itQuantity,
	itRunningTotal
HAVING
	itRunningTotal - itQuantity - SUM(
		CASE
			WHEN sotDate > itDate 
				OR (
					sotDate = itDate
					AND sotTransactionID > itTransactionID
				) THEN sotQuantity
			ELSE 0
	END) < itQtyReqd
	OR NOT itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY));
}

/*Total Quantity and ExtAmount by ItemID and TransactionTypeID but with duplicating keys. For generating the
 * relative value of any items that may be assigned to a Line Group in a Transaction for which we're finding 
 * Unit Amounts.
 */
function make_past_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "PastCost (
    pcItemID int,
    pcMonetaryExtAmount decimal(20, 6),
    pcMonetaryQuantity decimal(11, 2),
    pcTimeExtAmount decimal(20, 6),
    pcTimeQuantity decimal(11, 2),
	pcTransactionTypeID int
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PastCost (
    pcTransactionTypeID,
    pcItemID,
    pcMonetaryExtAmount,
    pcMonetaryQuantity,
    pcTimeExtAmount,
    pcTimeQuantity
) SELECT
    tTransactionTypeID,
    ptl.tlItemID,
    SUM((NOT(ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ")) * ptl.tlUnitAmount * ptl.tlQuantity),
    SUM((NOT(ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ")) * ptl.tlQuantity),
    SUM((ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") *  ptl.tlUnitAmount * ptl.tlQuantity),
    SUM((ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") * ptl.tlQuantity)
FROM
	TransactionLines ctl INNER JOIN (
		(SELECT DISTINCT
			rtTransactionID,
			rtTransactionTypeID
		FROM
			shopcharttmp." . $prefix . "ResponsibleTransactions
		) rt INNER JOIN (
			TransactionLines ptl INNER JOIN (
                Transactions INNER JOIN
                    TransactionTypes
                ON tTransactionTypeID = ttTransactionTypeID
			) ON ptl.tlTransactionID = tTransactionID
		) ON rtTransactionTypeID = tTransactionTypeID
	) ON ctl.tlTransactionID = rtTransactionID
	AND ctl.tlItemID = ptl.tlItemID
WHERE
	ptl.tlLineGroupID IS NULL
GROUP BY
	tTransactionID,
	ptl.tlItemID");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PastCost (
	pcTransactionTypeID,
    pcItemID,
    pcMonetaryExtAmount,
    pcMonetaryQuantity,
    pcTimeExtAmount,
    pcTimeQuantity
) SELECT
	tTransactionTypeID,
	tlItemID,
    SUM((NOT(ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ")) * (lgcTotalAmount / lgcLineGroupCount) * tlQuantity),
    SUM((NOT(ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ")) * tlQuantity),
    SUM((ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") * (lgcTotalAmount / lgcLineGroupCount) * tlQuantity),
    SUM((ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") * tlQuantity)
FROM
	(SELECT
		tlgTransactionID lgcTransactionID,
		tlgLineGroupID lgcLineGroupID,
		tlgTotalAmount lgcTotalAmount,
		SUM(tlQuantity) lgcLineGroupCount
	FROM
		TransactionLineGroups INNER JOIN
			TransactionLines
		ON tlgTransactionID = tlTransactionID
		AND tlgLineGroupID = tlLineGroupID
	GROUP BY
		tlgTransactionID,
		tlgLineGroupID,
		tlgTotalAmount
	) lgc INNER JOIN (
		TransactionLines INNER JOIN (
            TransactionTypes INNER JOIN (
                Transactions INNER JOIN
                    (SELECT DISTINCT
                        rtTransactionTypeID,
                        rtCurrentFlags
                    FROM
                        shopcharttmp." . $prefix . "ResponsibleTransactions
                    ) rt
                ON tTransactionTypeID = rtTransactionTypeID
            ) ON ttTransactionTypeID = tTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON lgcTransactionID = tlTransactionID
	AND lgcLineGroupID = tlLineGroupID
GROUP BY
	tTransactionTypeID,
	tlItemID");
}

function make_person_assemblies($person_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "PersonAssemblies (
    paAssemblyID int PRIMARY KEY
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PersonAssemblies (
    paAssemblyID
) SELECT
    pqAssemblyID
FROM
    shopcharttmp." . $prefix . "PersonQuantity
WHERE
    pqPersonID = $person_id");
}

function make_person_quantity($prefix) {
    global $mysqli;
    make_person_qty_to($prefix);
    make_person_qty_from($prefix);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "PersonQuantity (
    pqAssemblyID int,
    pqPersonID int,
    pqQuantity decimal(11, 2),
    PRIMARY KEY (
        pqAssemblyID,
        pqPersonID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PersonQuantity (
    pqAssemblyID,
    pqPersonID,
    pqQuantity
) SELECT
    pqtAssemblyID,
    pqtPersonID,
    SUM(pqtQuantity - COALESCE(pqfQuantity, 0))
FROM
    shopcharttmp." . $prefix . "PersonQtyFrom RIGHT JOIN
        shopcharttmp." . $prefix . "PersonQtyTo
    ON pqfAssemblyID = pqtAssemblyID
    AND pqfPersonID = pqtPersonID
GROUP BY
    pqtAssemblyID,
    pqtPersonID");
}

function make_item_qty_to($person_id, $assembly_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ItemQtyTo (
    iqtItemID int PRIMARY KEY,
    iqtQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ItemQtyTo (
    iqtItemID,
    iqtQuantity
) SELECT
    tlItemID,
    SUM(CASE
        WHEN ttFlags & " . (TTF_ADDS_INVENTORY_TO_ASSEMBLY | TTF_TRANSFERS_INVENTORY) . " THEN tlQuantity
        ELSE -tlQuantity
    END)
FROM
    TransactionLines INNER JOIN (
        Transactions INNER JOIN
            TransactionTypes
        ON tTransactionTypeID = ttTransactionTypeID
    ) ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tToPersonID = $person_id
    AND tlAssemblyID = $assembly_id
    AND ttFlags & " . (TTF_ADDS_INVENTORY_TO_ASSEMBLY | TTF_REMOVES_INVENTORY | 
        TTF_REMOVES_INVENTORY_FROM_ASSEMBLY | TTF_TRANSFERS_INVENTORY) . "
    AND (
        NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
        OR tFromPersonID <> tToPersonID
    )
GROUP BY
    tlItemID");
}

function make_item_qty_from($person_id, $assembly_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ItemQtyFrom (
    iqfItemID int PRIMARY KEY,
    iqfQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ItemQtyFrom (
    iqfItemID,
    iqfQuantity
) SELECT
    tlItemID,
    SUM(tlQuantity)
FROM
    TransactionLines INNER JOIN
        Transactions
    ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tFromPersonID = $person_id
    AND tFromPersonID <> tToPersonID
    AND tlAssemblyID = $assembly_id
GROUP BY
    tlItemID");
}

function make_person_qty_to($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "PersonQtyTo (
    pqtAssemblyID int,
    pqtDate datetime,
    pqtPersonID int,
    pqtQuantity decimal(11, 2),
    PRIMARY KEY (
        pqtAssemblyID,
        pqtPersonID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PersonQtyTo (
    pqtAssemblyID,
    pqtDate,
    pqtPersonID,
    pqtQuantity
) SELECT
    tlAssemblyID,
    MAX(tDate),
    tToPersonID,
    SUM(CASE
        WHEN ttFlags & " . (TTF_ADDS_INVENTORY_TO_ASSEMBLY | TTF_TRANSFERS_INVENTORY) . " THEN tlQuantity
        ELSE -tlQuantity
    END)
FROM
    TransactionLines INNER JOIN (
        Transactions INNER JOIN
            TransactionTypes
        ON tTransactionTypeID = ttTransactionTypeID
    ) ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tlAssemblyID IS NOT NULL
    AND (
        NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
        OR tFromPersonID <> tToPersonID
    )
GROUP BY
    tlAssemblyID,
    tToPersonID");
}

function make_person_qty_from($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "PersonQtyFrom (
    pqfAssemblyID int,
    pqfPersonID int,
    pqfQuantity decimal(11, 2),
    PRIMARY KEY (
        pqfAssemblyID,
        pqfPersonID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PersonQtyFrom (
    pqfAssemblyID,
    pqfPersonID,
    pqfQuantity
) SELECT
    tlAssemblyID,
    tFromPersonID,
    SUM(tlQuantity)
FROM
    TransactionLines INNER JOIN
        Transactions
    ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tFromPersonID <> tToPersonID
    AND tlAssemblyID IS NOT NULL
GROUP BY
    tlAssemblyID,
    tFromPersonID");
}

function make_eligible_assemblies($item_id, $person_id) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.EligibleAssemblies (
    eaAssemblyID int PRIMARY KEY
)");
    make_item_assemblies($item_id, $person_id);
    $mysqli->query("
INSERT INTO shopcharttmp.EligibleAssemblies (
    eaAssemblyID
) SELECT
    iiuAssemblyID
FROM
    (SELECT
        iaAssemblyID iiuAssemblyID,
        iaQuantity iiuQuantityInUse,
        0 iiuQuantityNeeded
    FROM
        shopcharttmp.ItemAssemblies
    UNION
    SELECT
        aAssemblyID,
        0,
        atiQuantity
    FROM
        Assemblies INNER JOIN
            AssemblyTypeItems
        ON aAssemblyTypeID = atiAssemblyTypeID
    WHERE
        aFlags & " . AF_WORK_IN_PROGRESS . "
        AND atiItemID = $item_id
    ) ai
GROUP BY
    iiuAssemblyID
HAVING
    MAX(iiuQuantityNeeded) > MAX(COALESCE(iiuQuantityInUse, 0))");
}

function make_item_assemblies($item_id, $person_id) {
    global $mysqli;
    make_assembly_qty_to($item_id, $person_id);
    make_assembly_qty_from($item_id, $person_id);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.ItemAssemblies (
	iaAssemblyID int PRIMARY KEY,
	iaQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp.ItemAssemblies (
    iaAssemblyID,
	iaQuantity
) SELECT
    aqtAssemblyID,
    SUM(aqtQuantity - COALESCE(aqfQuantity, 0))
FROM
    shopcharttmp.AssemblyQtyFrom RIGHT JOIN (
        shopcharttmp.AssemblyQtyTo INNER JOIN
            Assemblies
        ON aqtAssemblyID = aAssemblyID
    ) ON aqfAssemblyID = aqtAssemblyID
WHERE
    aFlags & " . AF_WORK_IN_PROGRESS . "
GROUP BY
    aqtAssemblyID");
}

function make_assembly_qty_to($item_id, $person_id) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.AssemblyQtyTo (
    aqtAssemblyID int PRIMARY KEY,
    aqtQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp.AssemblyQtyTo (
    aqtAssemblyID,
    aqtQuantity
) SELECT
    tlAssemblyID,
    SUM(CASE
        WHEN ttFlags & " . (TTF_ADDS_INVENTORY_TO_ASSEMBLY | TTF_TRANSFERS_INVENTORY) . " THEN tlQuantity
        ELSE -tlQuantity
    END)
FROM
    TransactionLines INNER JOIN (
        Transactions INNER JOIN
            TransactionTypes
        ON tTransactionTypeID = ttTransactionTypeID
    ) ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tlItemID = $item_id
    AND tToPersonID = $person_id
    AND ttFlags & " . (TTF_ADDS_INVENTORY_TO_ASSEMBLY | TTF_REMOVES_INVENTORY | 
        TTF_REMOVES_INVENTORY_FROM_ASSEMBLY | TTF_TRANSFERS_INVENTORY) . "
    AND (
        NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
        OR tFromPersonID <> tToPersonID
    )
GROUP BY
    tlAssemblyID");
}

function make_assembly_qty_from($item_id, $person_id) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.AssemblyQtyFrom (
    aqfAssemblyID int PRIMARY KEY,
    aqfQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp.AssemblyQtyFrom (
    aqfAssemblyID,
    aqfQuantity
) SELECT
    tlAssemblyID,
    SUM(tlQuantity)
FROM
    TransactionLines INNER JOIN
        Transactions
    ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tlItemID = $item_id
    AND tFromPersonID = $person_id
    AND tFromPersonID <> tToPersonID
GROUP BY
    tlAssemblyID");
}