<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/
require_once("util/transaction_type_flags.php");
require_once("util/assembly_flags.php");
define("TEMPORARY", "TEMPORARY");

function make_item_percent($item_id, $where) {
    global $mysqli;
    make_last_transaction($item_id, $where);
    make_ungrouped_items();
    make_past_cost();
    make_item_average_cost();
    make_total_unit_amount();
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp.ItemPercent (
    ipFromPersonID int,
    ipLineGroupID int,
    ipPercent decimal(20, 6),
    ipTransactionID int,
    ipTransactionTypeID int,
    PRIMARY KEY (
        ipTransactionID,
        ipLineGroupID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.ItemPercent (
    ipTransactionID,
    ipFromPersonID,
    ipTransactionTypeID,
    ipLineGroupID,
    ipPercent
) SELECT
    ltTransactionID,
    ltFromPersonID,
    ltTransactionTypeID,
    tuaLineGroupID,
    iacUnitAmount / tuaUnitAmount
FROM
    shopcharttmp.TotalUnitAmount INNER JOIN (
        shopcharttmp.ItemAverageCost INNER JOIN (
            TransactionLines INNER JOIN (
                TransactionLineGroups INNER JOIN
                    shopcharttmp.LastTransaction
                ON tlgTransactionID = ltTransactionID
            )ON tlTransactionID = tlgTransactionID
            AND tlLineGroupID = tlgLineGroupID
        ) ON iacFromPersonID = ltFromPersonID
        AND iacTransactionTypeID = ltTransactionTypeID
        AND iacItemID = tlItemID
    ) ON tuaFromPersonID = ltFromPersonID
    AND tuaTransactionTypeID = ltTransactionTypeID
    AND tuaLineGroupID = tlgLineGroupID
WHERE
    tlItemID = $item_id");
}

function make_total_unit_amount() {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp.TotalUnitAmount (
    tuaFromPersonID int,
    tuaLineGroupID int,
    tuaTransactionTypeID int,
    tuaUnitAmount decimal(20, 6),
    PRIMARY KEY (
        tuaFromPersonID,
        tuaLineGroupID,
        tuaTransactionTypeID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.TotalUnitAmount (
    tuaFromPersonID,
    tuaTransactionTypeID,
    tuaLineGroupID,
    tuaUnitAmount
) SELECT
    iacFromPersonID,
    iacTransactionTypeID,
    tlLineGroupID,
    SUM(iacUnitAmount * tlQuantity)
FROM
    shopcharttmp.ItemAverageCost INNER JOIN (
        TransactionLines INNER JOIN
            Transactions
        ON tlTransactionID = tTransactionID
    ) ON iacFromPersonID = tFromPersonID
    AND iacTransactionTypeID = tTransactionTypeID
    AND iacItemID = tlItemID
GROUP BY
    iacFromPersonID,
    iacTransactionTypeID,
    tlLineGroupID");
}

function make_item_average_cost() {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp.ItemAverageCost (
    iacFromPersonID int,
    iacItemID int,
    iacLineGroupID int,
    iacTransactionTypeID int,
    iacUnitAmount decimal(20, 6),
    PRIMARY KEY (
        iacFromPersonID,
        iacTransactionTypeID,
        iacItemID,
        iacLineGroupID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.ItemAverageCost (
    iacFromPersonID,
    iacTransactionTypeID,
    iacItemID,
    iacUnitAmount
) SELECT
    pcFromPersonID,
    pcTransactionTypeID,
    tlItemID,
    SUM(pcExtAmount) / SUM(pcTotalQuantity)
FROM
    shopcharttmp.PastCost INNER JOIN (
        TransactionLines INNER JOIN
            shopcharttmp.LastTransaction
        ON tlTransactionID = ltTransactionID
    ) ON pcFromPersonID = ltFromPersonID
    AND pcTransactionTypeID = ltTransactionTypeID
    AND pcItemID = tlItemID
WHERE
    tlLineGroupID IS NOT NULL
GROUP BY
    pcFromPersonID,
    pcTransactionTypeID,
    tlLineGroupID,
    tlItemID");
}

function make_past_cost() {
    global $mysqli;
     $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp.PastCost (
    pcExtAmount decimal(20, 6),
    pcFromPersonID int,
    pcItemID int,
    pcTotalQuantity decimal(11, 2),
    pcTransactionTypeID int
)");
    $mysqli->query("
INSERT INTO shopcharttmp.PastCost (
    pcFromPersonID,
    pcTransactionTypeID,
    pcItemID,
    pcExtAmount,
    pcTotalQuantity
) SELECT
    uiFromPersonID,
    uiTransactionTypeID,
	uiItemID,
	uiExtAmount,
    uiTotalQuantity
FROM
	shopcharttmp.UngroupedItems
UNION
SELECT
    ltFromPersonID,
    ltTransactionTypeID,
	tlItemID,
	SUM(lgcTotalAmount / lgcLineGroupCount * tlQuantity),
	SUM(tlQuantity)
FROM
	(SELECT
		tlgTransactionID lgcTransactionID,
		tlgLineGroupID lgcLineGroupID,
		tlgTotalAmount lgcTotalAmount,
		SUM(tlQuantity) lgcLineGroupCount
	FROM
		TransactionLineGroups INNER JOIN
			TransactionLines
		ON tlgTransactionID = tlTransactionID
		AND tlgLineGroupID = tlLineGroupID
	GROUP BY
		tlgTransactionID,
		tlgLineGroupID,
		tlgTotalAmount
	) lgc INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				shopcharttmp.LastTransaction
			ON tTransactionTypeID = ltTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON lgcTransactionID = tlTransactionID
	AND lgcLineGroupID = tlLineGroupID
WHERE
	COALESCE(ltUnitAmount, 0) = 0
	AND ltLineGroupID IS NOT NULL
GROUP BY
    ltFromPersonID,
    ltTransactionTypeID,
	tlItemID");
}

function make_ungrouped_items() {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp.UngroupedItems (
    uiExtAmount decimal(20, 6),
    uiFromPersonID int,
    uiItemID int,
    uiTotalQuantity decimal(11, 2),
    uiTransactionTypeID int,
    PRIMARY KEY (
        uiFromPersonID,
        uiTransactionTypeID,
        uiItemID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.UngroupedItems (
    uiFromPersonID,
    uiTransactionTypeID,
    uiItemID,
    uiTotalQuantity,
    uiExtAmount
) SELECT
    ltFromPersonID,
    ltTransactionTypeID,
	ptl.tlItemID,
	SUM(ptl.tlQuantity),
	SUM(ptl.tlUnitAmount * ptl.tlQuantity)
FROM
	TransactionLines ctl INNER JOIN (
		shopcharttmp.LastTransaction INNER JOIN (
			TransactionLines ptl INNER JOIN
				Transactions pt
			ON ptl.tlTransactionID = pt.tTransactionID
		) ON ltTransactionTypeID = pt.tTransactionTypeID
	) ON ctl.tlTransactionID = ltTransactionID
	AND ctl.tlLineGroupID = ltLineGroupID
	AND ctl.tlItemID = ptl.tlItemID
WHERE
	ptl.tlLineGroupID IS NULL
	AND COALESCE(ltUnitAmount, 0) = 0
	AND ltLineGroupID IS NOT NULL
GROUP BY
    ltFromPersonID,
    ltTransactionTypeID,
	ptl.tlItemID");
}

function make_last_transaction($item_id, $where) {
    global $mysqli;
    $mysqli->query("
CREATE TEMPORARY TABLE shopcharttmp.LastTransaction (
    ltDate datetime,
    ltFromPersonID int,
	ltLineGroupID int,
    ltTransactionTypeID int,
    ltTransactionID int,
	ltUnitAmount decimal(9, 4),
    PRIMARY KEY (
        ltFromPersonID,
        ltTransactionTypeID,
        ltTransactionID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.LastTransaction (
    ltFromPersonID,
    ltTransactionTypeID,
    ltTransactionID,
    ltDate
) SELECT
    tFromPersonID,
    tTransactionTypeID,
    MAX(tTransactionID),
    tDate
FROM
    (SELECT
        tFromPersonID ltdFromPersonID,
        tTransactionTypeID ltdTransactionTypeID,
        MAX(tDate) ltdDate
    FROM
        TransactionLines INNER JOIN (
            Transactions INNER JOIN
                TransactionTypes
            ON tTransactionTypeID = ttTransactionTypeID
        ) ON tlTransactionID = tTransactionID
    WHERE
        tlItemID = $item_id
        AND ttFlags & " . TTF_TRANSFERS_INVENTORY . $where . "
    GROUP BY
        tFromPersonID,
        tTransactionTypeID
    ) ltd INNER JOIN (
        TransactionLines INNER JOIN (
            Transactions INNER JOIN
                TransactionTypes
            ON tTransactionTypeID = ttTransactionTypeID
        ) ON tlTransactionID = tTransactionID
    ) ON ltdFromPersonID = tFromPersonID
    AND ltdTransactionTypeID = tTransactionTypeID
    AND ltdDate = tDate
WHERE
    tlItemID = $item_id
    AND ttFlags & " . TTF_TRANSFERS_INVENTORY . $where . "
GROUP BY
    tFromPersonID,
    tTransactionTypeID,
    tDate");
    $mysqli->query("
UPDATE
	shopcharttmp.LastTransaction INNER JOIN
		(SELECT
			tlTransactionID ltlTransactionID,
			MAX(tlUnitAmount * (tlLineGroupID IS NULL)) ltlUnitAmount,
			MAX(tlLineGroupID) ltlLineGroupID
		FROM
			TransactionLines
		WHERE
			tlItemID = $item_id
		GROUP BY
			tlTransactionID
		) hua
	ON ltTransactionID = ltlTransactionID
SET
	ltUnitAmount = ltlUnitAmount,
	ltLineGroupID = ltlLineGroupID");
}

function make_current_assembly_items($person_id) {
    global $mysqli;
    make_item_qty_to($person_id);
    make_item_qty_from($person_id);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.CurrentAssemblyItems (
    caiAssemblyID int,
	caiItemID int,
	caiQuantity decimal(11, 2),
    PRIMARY KEY (
        caiAssemblyID,
        caiItemID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.CurrentAssemblyItems (
	caiAssemblyID,
	caiItemID,
	caiQuantity
) SELECT
    iqtAssemblyID,
    iqtItemID,
    SUM(iqtQuantity - COALESCE(iqfQuantity, 0))
FROM
    shopcharttmp.ItemQtyFrom RIGHT JOIN
        shopcharttmp.ItemQtyTo
    ON iqfAssemblyID = iqtAssemblyID
    AND iqfItemID = iqtItemID
GROUP BY
    iqtAssemblyID,
    iqtItemID");
}

function make_item_qty_to($person_id) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.ItemQtyTo (
    iqtAssemblyID int,
    iqtItemID int,
    iqtQuantity decimal(11, 2),
    PRIMARY KEY (
        iqtAssemblyID,
        iqtItemID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.ItemQtyTo (
    iqtAssemblyID,
    iqtItemID,
    iqtQuantity
) SELECT
    tlAssemblyID,
    tlItemID,
    SUM(CASE
        WHEN ttFlags & " . (TTF_ADDS_INVENTORY_TO_ASSEMBLY | TTF_TRANSFERS_INVENTORY) . " THEN tlQuantity
        ELSE -tlQuantity
    END)
FROM
    Assemblies INNER JOIN (
        TransactionLines INNER JOIN (
            Transactions INNER JOIN
                TransactionTypes
            ON tTransactionTypeID = ttTransactionTypeID
        ) ON tlTransactionID = tTransactionID
    ) ON aAssemblyID = tlAssemblyID
WHERE
    tDate <= NOW()
    AND tToPersonID = $person_id
    AND aFlags & " . AF_WORK_IN_PROGRESS . "
    AND ttFlags & " . (TTF_ADDS_INVENTORY_TO_ASSEMBLY | TTF_REMOVES_INVENTORY | 
        TTF_REMOVES_INVENTORY_FROM_ASSEMBLY | TTF_TRANSFERS_INVENTORY) . "
    AND (
        NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
        OR tFromPersonID <> tToPersonID
    )
GROUP BY
    tlAssemblyID,
    tlItemID");
}

function make_item_qty_from($person_id) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.ItemQtyFrom (
    iqfAssemblyID int,
    iqfItemID int,
    iqfQuantity decimal(11, 2),
    PRIMARY KEY (
        iqfAssemblyID,
        iqfItemID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.ItemQtyFrom (
    iqfAssemblyID,
    iqfItemID,
    iqfQuantity
) SELECT
    tlAssemblyID,
    tlItemID,
    SUM(tlQuantity)
FROM
    Assemblies INNER JOIN (
        TransactionLines INNER JOIN
            Transactions
        ON tlTransactionID = tTransactionID
    ) ON aAssemblyID = tlAssemblyID
WHERE
    tDate <= NOW()
    AND tFromPersonID = $person_id
    AND tFromPersonID <> tToPersonID
    AND aFlags & " . AF_WORK_IN_PROGRESS . "
GROUP BY
    tlAssemblyID,
    tlItemID");
}

function make_qty_needed($person_id) {
    global $mysqli;
    make_current_assembly_items($person_id);
    make_assembly_best_quantity();
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.QtyNeeded (
    qnItemID int PRIMARY KEY,
    qnQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp.QtyNeeded (
    qnItemID,
    qnQuantity
) SELECT
    npaItemID,
    SUM(npaQuantity)
FROM
    (SELECT
        pqAssemblyID appAssemblyID
    FROM
        shopcharttmp.AssemblyBestQuantity INNER JOIN
            shopcharttmp.PersonQuantity
        ON abqAssemblyID = pqAssemblyID
        AND abqQuantity = pqQuantity
    GROUP BY
        pqAssemblyID
    HAVING
        MAX(pqPersonID) = $person_id
    ) app INNER JOIN
        (SELECT
            iiuAssemblyID npaAssemblyID,
            iiuItemID npaItemID,
            MAX(iiuQuantityNeeded) - MAX(COALESCE(iiuQuantityInUse, 0)) npaQuantity
        FROM
            (SELECT
                caiAssemblyID iiuAssemblyID,
                caiItemID iiuItemID,
                caiQuantity iiuQuantityInUse,
                0 iiuQuantityNeeded
            FROM
                shopcharttmp.CurrentAssemblyItems
            UNION
            SELECT
                aAssemblyID,
                atiItemID,
                0,
                atiQuantity
            FROM
                Assemblies INNER JOIN
                    AssemblyTypeItems
                ON aAssemblyTypeID = atiAssemblyTypeID
            WHERE
                aFlags & " . AF_WORK_IN_PROGRESS . "
            ) ai
        GROUP BY
            iiuAssemblyID,
            iiuItemID
        HAVING
            MAX(iiuQuantityNeeded) - MAX(COALESCE(iiuQuantityInUse, 0)) > 0
        ) npa
    ON appAssemblyID = npaAssemblyID
GROUP BY
    npaItemID");
}

function make_assembly_best_quantity() {
    global $mysqli;
    make_person_quantity();
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.AssemblyBestQuantity (
    abqAssemblyID int PRIMARY KEY,
    abqQuantity decimal(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp.AssemblyBestQuantity (
    abqAssemblyID,
    abqQuantity
) SELECT
    pqAssemblyID,
    MAX(pqQuantity)
FROM
    shopcharttmp.PersonQuantity
GROUP BY
    pqAssemblyID");
}

function make_person_quantity() {
    global $mysqli;
    make_person_qty_to();
    make_person_qty_from();
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.PersonQuantity (
    pqAssemblyID int,
    pqPersonID int,
    pqQuantity decimal(11, 2),
    PRIMARY KEY (
        pqAssemblyID,
        pqPersonID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.PersonQuantity (
    pqAssemblyID,
    pqPersonID,
    pqQuantity
) SELECT
    pqtAssemblyID,
    pqtPersonID,
    SUM(pqtQuantity - COALESCE(pqfQuantity, 0))
FROM
    shopcharttmp.PersonQtyFrom RIGHT JOIN
        shopcharttmp.PersonQtyTo
    ON pqfAssemblyID = pqtAssemblyID
    AND pqfPersonID = pqtPersonID
GROUP BY
    pqtAssemblyID,
    pqtPersonID");
}

function make_person_qty_to() {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.PersonQtyTo (
    pqtAssemblyID int,
    pqtDate datetime,
    pqtPersonID int,
    pqtQuantity decimal(11, 2),
    PRIMARY KEY (
        pqtAssemblyID,
        pqtPersonID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.PersonQtyTo (
    pqtAssemblyID,
    pqtDate,
    pqtPersonID,
    pqtQuantity
) SELECT
    tlAssemblyID,
    MAX(tDate),
    tToPersonID,
    SUM(CASE
        WHEN ttFlags & " . (TTF_ADDS_INVENTORY_TO_ASSEMBLY | TTF_TRANSFERS_INVENTORY) . " THEN tlQuantity
        ELSE -tlQuantity
    END)
FROM
    TransactionLines INNER JOIN (
        Transactions INNER JOIN
            TransactionTypes
        ON tTransactionTypeID = ttTransactionTypeID
    ) ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tlAssemblyID IS NOT NULL
    AND (
        NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
        OR tFromPersonID <> tToPersonID
    )
GROUP BY
    tlAssemblyID,
    tToPersonID");
}

function make_person_qty_from() {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp.PersonQtyFrom (
    pqfAssemblyID int,
    pqfPersonID int,
    pqfQuantity decimal(11, 2),
    PRIMARY KEY (
        pqfAssemblyID,
        pqfPersonID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp.PersonQtyFrom (
    pqfAssemblyID,
    pqfPersonID,
    pqfQuantity
) SELECT
    tlAssemblyID,
    tFromPersonID,
    SUM(tlQuantity)
FROM
    TransactionLines INNER JOIN
        Transactions
    ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tFromPersonID <> tToPersonID
    AND tlAssemblyID IS NOT NULL
GROUP BY
    tlAssemblyID,
    tFromPersonID");
}