<?php
require_once('util/transaction_type_flags.php');
define("TEMPORARY", "TEMPORARY");

function make_transaction_assembly($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "TransactionAssembly (
    taAssemblyID INT,
    taQuantity DECIMAL(11, 2),
    taTransactionID INT,
    PRIMARY KEY (
        taTransactionID,
        taAssemblyID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TransactionAssembly (
    taTransactionID,
    taAssemblyID,
    taQuantity
) SELECT
    tlTransactionID,
    tlAssemblyID,
    SUM(tlQuantity)
FROM
    TransactionLines
WHERE
    tlAssemblyID IS NOT NULL
GROUP BY
    tlTransactionID,
    tlAssemblyID");
}

function get_item_description($item_id) {
    global $mysqli;
    $item = $mysqli->query("
SELECT
    iDescription
FROM
    Items
WHERE
    iItemID = $item_id");
    $item = $item->fetch_assoc();
    return $item['iDescription'];
}

function make_highest_item_count($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "HighestItemCount (
    hicQuantity DECIMAL(11, 2),
    hicTransactionID INT PRIMARY KEY
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "HighestItemCount (
    hicTransactionID,
    hicQuantity
) SELECT
    tlTransactionID,
    MAX(taQuantity)
FROM
    shopcharttmp." . $prefix . "TransactionAssembly INNER JOIN
        TransactionLines
    ON taTransactionID = tlTransactionID
    AND taAssemblyID = tlAssemblyID
GROUP BY
    tlTransactionID");
}

function get_all_transactions($where, $prefix) {
    global $mysqli;
    return $mysqli->query("
SELECT
    tTransactionID,
    DATE_FORMAT(tDate, '%y-%m-%d %h:%i %p') TransactionDate,
    tTransactionTypeID,
    tFromPersonID,
    tToPersonID,
    tShipping,
    tTax,
    ttFlags,
    COALESCE(tlgTotalAmount, 0) + COALESCE(tlTotalAmount, 0) + tShipping + tTax tTotalAmount,
    COALESCE(paiAssemblyID, 0) tAssemblyID
FROM
    (SELECT
        tlgTransactionID,
        SUM(tlgTotalAmount) tlgTotalAmount
    FROM
        TransactionLineGroups
    GROUP BY
        tlgTransactionId
    ) tlg RIGHT JOIN (
        (SELECT
            tlTransactionID,
            SUM(tlUnitAmount * tlQuantity) tlTotalAmount
        FROM
            TransactionLines
        GROUP BY
            tlTransactionID
        ) tl RIGHT JOIN (
            (SELECT
                taTransactionID paiTransactionID,
                MAX(taAssemblyID) paiAssemblyID
            FROM
                shopcharttmp." . $prefix . "HighestItemCount INNER JOIN
                    shopcharttmp." . $prefix . "TransactionAssembly
                ON hicTransactionID = taTransactionID
                AND hicQuantity = taQuantity
            GROUP BY
                taTransactionID
            ) pai RIGHT JOIN (
                TransactionTypes INNER JOIN
                    Transactions
                ON ttTransactionTypeID = tTransactionTypeID
            ) ON paiTransactionID = tTransactionID
        ) ON tlTransactionID = tTransactionID
    ) ON tlgTransactionID = tTransactionID        
$where
ORDER BY
    tDate DESC");
}

function make_assembly_item_included_transactions($assembly_id, $item_id, $person_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ResponsibleTransactions (
    rtTransactionID int PRIMARY KEY
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
    rtTransactionID
) SELECT DISTINCT
    tTransactionID
FROM
    TransactionLines INNER JOIN (
        Transactions INNER JOIN
            TransactionTypes
        ON tTransactionTypeID = ttTransactionTypeID
    ) ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tlItemID = $item_id
    AND tlAssemblyID = $assembly_id
    AND (    
            tToPersonID = $person_id
            AND (
                NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
                OR tFromPersonID <> tToPersonID
        ) OR (
            tFromPersonID = $person_id
            AND tFromPersonID <> tToPersonID
        )
    )");
}

function make_qty_on_hand_transactions($item_id, $person_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ResponsibleTransactions (
    rtTransactionID int PRIMARY KEY
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
    rtTransactionID
) SELECT DISTINCT
    tTransactionID
FROM
    TransactionLines INNER JOIN (
        Transactions INNER JOIN
            TransactionTypes
        ON tTransactionTypeID = ttTransactionTypeID
    ) ON tlTransactionID = tTransactionID
WHERE
    tDate <= NOW()
    AND tlItemID = $item_id
    AND (
        (
            tlAssemblyID IS NULL
            AND (
                tFromPersonID = $person_id
                OR tToPersonID = $person_id
            ) AND (
                ttFlags & " . TTF_REMOVES_INVENTORY . "
                OR (
                    ttFlags & " . TTF_TRANSFERS_INVENTORY . "
                    AND tToPersonID <> tFromPersonID
                )
            )
        ) OR (
            tlAssemblyID IS NOT NULL
            AND (
                (
                    ttFlags & " . TTF_ADDS_INVENTORY_TO_ASSEMBLY . "
                    AND tFromPersonID = $person_id
                ) OR (
                    ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
                    AND tToPersonID = $person_id
                )
            )
        )
    )");
}

function get_special_transactions($where, $prefix) {
    global $mysqli;
    return $mysqli->query("
SELECT
    tTransactionID,
    DATE_FORMAT(tDate, '%y-%m-%d %h:%i %p') TransactionDate,
    tTransactionTypeID,
    tFromPersonID,
    tToPersonID,
    tShipping,
    tTax,
    ttFlags,
    COALESCE(tlgTotalAmount, 0) + COALESCE(tlTotalAmount, 0) + tShipping + tTax tTotalAmount,
    COALESCE(paiAssemblyID, 0) tAssemblyID
FROM
    (SELECT
        tlgTransactionID,
        SUM(tlgTotalAmount) tlgTotalAmount
    FROM
        TransactionLineGroups
    GROUP BY
        tlgTransactionId
    ) tlg RIGHT JOIN (
        (SELECT
            tlTransactionID,
            SUM(tlUnitAmount * tlQuantity) tlTotalAmount
        FROM
            TransactionLines
        GROUP BY
            tlTransactionID
        ) tl RIGHT JOIN (
            (SELECT
                taTransactionID paiTransactionID,
                MAX(taAssemblyID) paiAssemblyID
            FROM
                shopcharttmp." . $prefix . "HighestItemCount INNER JOIN
                    shopcharttmp." . $prefix . "TransactionAssembly
                ON hicTransactionID = taTransactionID
                AND hicQuantity = taQuantity
            GROUP BY
                taTransactionID
            ) pai RIGHT JOIN (
                TransactionTypes INNER JOIN (
                    Transactions INNER JOIN
                        (SELECT DISTINCT
                            rtTransactionID
                        FROM
                            shopcharttmp." . $prefix . "ResponsibleTransactions
                        ) rt
                    ON tTransactionID = rtTransactionID
                ) ON ttTransactionTypeID = tTransactionTypeID
            ) ON paiTransactionID = tTransactionID
        ) ON tlTransactionID = tTransactionID
    ) ON tlgTransactionID = tTransactionID        
$where
ORDER BY
    tDate DESC");
}



function make_responsible_transactions($transaction_id, $line_id, $prefix) {
    global $mysqli;
    make_current_transaction_items($transaction_id, $line_id, $prefix);
    make_out_transactions($prefix);
    make_reference_transactions($prefix);
    make_past_transactions($prefix);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtAssemblyID int,
	rtCurrentFlags int,
	rtDate datetime,
	rtFlags int,
	rtFromPersonID int,
	rtItemID int,
	rtLineGroupID int,
	rtQtyUsed decimal(11, 2),
	rtTransactionID int,
	rtTransactionTypeID int,
	PRIMARY KEY (
		rtTransactionID,
		rtItemID,
		rtLineGroupID,
		rtAssemblyID
	)
)");
    add_assy_transactions($prefix);
    make_non_assy_items($transaction_id, $line_id, $prefix);
    add_non_assy_transactions($prefix);
    make_salvage_items($prefix);
    make_salvage_out_transactions($prefix);
    make_salvage_reference_transactions($prefix);
    make_salvage_past_transactions($prefix);
    add_salvage_transactions($prefix);
}
    

function make_transaction_line_from_amount($transaction_id, $prefix) {
    global $mysqli;
    make_responsible_transactions($transaction_id, 0, $prefix);
    make_past_cost($prefix);
    make_item_average_cost($prefix);
    make_total_unit_amount($prefix);
    make_item_percent($prefix);
    make_grouped_items($prefix);
    make_transaction_item_cost($prefix);
    make_directly_responsible($prefix);
    make_general_invy_cost($prefix);
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "TransactionLineFromAmount (
    tlfaHasUnitAmount bit,
	tlfaLineID int PRIMARY KEY,
	tlfaUnitAmount decimal(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TransactionLineFromAmount (
	tlfaLineID,
    tlfaHasUnitAmount,
	tlfaUnitAmount
) SELECT
	tlLineID,
    drItemID IS NULL AND gicItemID IS NULL,
	((ctiQuantity - COALESCE(drQuantity, 0)) * COALESCE(gicUnitAmount, 0) + COALESCE(drTotalAmount, 0)) / ctiQuantity
FROM
	shopcharttmp." . $prefix . "DirectlyResponsible RIGHT JOIN (
		shopcharttmp." . $prefix . "GeneralInvyCost RIGHT JOIN (
			shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN
				TransactionLines
			ON ctiTransactionID = tlTransactionID
			AND ctiItemID = tlItemID
		) ON gicItemID = ctiItemID
	) ON drItemID = ctiItemID");
}

function make_current_transaction_items($transaction_id, $line_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "CurrentTransactionItems (
	ctiAssemblyID int,
	ctiDate datetime,
	ctiFlags int,
	ctiFromPersonID int,
	ctiItemID int,
	ctiQuantity decimal(11, 2),
	ctiTransactionID int,
	PRIMARY KEY (
		ctiTransactionID,
		ctiItemID,
		ctiAssemblyID
	)
)");
    if($line_id == 0) {
        $line_id = "";
    } else {
        $line_id = "
    AND tlLineID = $line_id";
    }
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "CurrentTransactionItems (
	ctiTransactionID,
	ctiDate,
	ctiFromPersonID,
	ctiFlags,
	ctiItemID,
	ctiAssemblyID,
	ctiQuantity
) SELECT
	tTransactionID,
	tDate,
	tFromPersonID,
	ttFlags,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	TransactionLines INNER JOIN (
		Transactions INNER JOIN
			TransactionTypes
		ON tTransactionTypeID = ttTransactionTypeID
	) ON tlTransactionID = tTransactionID
WHERE
	tTransactionID = $transaction_id $line_id
GROUP BY
	tTransactionID,
	tDate,
	tFromPersonID,
	ttFlags,
	tlItemID,
	tlAssemblyID");
}

function make_out_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "OutTransactions (
	otAssemblyID int,
	otDate datetime,
	otItemID int,
	otQuantity decimal(11, 2),
	otTransactionID int,
	PRIMARY KEY (
		otTransactionID,
		otItemID,
		otAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "OutTransactions (
	otTransactionID,
	otDate,
	otItemID,
	otAssemblyID,
	otQuantity
) SELECT
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON ctiFromPersonID = tFromPersonID
	AND ctiItemID = tlItemID
	AND ctiAssemblyID = tlAssemblyID
WHERE
	(
		tDate < ctiDate
		OR (
			tDate = ctiDate
			AND tTransactionID < ctiTransactionID
		)
	) AND (
		(
			ttFlags & " . TTF_TRANSFERS_INVENTORY . "
			AND tFromPersonID <> tToPersonID
		) OR ttFlags & " . (TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY) . "
	)
GROUP BY
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID");
}

function make_reference_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ReferenceTransactions (
	rtDate datetime,
	rtTransactionID int PRIMARY KEY,
	INDEX rtiDate (
		rtDate
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ReferenceTransactions (
	rtTransactionID,
	rtDate
) SELECT DISTINCT
	tTransactionID,
	tDate
FROM
	shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN (
		TransactionLines INNER JOIN
			Transactions
		ON tlTransactionID = tTransactionID
	) ON ctiItemID = tlItemID
	AND ctiAssemblyID = tlAssemblyID
WHERE
	tFromPersonID = ctiFromPersonID
	OR tToPersonID = ctiFromPersonID");
}

function make_past_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "PastTransactions (
	ptAssemblyID int,
	ptItemID int,
	ptPersonID int,
	ptQuantity decimal(11, 2),
	ptTransactionID int,
	PRIMARY KEY (
		ptTransactionID,
		ptPersonID,
		ptItemID,
		ptAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PastTransactions (
	ptTransactionID,
	ptPersonID,
	ptItemID,
	ptAssemblyID,
	ptQuantity
) SELECT
	rtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID,
	SUM(
		CASE
			WHEN rtDate > tDate
				OR (
					rtDate = tDate
					AND rtTransactionID > tTransactionID
				) THEN CASE
				WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tlQuantity
				ELSE -tlQuantity
			END
			ELSE 0
		END
	)
FROM
	shopcharttmp." . $prefix . "ReferenceTransactions,
	shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON ctiItemID = tlItemID
	AND ctiAssemblyID = tlAssemblyID
WHERE
	ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY |
        TTF_ADDS_INVENTORY_TO_ASSEMBLY) . "
	AND tDate <= rtDate
	AND (
		tToPersonID = ctiFromPersonID
		OR tFromPersonID = ctiFromPersonID
	)
GROUP BY
	rtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID");
}

function add_assy_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtTransactionID,
	rtTransactionTypeID,
	rtFromPersonID,
	rtDate,
	rtFlags,
	rtCurrentFlags,
	rtItemID,
	rtLineGroupID,
	rtAssemblyID,
	rtQtyUsed
) SELECT
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	COALESCE(itLineGroupID, 0),
	itAssemblyID,
	CASE
		WHEN itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN
			CASE
				WHEN itRunningTotal < itQtyReqd THEN itQuantity
				ELSE itQtyReqd - (itRunningTotal - itQuantity)
			END - SUM(
				CASE
					WHEN otDate > itDate OR (otDate = itDate AND otTransactionID > itTransactionID) THEN otQuantity
					ELSE 0
				END
			)
		ELSE 
			CASE
				WHEN itQuantity < itQtyReqd THEN itQuantity
				ELSE itQtyReqd
			END
	END
FROM
	shopcharttmp." . $prefix . "OutTransactions RIGHT JOIN
		(SELECT
			tTransactionID itTransactionID,
			tTransactionTypeID itTransactionTypeID,
			tFromPersonID itFromPersonID,
			tDate itDate,
			ttFlags itFlags,
			ctiFlags itCurrentFlags,
			tlItemID itItemID,
			tlLineGroupID itLineGroupID,
			tlAssemblyID itAssemblyID,
			ctiQuantity itQtyReqd,
			SUM(tlQuantity) itQuantity,
			SUM(tlQuantity) + COALESCE(ptQuantity, 0) itRunningTotal
		FROM
			shopcharttmp." . $prefix . "PastTransactions RIGHT JOIN (
				shopcharttmp." . $prefix . "CurrentTransactionItems INNER JOIN (
					TransactionLines INNER JOIN (
						Transactions INNER JOIN
							TransactionTypes
						ON tTransactionTypeID = ttTransactionTypeID
					) ON tlTransactionID = tTransactionID
				) ON ctiFromPersonID = tToPersonID
				AND ctiItemID = tlItemID
				AND ctiAssemblyID = tlAssemblyID
			) ON ptTransactionID = tTransactionID
			AND ptPersonID = tToPersonID
			AND ptItemID = tlItemID
			AND ptAssemblyID = tlAssemblyID
		WHERE
			NOT ttFlags & " . (TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY) . "
			AND (
				tDate < ctiDate
				OR (
					tDate = ctiDate
					AND tTransactionID < ctiTransactionID
				)
			) AND (
				(
					NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
					OR tFromPersonID <> tToPersonID
				)
			)
		GROUP BY
			tTransactionID,
			tTransactionTypeID,
			tFromPersonID,
			tDate,
			ttFlags,
			ctiFlags,
			tlItemID,
			tlLineGroupID,
			ctiQuantity,
			ptQuantity
		) it
	ON otItemID = itItemID
	AND otAssemblyID = itAssemblyID
GROUP BY
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	itLineGroupID,
	itAssemblyID,
	itQtyReqd,
	itQuantity,
	itRunningTotal
HAVING
	itRunningTotal - itQuantity - SUM(
		CASE
			WHEN otDate > itDate 
				OR (
					otDate = itDate
					AND otTransactionID > itTransactionID
				) THEN otQuantity
			ELSE 0
	END) < itQtyReqd
	OR NOT itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY));
}

function make_non_assy_items($transaction_id, $line_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "NonAssyItems (
	naiDate datetime,
	naiFlags int,
	naiFromPersonID int,
	naiItemID int PRIMARY KEY,
	naiQuantity decimal(11, 2),
	naiTransactionID int
)");
    if($line_id == 0) {
        $line_id = "";
    } else {
        $line_id = "
    AND tlLineID = $line_id";
    }
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "NonAssyItems (
	naiTransactionID,
	naiFromPersonID,
	naiDate,
	naiFlags,
	naiItemID,
	naiQuantity
) SELECT
	$transaction_id,
	iaqFromPersonID,
	iaqDate,
	iaqFlags,
	iaqItemID,
	SUM(iaqQuantity - COALESCE(alqQuantity, 0))
FROM
	(SELECT
		rtItemID alqItemID,
		rtAssemblyID alqAssemblyID,
		SUM(rtQtyUsed) alqQuantity
	FROM
		shopcharttmp." . $prefix . "ResponsibleTransactions
	WHERE
		rtFlags & " . TTF_TRANSFERS_INVENTORY . "
	GROUP BY
		rtItemID,
		rtAssemblyID
	) alq RIGHT JOIN
		(SELECT
			tFromPersonID iaqFromPersonID,
			tDate iaqDate,
			ttFlags iaqFlags,
			tlItemID iaqItemID,
			tlAssemblyID iaqAssemblyID,
			SUM(tlQuantity) iaqQuantity
		FROM
			TransactionLines INNER JOIN (
				Transactions INNER JOIN
					TransactionTypes
				ON tTransactionTypeID = ttTransactionTypeID
			) ON tlTransactionID = tTransactionID
		WHERE
			tTransactionID = $transaction_id $line_id
		GROUP BY
			tFromPersonID,
			tDate,
			ttFlags,
			tlItemID,
			tlAssemblyID
		) iaq
	ON alqItemID = iaqItemID
	AND alqAssemblyID = iaqAssemblyID
GROUP BY
	iaqFlags,
	iaqItemID
HAVING
	SUM(iaqQuantity - COALESCE(alqQuantity, 0)) > 0");
}

function add_non_assy_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtTransactionID,
	rtTransactionTypeID,
	rtFromPersonID,
	rtDate,
	rtFlags,
	rtCurrentFlags,
	rtItemID,
	rtLineGroupID,
	rtAssemblyID,
	rtQtyUsed
) SELECT
	tTransactionID,
	tTransactionTypeID,
	tFromPersonID,
	tDate,
	ttFlags | " . TTF_GENERAL_INVENTORY . ",
	naiFlags,
	tlItemID,
	COALESCE(tlLineGroupID, 0),
	COALESCE(tlAssemblyID, 0),
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "NonAssyItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON naiItemID = tlItemID
WHERE
	(
		tDate < naiDate
		OR (
			tDate = naiDate
			AND tTransactionID < naiTransactionID
		)
	) AND NOT ttFlags & " . TTF_REMOVES_INVENTORY . "
	AND tToPersonID = naiFromPersonID
	AND (
		tlAssemblyID IS NULL
		OR ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
	)
GROUP BY
	tTransactionID,
	tTransactionTypeID,
	tFromPersonID,
	tDate,
	ttFlags | " . TTF_GENERAL_INVENTORY . ",
	naiFlags,
	tlItemID,
	COALESCE(tlLineGroupID, 0),
	COALESCE(tlAssemblyID, 0)");
}

function make_salvage_items($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "SalvageItems (
	siAssemblyID int,
	siDate datetime,
	siFromPersonID int,
	siFlags int,
	siItemID int,
	siQuantity decimal(11, 2),
	siTransactionID int,
	PRIMARY KEY (
		siTransactionID,
		siItemID,
		siAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvageItems (
	siTransactionID,
	siDate,
	siFromPersonID,
	siFlags,
	siItemID,
	siAssemblyID,
	siQuantity
) SELECT
	tTransactionID,
	tDate,
	tFromPersonID,
	naiFlags,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "NonAssyItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON naiFromPersonID = tToPersonID
    AND naiItemID = tlItemID
WHERE
	ttFlags & " . TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
	AND tlAssemblyID IS NOT NULL
GROUP BY
	tTransactionID,
	tDate,
	tFromPersonID,
	naiFlags,
	tlItemID,
	tlAssemblyID");
}

function make_salvage_out_transactions($prefix) {
    global $mysqli;
    $mysqli->query ("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "SalvageOutTransactions (
	sotAssemblyID int,
	sotDate datetime,
	sotItemID int,
	sotQuantity decimal(11, 2),
	sotTransactionID int,
	PRIMARY KEY (
		sotTransactionID,
		sotItemID,
		sotAssemblyID
	)
)") ;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvageOutTransactions (
	sotTransactionID,
	sotDate,
	sotItemID,
	sotAssemblyID,
	sotQuantity
) SELECT
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID,
	SUM(tlQuantity)
FROM
	shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON siFromPersonID = tFromPersonID
	AND siItemID = tlItemID
	AND siAssemblyID = tlAssemblyID
WHERE
	(
		tDate < siDate
		OR (
			tDate = siDate
			AND tTransactionID < siTransactionID
		)
	) AND (
		(
			ttFlags & " . TTF_TRANSFERS_INVENTORY . "
			AND tFromPersonID <> tToPersonID
		) OR ttFlags & " . TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY . "
	)
GROUP BY
	tTransactionID,
	tDate,
	tlItemID,
	tlAssemblyID");
}

function make_salvage_reference_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "SalvageReferenceTransactions (
	srtDate datetime,
	srtTransactionID int PRIMARY KEY,
	INDEX srtiDate (
		srtDate
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvageReferenceTransactions (
	srtTransactionID,
	srtDate
) SELECT DISTINCT
	tTransactionID,
	tDate
FROM
	shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
		TransactionLines INNER JOIN
			Transactions
		ON tlTransactionID = tTransactionID
	) ON siItemID = tlItemID
	AND siAssemblyID = tlAssemblyID
WHERE
	tFromPersonID = siFromPersonID
	OR tToPersonID = siFromPersonID");
}

function make_salvage_past_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "SalvagePastTransactions (
	sptAssemblyID int,
	sptItemID int,
	sptPersonID int,
	sptQuantity decimal(11, 2),
	sptTransactionID int,
	PRIMARY KEY (
		sptTransactionID,
		sptPersonID,
		sptItemID,
		sptAssemblyID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "SalvagePastTransactions (
	sptTransactionID,
	sptPersonID,
	sptItemID,
	sptAssemblyID,
	sptQuantity
) SELECT
	srtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID,
	SUM(
		CASE
			WHEN srtDate > tDate
				OR (
					srtDate = tDate
					AND srtTransactionID > tTransactionID
				) THEN CASE
				WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tlQuantity
				ELSE -tlQuantity
			END
			ELSE 0
		END
	)
FROM
	shopcharttmp." . $prefix . "SalvageReferenceTransactions,
	shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
		TransactionLines INNER JOIN (
			Transactions INNER JOIN
				TransactionTypes
			ON tTransactionTypeID = ttTransactionTypeID
		) ON tlTransactionID = tTransactionID
	) ON siItemID = tlItemID
	AND siAssemblyID = tlAssemblyID
WHERE
	ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY |
        TTF_ADDS_INVENTORY_TO_ASSEMBLY) . "
	AND tDate <= srtDate
	AND (
		tToPersonID = siFromPersonID
		OR tFromPersonID = siFromPersonID
	)
GROUP BY
	srtTransactionID,
	CASE
		WHEN ttFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN tToPersonID
		ELSE tFromPersonID
	END,
	tlItemID,
	tlAssemblyID");
}

function add_salvage_transactions($prefix) {
    global $mysqli;
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ResponsibleTransactions (
	rtTransactionID,
	rtTransactionTypeID,
	rtFromPersonID,
	rtDate,
	rtFlags,
	rtCurrentFlags,
	rtItemID,
	rtLineGroupID,
	rtAssemblyID,
	rtQtyUsed
) SELECT
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	COALESCE(itLineGroupID, 0),
	itAssemblyID,
	CASE
		WHEN itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY) . " THEN
			CASE
				WHEN itRunningTotal < itQtyReqd THEN itQuantity
				ELSE itQtyReqd - (itRunningTotal - itQuantity)
			END - SUM(
				CASE
					WHEN sotDate > itDate OR (sotDate = itDate AND sotTransactionID > itTransactionID) THEN sotQuantity
					ELSE 0
				END
			)
		ELSE 
			CASE
				WHEN itQuantity < itQtyReqd THEN itQuantity
				ELSE itQtyReqd
			END
	END
FROM
	shopcharttmp." . $prefix . "SalvageOutTransactions RIGHT JOIN
		(SELECT
			tTransactionID itTransactionID,
			tTransactionTypeID itTransactionTypeID,
			tFromPersonID itFromPersonID,
			tDate itDate,
			ttFlags itFlags,
			siFlags itCurrentFlags,
			tlItemID itItemID,
			tlLineGroupID itLineGroupID,
			tlAssemblyID itAssemblyID,
			siQuantity itQtyReqd,
			SUM(tlQuantity) itQuantity,
			SUM(tlQuantity) + sptQuantity itRunningTotal
		FROM
			shopcharttmp." . $prefix . "SalvagePastTransactions RIGHT JOIN (
				shopcharttmp." . $prefix . "SalvageItems INNER JOIN (
					TransactionLines INNER JOIN (
						Transactions INNER JOIN
							TransactionTypes
						ON tTransactionTypeID = ttTransactionTypeID
					) ON tlTransactionID = tTransactionID
				) ON siFromPersonID = tToPersonID
				AND siItemID = tlItemID
				AND siAssemblyID = tlAssemblyID
			) ON sptTransactionID = tTransactionID
			AND sptPersonID = tToPersonID
			AND sptItemID = tlItemID
			AND sptAssemblyID = tlAssemblyID
		WHERE
			NOT ttFlags & " . (TTF_REMOVES_INVENTORY | TTF_REMOVES_INVENTORY_FROM_ASSEMBLY) . "
			AND (
				tDate < siDate
				OR (
					tDate = siDate
					AND tTransactionID < siTransactionID
				)
			) AND (
				(
					NOT ttFlags & " . TTF_TRANSFERS_INVENTORY . "
					OR tFromPersonID <> tToPersonID
				)
			)
		GROUP BY
			tTransactionID,
			tTransactionTypeID,
			tDate,
			ttFlags,
			siFlags,
			tlItemID,
			tlLineGroupID,
			siQuantity,
			sptQuantity
		) it
	ON sotItemID = itItemID
	AND sotAssemblyID = itAssemblyID
GROUP BY
	itTransactionID,
	itTransactionTypeID,
	itFromPersonID,
	itDate,
	itFlags,
	itCurrentFlags,
	itItemID,
	COALESCE(itLineGroupID, 0),
	itAssemblyID,
	itQtyReqd,
	itQuantity,
	itRunningTotal
HAVING
	itRunningTotal - itQuantity - SUM(
		CASE
			WHEN sotDate > itDate 
				OR (
					sotDate = itDate
					AND sotTransactionID > itTransactionID
				) THEN sotQuantity
			ELSE 0
	END) < itQtyReqd
	OR NOT itFlags & " . (TTF_TRANSFERS_INVENTORY | TTF_ADDS_INVENTORY_TO_ASSEMBLY));
}

/*For each Item that's a member of a TransactionLineGroup on $transaction_id,
 * the total average unit amount of all Transactions (including $transaction_id)
 * of the same TransactionType as $transaction_id. For this calculation, members
 * of TransactionLineGroups are treated equally, not prorated based on historic
 * information.
 */
function make_transaction_item_average_cost($transaction_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "TransactionItemAverageCost (
    tiacItemID INT,
    tiacUnitAmount DECIMAL(20, 6),
    PRIMARY KEY (
        tiacItemID
    )
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TransactionItemAverageCost (
    tiacItemID,
    tiacUnitAmount
) SELECT
    tlItemID,
    SUM(tpcExtAmount) / SUM(tpcTotalQuantity)
FROM
    shopcharttmp." . $prefix . "TransactionPastCost INNER JOIN
        TransactionLines
    ON tpcItemID = tlItemID
WHERE
    tlTransactionID = $transaction_id
    AND tlLineGroupID IS NOT NULL
GROUP BY
    tlItemID");
}

function make_transaction_total_unit_amount($transaction_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "TransactionTotalUnitAmount (
    ttuaLineGroupID INT PRIMARY KEY,
    ttuaUnitAmount DECIMAL(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TransactionTotalUnitAmount (
    ttuaLineGroupID,
    ttuaUnitAmount
) SELECT
    tlLineGroupID,
    SUM(tiacUnitAmount * tlQuantity)
FROM
    shopcharttmp." . $prefix . "TransactionItemAverageCost INNER JOIN
        TransactionLines
    ON tiacItemID = tlItemID
WHERE
    tlTransactionID = $transaction_id
GROUP BY
    tlLineGroupID");
}

/*For each Item of $transaction_id, the total quantity and total extended amount
 * for all Transactions (including $transaction_id) of the same TransactionType
 * as $transaction_id. Members of TransactionLineGroups are treated equally per
 * TransactionLineGroup, not pro-rated based on historic information. 
 */
function make_transaction_past_cost($transaction_id, $prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "TransactionPastCost (
    tpcExtAmount DECIMAL(20, 6),
    tpcItemID INT,
    tpcTotalQuantity DECIMAL(11, 2)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TransactionPastCost (
    tpcItemID,
    tpcExtAmount,
    tpcTotalQuantity
) SELECT
	ptl.tlItemID,
	SUM(ptl.tlUnitAmount * ptl.tlQuantity),
	SUM(ptl.tlQuantity)
FROM
	TransactionLines ctl INNER JOIN (
		Transactions ct INNER JOIN (
			TransactionLines ptl INNER JOIN
				Transactions pt
			ON ptl.tlTransactionID = pt.tTransactionID
		) ON ct.tTransactionTypeID = pt.tTransactionTypeID
	) ON ctl.tlTransactionID = ct.tTransactionID
	AND ctl.tlItemID = ptl.tlItemID
WHERE
	ct.tTransactionID = $transaction_id
	AND ptl.tlLineGroupID IS NULL
GROUP BY
	ptl.tlItemID
UNION
SELECT
	tlItemID,
	SUM(lgcTotalAmount / lgcLineGroupCount * ptl.tlQuantity),
	SUM(ptl.tlQuantity)
FROM
	(SELECT
		tlgTransactionID lgcTransactionID,
		tlgLineGroupID lgcLineGroupID,
		tlgTotalAmount lgcTotalAmount,
		SUM(tlQuantity) lgcLineGroupCount
	FROM
		TransactionLineGroups INNER JOIN
			TransactionLines
		ON tlgTransactionID = tlTransactionID
		AND tlgLineGroupID = tlLineGroupID
	GROUP BY
		tlgTransactionID,
		tlgLineGroupID,
		tlgTotalAmount
	) lgc INNER JOIN (
		TransactionLines ptl INNER JOIN (
            Transactions pt INNER JOIN
                    Transactions ct
            ON pt.tTransactionTypeID = ct.tTransactionTypeID
		) ON ptl.tlTransactionID = pt.tTransactionID
	) ON lgcTransactionID = ptl.tlTransactionID
	AND lgcLineGroupID = tlLineGroupID
WHERE
    ct.tTransactionID = $transaction_id
GROUP BY
	tlItemID");
}

function make_past_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "PastCost (
    pcExtAmount decimal(20, 6),
    pcItemID int,
    pcTotalQuantity decimal(11, 2),
	pcTransactionTypeID int
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PastCost (
	pcTransactionTypeID,
    pcItemID,
    pcExtAmount,
    pcTotalQuantity
) SELECT
	tTransactionTypeID,
	ptl.tlItemID,
	SUM(
		CASE
			WHEN (rtCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") <> (ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") THEN
				CASE
					WHEN rtCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN ptl.tlUnitAmount / pMonetaryPerSecond
					ELSE ptl.tlUnitAmount * pMonetaryPerSecond
				END
			ELSE ptl.tlUnitAmount
		END * ptl.tlQuantity),
	SUM(ptl.tlQuantity)
FROM
	TransactionLines ctl INNER JOIN (
		(SELECT DISTINCT
			rtTransactionID,
			rtTransactionTypeID,
			rtCurrentFlags
		FROM
			shopcharttmp." . $prefix . "ResponsibleTransactions
		) rt INNER JOIN (
			TransactionLines ptl INNER JOIN (
				People INNER JOIN (
					Transactions INNER JOIN
						TransactionTypes
					ON tTransactionTypeID = ttTransactionTypeID
				) ON pPersonID = tFromPersonID
			) ON ptl.tlTransactionID = tTransactionID
		) ON rtTransactionTypeID = tTransactionTypeID
	) ON ctl.tlTransactionID = rtTransactionID
	AND ctl.tlItemID = ptl.tlItemID
WHERE
	ptl.tlLineGroupID IS NULL
GROUP BY
	tTransactionID,
	ptl.tlItemID");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "PastCost (
	pcTransactionTypeID,
    pcItemID,
    pcExtAmount,
    pcTotalQuantity
) SELECT
	tTransactionTypeID,
	tlItemID,
	SUM(
		CASE
			WHEN (rtCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") <> (ttFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") THEN
				CASE
					WHEN rtCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN lgcTotalAmount / lgcLineGroupCount / pMonetaryPerSecond
					ELSE lgcTotalAmount / lgcLineGroupCount * pMonetaryPerSecond
				END
			ELSE lgcTotalAmount / lgcLineGroupCount
		END * tlQuantity),
	SUM(tlQuantity)
FROM
	(SELECT
		tlgTransactionID lgcTransactionID,
		tlgLineGroupID lgcLineGroupID,
		tlgTotalAmount lgcTotalAmount,
		SUM(tlQuantity) lgcLineGroupCount
	FROM
		TransactionLineGroups INNER JOIN
			TransactionLines
		ON tlgTransactionID = tlTransactionID
		AND tlgLineGroupID = tlLineGroupID
	GROUP BY
		tlgTransactionID,
		tlgLineGroupID,
		tlgTotalAmount
	) lgc INNER JOIN (
		TransactionLines INNER JOIN (
			People INNER JOIN (
				TransactionTypes INNER JOIN (
					Transactions INNER JOIN
						(SELECT DISTINCT
							rtTransactionTypeID,
							rtCurrentFlags
						FROM
							shopcharttmp." . $prefix . "ResponsibleTransactions
						) rt
					ON tTransactionTypeID = rtTransactionTypeID
				) ON ttTransactionTypeID = tTransactionTypeID
			) ON pPersonID = tFromPersonID
		) ON tlTransactionID = tTransactionID
	) ON lgcTransactionID = tlTransactionID
	AND lgcLineGroupID = tlLineGroupID
GROUP BY
	tTransactionTypeID,
	tlItemID");
}

function make_item_average_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ItemAverageCost (
    iacItemID int PRIMARY KEY,
    iacUnitAmount decimal(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ItemAverageCost (
    iacItemID,
    iacUnitAmount
) SELECT
    tlItemID,
    SUM(pcExtAmount) / SUM(pcTotalQuantity)
FROM
    shopcharttmp." . $prefix . "PastCost INNER JOIN (
		TransactionLines INNER JOIN
			(SELECT DISTINCT
				rtTransactionID,
				rtLineGroupID
			FROM
				shopcharttmp." . $prefix . "ResponsibleTransactions
			) rt
		ON tlTransactionID = rtTransactionID
		AND tlLineGroupID = rtLineGroupID
    ) ON pcItemID = tlItemID
WHERE
    tlLineGroupID <> 0
GROUP BY
    tlItemID");
}

function make_total_unit_amount($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "TotalUnitAmount (
    tuaLineGroupID int,
	tuaTransactionID int,
    tuaUnitAmount decimal(20, 6),
	PRIMARY KEY (
		tuaTransactionID,
		tuaLineGroupID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TotalUnitAmount (
	tuaTransactionID,
    tuaLineGroupID,
    tuaUnitAmount
) SELECT
	tlTransactionID,
    tlLineGroupID,
    SUM(iacUnitAmount * tlQuantity)
FROM
    shopcharttmp." . $prefix . "ItemAverageCost INNER JOIN (
		TransactionLines INNER JOIN
			(SELECT DISTINCT
				rtTransactionID
			FROM
				shopcharttmp." . $prefix . "ResponsibleTransactions
			) rt
		ON tlTransactionID = rtTransactionID
    ) ON iacItemID = tlItemID
GROUP BY
	tlTransactionID,
    tlLineGroupID");
}

function make_item_percent($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "ItemPercent (
	ipItemID int,
	ipLineGroupID int,
	ipPercent decimal(20, 6),
	ipTransactionID int,
	PRIMARY KEY (
		ipTransactionID,
		ipLineGroupID,
		ipItemID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "ItemPercent (
	ipTransactionID,
	ipLineGroupID,
	ipItemID,
	ipPercent
) SELECT
	tlTransactionID,
	tuaLineGroupID,
	tlItemID,
	iacUnitAmount / tuaUnitAmount
FROM
	shopcharttmp." . $prefix . "TotalUnitAmount INNER JOIN (
		shopcharttmp." . $prefix . "ItemAverageCost INNER JOIN (
			TransactionLines INNER JOIN (
				TransactionLineGroups INNER JOIN
					(SELECT DISTINCT
						rtTransactionID
					FROM
						shopcharttmp." . $prefix . "ResponsibleTransactions
					) rt
				ON tlgTransactionID = rtTransactionID
			) ON tlTransactionID = tlgTransactionID
			AND tlLineGroupID = tlgLineGroupID
		) ON iacItemID = tlItemID
	) ON tuaTransactionID = rtTransactionID
	AND tuaLineGroupID = tlgLineGroupID");
}

function make_grouped_items($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "GroupedItems (
	giLineGroupID int,
	giItemID int,
	giTransactionID int,
	giUnitAmount decimal(20, 6),
	PRIMARY KEY (
		giTransactionID,
		giLineGroupID,
		giItemID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "GroupedItems (
	giTransactionID,
	giLineGroupID,
	giItemID,
	giUnitAmount
) SELECT
	ipTransactionID,
	ipLineGroupID,
	ipItemID,
	ipPercent * tlgTotalAmount
FROM
	shopcharttmp." . $prefix . "ItemPercent INNER JOIN (
		TransactionLineGroups INNER JOIN
			(SELECT DISTINCT
				rtTransactionID
			FROM
				shopcharttmp." . $prefix . "ResponsibleTransactions
			) rt
		ON tlgTransactionID = rtTransactionID
	) ON ipTransactionID = rtTransactionID
	AND ipLineGroupID = tlgLineGroupID");
}

function make_transaction_item_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "TransactionItemCost (
	ticAssemblyID int,
	ticCurrentFlags int,
	ticFlags int,
	ticFromPersonID int,
	ticItemID int,
	ticQuantity decimal(11, 2),
	ticTransactionID int,
	ticUnitAmount decimal(20, 6),
	PRIMARY KEY (
		ticTransactionID,
		ticAssemblyID,
		ticItemID
	)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "TransactionItemCost (
	ticTransactionID,
	ticFromPersonID,
	ticCurrentFlags,
	ticFlags,
	ticAssemblyID,
	ticItemID,
	ticUnitAmount,
	ticQuantity
) SELECT
	rtTransactionID,
	rtFromPersonID,
	rtCurrentFlags,
	rtFlags,
	rtAssemblyID,
	rtItemID,
	AVG(COALESCE(giUnitAmount, ugiUnitAmount)),
	rtQtyUsed
FROM
    shopcharttmp." . $prefix . "GroupedItems RIGHT JOIN (
		(SELECT
			tlTransactionID ugiTransactionID,
			tlItemID ugiItemID,
			AVG(tlUnitAmount) ugiUnitAmount
		FROM
			TransactionLines
		WHERE
			tlLineGroupID IS NULL
		GROUP BY
			tlTransactionID,
			tlItemID
		) ugi RIGHT JOIN
			shopcharttmp." . $prefix . "ResponsibleTransactions
		ON ugiTransactionID = rtTransactionID
		AND ugiItemID = rtItemID
    ) ON giTransactionID = rtTransactionID
	AND giLineGroupID = rtLineGroupID
	AND giItemID = rtItemID
GROUP BY
	rtTransactionID,
	rtFromPersonID,
	rtCurrentFlags,
	rtFlags,
	rtItemID,
	rtQtyUsed");
}

function make_directly_responsible($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "DirectlyResponsible (
	drItemID int PRIMARY KEY,
	drQuantity decimal(11, 2),
	drTotalAmount decimal(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "DirectlyResponsible (
	drItemID,
	drQuantity,
	drTotalAmount
) SELECT
	ticItemID,
	SUM((ticFlags & " . TTF_TRANSFERS_INVENTORY . ") * ticQuantity),
	SUM(
		CASE
			WHEN (ticCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") <> (ticFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") THEN
				CASE
					WHEN ticCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN ticUnitAmount / pMonetaryPerSecond
					ELSE ticUnitAmount * pMonetaryPerSecond
				END
			ELSE ticUnitAmount
		END * ticQuantity)
FROM
	shopcharttmp." . $prefix . "TransactionItemCost INNER JOIN
		People
	ON ticFromPersonID = pPersonID
WHERE
	NOT ticFlags & " . TTF_GENERAL_INVENTORY . "
GROUP BY
	ticItemID");
}

function make_general_invy_cost($prefix) {
    global $mysqli;
    $mysqli->query("
CREATE " . TEMPORARY . " TABLE shopcharttmp." . $prefix . "GeneralInvyCost (
	gicItemID int PRIMARY KEY,
	gicUnitAmount decimal(20, 6)
)");
    $mysqli->query("
INSERT INTO shopcharttmp." . $prefix . "GeneralInvyCost (
	gicItemID,
	gicUnitAmount
) SELECT
	ticItemID,
	SUM(
		CASE
			WHEN (ticCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") <> (ticFlags & " . TTF_SHOW_AMOUNT_AS_TIME . ") THEN
				CASE
					WHEN ticCurrentFlags & " . TTF_SHOW_AMOUNT_AS_TIME . " THEN ticUnitAmount / pMonetaryPerSecond
					ELSE ticUnitAmount * pMonetaryPerSecond
				END
			ELSE ticUnitAmount
		END * ticQuantity) / SUM(ticQuantity)
FROM
	shopcharttmp." . $prefix . "TransactionItemCost INNER JOIN
		People
	ON ticFromPersonID = pPersonID
WHERE
	ticFlags & " . TTF_GENERAL_INVENTORY . "
GROUP BY
	ticItemID");
}