<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once 'fun/transaction_types.php';
require_once 'fun/common.php';
require_once 'util/transaction_type_flags.php';
define("TRANSACTION_TYPE_PAGE_SIZE", 18);
$default_keys = array(
    'transaction_type_id' => 0);
$keys = get_settings($default_keys);
$default_settings = array(
    'p_page_number' => 1,
    'page_number' => 1,
    'description_filter' => '',
    'inventory_modifier_filter' => -1,
    'show_amount_as_filter' => -1);
$settings = get_settings($default_settings);
if(isset($_SERVER['HTTP_REFERER'])) {
    $referer = parse_url($_SERVER['HTTP_REFERER']);
    if($referer['path'] == '/ShopChart/transaction_types.php') {
        $settings['page_number'] = get_page_number('page_number', 'previous_page', 'next_page');
        if(isset($_POST['add_transaction_type'])) {
            try {
                $keys['transaction_type_id'] = add_transaction_type ($_POST['new_description'],
                    $_POST['new_inventory_modifier']);
            } catch(Exception $e) {
                $error = $e->getMessage();
            }
        } elseif($_SERVER['REQUEST_METHOD'] == 'POST') {
            try {
                update_attributes($keys['transaction_type_id'], $_POST);
            } catch(Exception $e) {
                $error = $e->getMessage();
            }
        }
    }
}
$transaction_types = get_transaction_types($settings['description_filter'], 
    $settings['inventory_modifier_filter'], $settings['show_amount_as_filter']);
$max_pages = get_max_pages($transaction_types, TRANSACTION_TYPE_PAGE_SIZE);
$settings['page_number'] = go_to_page($keys['transaction_type_id'], 'ttTransactionTypeID', $transaction_types,
    TRANSACTION_TYPE_PAGE_SIZE, $settings['page_number'], $max_pages,
    ($settings['p_page_number'] == $settings['page_number']));
print '
<!DOCTYPE html>
<html>
    <head>
        <style type="text/css">
            @import url(css/shop_chart.css);
            @import url(css/transaction_types.css);
        </style>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <title>Transaction Types</title>
    </head>
    <body>
        <form action="transaction_types.php" method="post">';
$nav_links = '
                    <div id="selected_nav_link">
                        <a href="transaction_types.php">Transaction Types</a>
                    </div>';
$help_link = 'doc/transaction_types.xhtml';
include 'top_nav.php';
print '
            <table id="transaction_type">
                <caption>Transaction Types</caption>';
if($max_pages > 1) {
    print '
                <tr class="page_nav">
                        <td colspan="3">
                            <input type="submit" name="previous_page" value="Previous" />
                            <input type="text" name="page_number" value="' . $settings['page_number'] . '" /> of ' .
        $max_pages . '
                            <input type="submit" name="next_page" value="Next" />
                        </td>
                    </tr>';
}
print '
                <tr class="field_header">
                    <th>Description</th>
                    <th>Inventory Modifier</th>
                    <th>Show Amount As</th>
                </tr>';
$inventory_modifiers = array(
    0 => 'None',
    TTF_TRANSFERS_INVENTORY => 'Transfer',
    TTF_ADDS_INVENTORY_TO_ASSEMBLY => 'Installation',
    TTF_REMOVES_INVENTORY_FROM_ASSEMBLY => 'Salvage',
    TTF_REMOVES_INVENTORY => 'Scrap');
$amount_types = array(
    0 => 'Currency',
    TTF_SHOW_AMOUNT_AS_TIME => 'Time');
print '
                <tr class="field_header">
                    <th><input type="text" name="description_filter" value="' .
    htmlspecialchars($settings['description_filter']) . '" /></th>
                    <th><select name="inventory_modifier_filter" onchange="this.form.submit();">
                        <option value="-1"></option>';
foreach($inventory_modifiers as $id => $description) {
    print '
                        <option value="' . $id . '"' . ($id == $settings['inventory_modifier_filter'] ?
        ' selected="true"' : '') . '>' . $description . '</option>';
}
print '
                    </select></th>
                    <th><select name="show_amount_as_filter" onchange="this.form.submit();">
                        <option value="-1"></option>';
foreach($amount_types as $id => $description) {
    print '
                        <option value="' . $id . '"' . ($id == $settings['show_amount_as_filter'] ?
        ' selected="true"' : '') . '>' . $description . '</option>';
}
print '
                    </select></th>
                </tr>';
$displayed = 0;
while ($transaction_type = $transaction_types->fetch_assoc()) {
    if($transaction_type['ttTransactionTypeID'] <> null) {
        if ($transaction_type['ttTransactionTypeID'] == $keys['transaction_type_id']) {
            $selected_transaction_type = $transaction_type;
            print '
                <tr class="selected_row">
                    <td class="description"><input type="text" name="ttDescription" value="' .
                htmlspecialchars($transaction_type['ttDescription']) . '" /></td>
                    <td class="inventory_modifier"><select name="ttInventoryModifier" onchange="this.form.submit();">';
            foreach($inventory_modifiers as $id => $name) {
                print '
                        <option value="' . $id . '"' . ($id & $transaction_type['ttFlags'] ? ' selected="true"'
                    : '') . '>' . $name . '</option>';
            }
            print '
                    </select></td>
                    <td class="amount_type"><select name="ttAmountType" onchange="this.form.submit();">';
            foreach($amount_types as $id => $name) {
                print '
                        <option value="' . $id . '"' . ($id & $transaction_type['ttFlags'] ? ' selected="true"' :
                    '') . '>' . $name . '</option>';
            }
            print '
                    </select></td>
                </tr>';
        } else {
            $inventory_modifier = 0;
            foreach($inventory_modifiers as $id => $name) {
                if($id & $transaction_type['ttFlags']) {
                    $inventory_modifier = $id;
                    break;
                }
            }
            $amount_type = 0;
            foreach($amount_types as $id => $name) {
                if($id & $transaction_type['ttFlags']) {
                    $amount_type = $id;
                    break;
                }
            }
            if($displayed % 2) {
                print '
                <tr class="odd_row">';
            } else {
                print '
                <tr class="even_row">';
            }
            print '
                    <td class="description"><a href="transaction_types.php?transaction_type_id=' .
                $transaction_type['ttTransactionTypeID'] . get_params($settings, false) . '">' .
                htmlspecialchars($transaction_type['ttDescription']) . '</a></td>
                    <td class="inventory_modifier">' . $inventory_modifiers[$inventory_modifier] . '</td>
                    <td class="amount_type">' . $amount_types[$amount_type] . '</td>
                </tr>';
        }
        if(++$displayed >= TRANSACTION_TYPE_PAGE_SIZE) {
        break;
    }
    }
}
print '
                <tr class="footer">
                    <td><input type="text" name="new_description" /></td>
                    <td><select name="new_inventory_modifier">';
foreach($inventory_modifiers as $id => $name) {
    print '
                        <option value="' . $id . '">' . $name . '</option>';
}
print '
                    </select></td>
                    <td>
                        <input type="submit" name="add_transaction_type" value="Add Transaction Type" />
                    </td>
                </tr>
            </table>';
if(isset($selected_transaction_type)) {
    print '
            <input type="hidden" name="p_page_number" value="' . $settings['page_number'] . '" />
            <input type="hidden" name="transaction_type_id" value="' . $keys['transaction_type_id'] . '" />
            <input type="hidden" name="p_ttDescription" value="' .
        htmlspecialchars($selected_transaction_type['ttDescription']) . '" />
            <input type="hidden" name="p_ttFlags" value="' . $selected_transaction_type['ttFlags'] . '" />
            <input type="hidden" name="p_ttAmounttype" value="' . $selected_transaction_type['ttAmountType'] .
        '" />';
}
print '
        </form>
    </body>
</html>';

