<?php
/*
This file is part of ShopChart.

ShopChart is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

ShopChart is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ShopChart.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once 'fun/common.php';
require_once 'fun/transactions.php';
require_once 'util/transaction_type_flags.php';
define("TRANSACTION_PAGE_SIZE", 18);
define("LINE_PAGE_SIZE", 20);
$default_keys = array(
    'transaction_id' => 0,
    'line_id' => 0,
    'line_group_id' => 0);
$keys = get_settings($default_keys);
$default_settings = array(
    'page_number' => 1,
    'p_page_number' => 1,
    'line_page_number' => 1,
    'p_line_page_number' => 1,
    'transaction_type_filter' => 0,
    'from_person_filter' => 0,
    'to_person_filter' => 0,
    'assembly_filter' => 0,
    'item_description_filter' => '',
    'item_assembly_filter' => 0,
    'line_group_filter' => 0,
    'new_item_description_filter' => '',
    'fua_transaction_id' => null,
    'fua_line_id' => null,
    'aii_assembly_id' => null,
    'aii_item_id' => null,
    'aii_person_id' => null,
    'qoh_item_id' => null);
$settings = get_settings($default_settings);
if(isset($_POST['confirm_delete'])) {
    delete_transaction($_POST['transaction_id']);
}
$max_lines_pages = 0;
if(isset($_SERVER['HTTP_REFERER'])) {
    $referer = parse_url($_SERVER['HTTP_REFERER']);
    if($referer['path'] == '/ShopChart/transactions.php' || $referer['path'] == '/ShopChart/items.php') {
        $settings['page_number'] = get_page_number('page_number', 'previous_page', 'next_page');
        $settings['line_page_number'] = get_page_number('line_page_number', 'previous_line_page',
            'next_line_page');
        if(isset($_POST['add_transaction'])) {
            try {
                $keys['transaction_id'] = add_transaction($_POST['new_transaction_type_id'],
                    $_POST['new_from_person_id'], $_POST['new_to_person_id']);
            } catch(Exception $e) {
                $error = $e->getMessage();
            }
            if($settings['transaction_type_filter'] <> 0 && $settings['transaction_type_filter'] <>
                $_POST['new_transaction_type_id']) {
                $settings['transaction_type_filter'] = $_POST['new_transaction_type_id'];
            }
            if($settings['from_person_filter'] <> 0 && $settings['from_person_filter'] <>
                $_POST['new_from_person_id']) {
                $settings['from_person_filter'] = $_POST['new_from_person_id'];
            }
            if($settings['to_person_filter'] <> 0 && $settings['to_person_filter'] <> $_POST['new_to_person_id']) {
                $settings['to_person_filter'] = $_POST['new_to_person_id'];
            }
            $settings['assembly_filter'] = 0;
        } elseif(isset($_POST['go_to_url'])) {
            header('Location: ' . $_POST['tlFromURL']);
        } else {
            if(isset($_POST['delete_transaction'])) {
                header('Location: delete.php?transaction_id=' . $keys['transaction_id'] .
                    '&sender=transactions.php' . get_params($settings, true));
            }
            if(isset($_REQUEST['item_id'])) {
                if($referer['path'] == '/ShopChart/items.php') {
                    try {
                        $keys['line_id'] = add_transaction_line($keys['transaction_id'], $_REQUEST['item_id']);
                    } catch(Exception $e) {
                        $error = $e->getMessage();
                    }
                }
            } elseif(isset($_REQUEST['assembly_id'])) {
                    try {
                        add_assembly($keys['transaction_id'], $_REQUEST['assembly_id']);
                    } catch(Exception $e) {
                        $error = $e->getMessage();
                    }
            }
            if(isset($_POST['add_transaction_item']) || (isset($_POST['p_new_item_description_filter']) &&
                    $_POST['p_new_item_description_filter'] <> $settings['new_item_description_filter'])) {
                header('Location: items.php?description_filter=' .
                    htmlspecialchars($settings['new_item_description_filter']) .
                    '&sender=transactions.php?transaction_id=' . $keys['transaction_id'] .
                    get_params($settings, true));
            }
            if($_SERVER['REQUEST_METHOD'] == 'POST') {
                try {
                    update_attributes($keys['transaction_id'], $keys['line_id'], $keys['line_group_id'], $_POST);
                } catch(Exception $e) {
                    $error = $e->getMessage();
                }
            }
        }
    }
}
$transaction_types = get_transaction_types();
$people = get_people();
$assemblies = get_assemblies();
$transactions = get_transactions($settings['transaction_type_filter'], $settings['from_person_filter'],
    $settings['to_person_filter'], $settings['assembly_filter'], $settings);
$max_pages = get_max_pages($transactions, TRANSACTION_PAGE_SIZE);
$settings['page_number'] = go_to_page($keys['transaction_id'], 'tTransactionID', $transactions,
    TRANSACTION_PAGE_SIZE, $settings['page_number'], $max_pages, ($settings['p_page_number'] ==
    $settings['page_number']));
print '
<!DOCTYPE html>
<html>
    <head>
        <style type="text/css">
            @import url(css/shop_chart.css);
            @import url(css/transactions.css);
        </style>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <title>Transactions</title>
    </head>
    <body>
        <form action="transactions.php" method="post">';
$nav_links = '
                    <div class="nav_link">
                        <a href="transaction_types.php">Transaction Types</a>
                    </div>';
include 'top_nav.php';
if(isset($settings['fua_transaction_id'])) {
    print fua_notice($settings, $transaction_types, $people, $keys);
} elseif(isset($settings['aii_assembly_id'])) {
    print aii_notice($settings, $people, $keys);
} elseif(isset($settings['qoh_item_id'])) {
    print qoh_notice($settings, $keys);
}
print '
            <div id="top_half">
                <table id="transactions">
                    <caption>Transactions</caption>';
if($max_pages > 1) {
    print '
                    <tr class="page_nav">
                        <td colspan="7">
                            <input type="submit" name="previous_page" value="Previous" />
                            <input type="text" name="page_number" class="page_num" value="' .
    $settings['page_number'] . '" /> of ' . $max_pages . '
                            <input type="submit" name="next_page" value="Next" />
                        </td>
                    </tr>';
}
print '
                    <tr class="field_header">
                        <th>Date</th>
                        <th>Type</th>
                        <th>From</th>
                        <th></th>
                        <th>To</th>
                        <th>To</th>
                        <th>Assembly</th>
                    </tr>
                    <tr class="field_header">
                        <th></th>
                        <th><select name="transaction_type_filter" onchange="this.form.submit();">
                            <option value="0"></option>';
foreach($transaction_types as $id => $fields) {
    if($fields['active']) {
        print '
                            <option value="' . $id . '"' . ($id == $settings['transaction_type_filter'] ?
        ' selected="true"' : '') . '>' . htmlspecialchars($fields['description']) . '</option>';
    }
}
print '
                        </select></th>
                        <th class="person"><select name="from_person_filter" onchange="this.form.submit();">
                            <option value="0"></option>';
foreach($people as $id => $fields) {
    if($fields['from']) {
        print '
                            <option value="' . $id . '"' . ($id == $settings['from_person_filter'] ?
        ' selected="true"' : '') . '>' . htmlspecialchars($fields['full_name']) . '</option>';
    }
}
print '
                        </select></th>
                        <th></th>
                        <th class="person"><select name="to_person_filter" onchange="this.form.submit();">
                            <option value="0"></option>';
foreach($people as $id => $fields) {
    if($fields['to']) {
        print '
                            <option value="' . $id . '"' . ($id == $settings['to_person_filter'] ?
        ' selected="true"' : '') . '>' . htmlspecialchars($fields['full_name']) . '</option>';
    }
}
print '
                        </select></th>
                        <th>Amt</th>
                        <th class="serial_number"><select name="assembly_filter" onchange="this.form.submit();">';
$assembly_options = '';
foreach($assemblies as $id => $serial_number) {
    $assembly_options .= '
                            <option value="' . $id . '"' . ($id == $settings['item_assembly_filter'] ?
        ' selected="true"' : '') . '>' . ($serial_number == '' ? '' : htmlspecialchars($serial_number)) .
        '</option>';
    print '
                            <option value="' . $id . '"' . ($id == $settings['assembly_filter'] ?
        ' selected="true"' : '') . '>' . ($serial_number == '' ? '' : htmlspecialchars($serial_number)) .
        '</option>';  
}
print '
                        </select></th>
                    </tr>';
$to_total = 0;
$from_total = 0;
$shipping_total = 0;
$tax_total = 0;
$displayed = 0;
define('MIXED_UNIT_TYPES', 1);
define('UNINITIALIZED', 2);
$unit_type = UNINITIALIZED;
while ($transaction = $transactions->fetch_assoc()) {
    if($transaction['tTransactionID'] <> null) {
        if (!isset($selected_transaction) && ($transaction['tTransactionID'] == $keys['transaction_id'] || 
            $keys['transaction_id'] == 0)) {
            $keys['transaction_id'] = $transaction['tTransactionID'];
            $selected_transaction = $transaction;
            $next_group_id = 1;
            $transaction_line_groups = get_transaction_line_groups($keys['transaction_id']);
            $transaction_lines = get_transaction_lines($transaction['tTransactionID'],
                $settings['item_description_filter'], $settings['line_group_filter'],
                $settings['item_assembly_filter']);
            if($keys['line_id'] == 0 && isset($_REQUEST['aii_item_id'])) {
                $keys['line_id'] = get_first_item_line_id($transaction['tTransactionID'], $_REQUEST['aii_item_id']);
            }
            $max_lines_pages = get_max_pages($transaction_lines, LINE_PAGE_SIZE);
            $settings['line_page_number'] = go_to_page($keys['line_id'], 'tlLineID', $transaction_lines,
                LINE_PAGE_SIZE, $settings['line_page_number'], $max_lines_pages,
                ($settings['p_line_page_number'] == $settings['line_page_number']));
            print '
                    <tr class="selected_row">
                        <td class="date"><input type="text" name="tDate" value="' . $transaction['TransactionDate'] .
                '" /></td>
                        <td class="transaction_type"><select name="tTransactionTypeID" onchange="this.form.submit();">';
            foreach($transaction_types as $transaction_type_id => $fields) {
                print '
                            <option value="' . $transaction_type_id . '"' .
                    ($transaction['tTransactionTypeID'] == $transaction_type_id ? ' selected="true"' : '') .
                    ' >' . htmlspecialchars($fields['description']) . '</option>';
            }
            print '
                        </select></td>
                        <td class="person"><select name="tFromPersonID" onchange="this.form.submit();">';
            foreach($people as $person_id => $fields) {
                print '
                            <option value="' . $person_id . '"' . ($transaction['tFromPersonID'] ==
                    $person_id ? ' selected="true"' : '') . '>' . htmlspecialchars($fields['full_name']) .
                    '</option>';
            }
            print '
                        </select></td>
                        <td></td>
                        <td class="person"><select name="tToPersonID" onchange="this.form.submit();">';
            foreach($people as $person_id => $fields) {
                print '
                            <option value="' . $person_id . '"' . ($transaction['tToPersonID'] ==
                    $person_id ? ' selected="true"' : '') . '>' . htmlspecialchars($fields['full_name']) .
                    '</option>';
            }
            print '
                        </select></td>';
            if($transaction['ttFlags'] & TTF_SHOW_AMOUNT_AS_TIME) {
                print '
                        <td class="time">' . gmdate("H:i:s", $transaction['tTotalAmount']) . '</td>';
            } else {
                print '
                        <td class="money">' . money_format('%.2n', $transaction['tTotalAmount']) . '</td>';
            }
            print '
                        <td class="serial_number">' .
                htmlspecialchars($assemblies[$transaction['tAssemblyID']]) . '</td>
                    </tr>';
        } else {
            if($displayed % 2) {
                print '
                    <tr class="odd_row">';
            } else {
                print '
                    <tr class="even_row">';
            }
            print '
                        <td class="date"><a href="transactions.php?transaction_id=' .
                $transaction['tTransactionID'] . get_params($settings, false) . '">' . $transaction['TransactionDate'] .
                '</a></td>
                        <td class="transaction_type">' .
                htmlspecialchars($transaction_types[$transaction['tTransactionTypeID']]['description']) . '</td>
                        <td class="person">' .
                htmlspecialchars($people[$transaction['tFromPersonID']]['full_name']) . '</td>';
            if($transaction['ttFlags'] & TTF_SHOW_AMOUNT_AS_TIME) {
                print '
                        <td></td>
                        <td class="person">' .
                    htmlspecialchars($people[$transaction['tToPersonID']]['full_name']) . '</td>
                            <td class="time">' . gmdate('H:i:s', $transaction['tTotalAmount']) . '</td>';
            } else {
                print '
                        <td></td>
                        <td class="person">' .
                    htmlspecialchars($people[$transaction['tToPersonID']]['full_name']) . '</td>
                        <td class="money">' . money_format('%.2n', $transaction['tTotalAmount']) . '</td>';
            }
            print '
                        <td class="serial_number">' .
                htmlspecialchars($assemblies[$transaction['tAssemblyID']]) . '</td>
                    </tr>';
        }
        if(($unit_type & MIXED_UNIT_TYPES) == 0) {
            if(($unit_type & UNINITIALIZED) == 0 && ($unit_type & TTF_SHOW_AMOUNT_AS_TIME) <>
                ($transaction['ttFlags'] & TTF_SHOW_AMOUNT_AS_TIME)) {
                $unit_type = ($unit_type | MIXED_UNIT_TYPES);
            } else {
                $unit_type = ($transaction['ttFlags'] & TTF_SHOW_AMOUNT_AS_TIME);
                $to_total += $transaction['tTotalAmount'];
//                $from_total += $transaction['tFromTotalAmount'];
            }
        }
        $shipping_total += $transaction['tShipping'];
        $tax_total += $transaction['tTax'];
        if(++$displayed >= TRANSACTION_PAGE_SIZE) {
            break;
        }
    }
}
print '
                    <tr class="footer">
                        <td></td>
                        <td class="transaction_type"><select name="new_transaction_type_id">';
foreach($transaction_types as $transaction_type_id => $fields) {
    print '
                            <option value="' . $transaction_type_id . '">' .
        htmlspecialchars($fields['description']) . '</option>';
}
print '
                        </select></td>';
print '
                        <td class="person"><select name="new_from_person_id">';
foreach($people as $person_id => $fields) {
    print '
                            <option value="' . $person_id . '"' . ($person_id == $settings['user'] ?
        'selected="true"' : '') . '>' . htmlspecialchars($fields['full_name']) . '</option>';
    }
print '
                        </select></td>';
if(($unit_type & MIXED_UNIT_TYPES) == 0) {
    if($unit_type & TTF_SHOW_AMOUNT_AS_TIME) {
        print '
                        <td class="time">' . gmdate('H:i:s', $from_total) . '</td>';
    } else {
        print '
                        <td class="money">' . money_format('%.2n', $from_total) . '</td>';
    }
} else {
    print '
                        <td></td>';
}
print '
                        <td class="person"><select name="new_to_person_id">';
foreach($people as $person_id => $fields) {
    print '
                            <option value="' . $person_id . '"' . ($person_id == $settings['user'] ?
        'selected="true"' : '') . '>' . htmlspecialchars($fields['full_name']) . '</option>';
}
print '
                        </select></td>';
if(($unit_type & MIXED_UNIT_TYPES) == 0) {
    if($unit_type & TTF_SHOW_AMOUNT_AS_TIME) {
        print '
                        <td class="time">' . gmdate('H:i:s', $to_total) . '</td>';
    } else {
        print '
                        <td class="money">' . money_format('%.2n', $to_total) . '</td>';
    }
} else {
    print '
                        <td></td>';
}
print '
                        <td><input type="submit" name="add_transaction" value="Add Transaction"' .
    (count($transaction_types) == 0 ? ' disabled="true"' : '') . ' /></td>
                    </tr>
                </table>
                <table id="transaction_attributes">
                    <caption>Transaction Attributes</caption>
                    <tr class="field_header">
                        <th>Name</th>
                        <th>Value</th>
                    </tr>';
if(isset($selected_transaction)) {
    print '
                    <tr>
                        <td>Shipping</td>
                        <td class="money"><input type="text" name="tShipping" value="' .
        money_format('%.2n', $selected_transaction['tShipping']) . '" /></td>
                    </tr>
                    <tr>
                        <td>Tax</td>
                        <td class="money"><input type="text" name="tTax" value="' . money_format('%.2n',
        $selected_transaction['tTax']) .
        '" /></td>
                    </tr>';
} else {
    print '
                    <tr>
                        <td>Shipping</td>
                        <td class="money">' . money_format('%.2n', $shipping_total) . '</td>
                    </tr>
                    <tr>
                        <td>Tax</td>
                        <td class="money">' . money_format('%.2n', $tax_total) . '</td>
                    </tr>';
}
print '
                </table>
            </div>
            <div id="bottom_half">
                <table id="transaction_line_groups">
                    <caption>Transaction Line Groups</caption>
                    <tr class="field_header">
                        <th>Group</th>
                        <th>Total Amt</th>
                    <tr>';
$displayed = 0;
if(isset($transaction_line_groups)) {
    foreach($transaction_line_groups as $id => $total) {
        if(!isset($selected_line_group) && ($id == $keys['line_group_id'] || $keys['line_group_id'] ==0)) {
            $keys['line_group_id'] = $id;
            $selected_line_group = $id;
            print '
                    <tr class="selected_row">
                        <td class="line_group">' . $id . '</td>';
            if(($selected_transaction['ttFlags'] & TTF_SHOW_AMOUNT_AS_TIME) == 0) {
                print '
                        <td class="money">
                            <input type="text" name="tlgTotalAmount" value="' . money_format('%.2n', $total)
                    . '" />';
            } else {
                print '
                        <td class="time_from_start">
                            <input type="text" name="tlgTotalAmount" value="' . gmdate('H:i:s', $total) . '" />
                            <input type="submit" name="line_group_tfs" value="TFS"/>';
            }
            print '
                        </td>
                    <tr>';
        } else {
            if($displayed % 2) {
                print '
                    <tr class="odd_row">';
            } else {
                print '
                    <tr class="even_row">';
            }
            print '
                        <td class="line_group"><a href="transactions.php?transaction_id=' .
                $keys['transaction_id'] . '&line_id=' . $keys['line_id'] . '&line_group_id=' . $id .
                get_params($settings, false) . '">' . $id . '</a></td>';
            if($selected_transaction['ttFlags'] & TTF_SHOW_AMOUNT_AS_TIME) {
                print '
                        <td class="time">' . gmdate('H:i:s', $total) . '</td>
                    </tr>';
            } else {
                print '
                        <td class="money">' . money_format('%.2n', $total) . '</td>
                    </tr>';
            }
        }
        ++$displayed;
        $next_group_id = $id + 1;
    }
}
print '
                </table>
                <table id="transaction_lines">
                    <caption>Transaction Lines</caption>';
if($max_lines_pages > 1) {
    print '
                    <tr class="page_nav">
                        <td colspan="8">
                            <input type="submit" name="previous_line_page" value="Previous" />
                            <input type="text" name="line_page_number" value="' . $settings['line_page_number'] .
    '" /> of ' . $max_lines_pages . '
                            <input type="submit" name="next_line_page" value="Next" />
                        </td>
                    </tr>';
}
print '
                    <tr class="field_header">
                        <th>Description</th>
                        <th>Group</th>
                        <th>Quantity</th>
                        <th>From</th>
                        <th>From</th>
                        <th>To Unit</th>
                        <th>To Ext</th>
                        <th>Assembly</th>
                    </tr>
                    <tr class="field_header">
                        <th><input type="text" name="item_description_filter" value="' .
    htmlspecialchars($settings['item_description_filter']) . '" /></th>
                        <th><select name="line_group_filter" onchange="this.form.submit();">
                            <option value="0"></option>';
if(isset($transaction_line_groups)) {
    foreach($transaction_line_groups as $id => $total) {
        print '
                            <option value="' . $id . '"' . ($id == $settings['line_group_filter'] ?
        ' selected="true"' : '') . '>' . $id . '</option>';
    }
}
print '
                        </select></th>
                        <th></th>
                        <th>Unit Amt</th>
                        <th>Ext Amt</th>
                        <th>Amt</th>
                        <th>Amt</th>
                        <th class="serial_number"><select name="item_assembly_filter" onchange="this.form.submit();">' . $assembly_options . '
                        </select></th>
                    </tr>';
if(isset($transaction_lines)) {
    $displayed = 0;
    while($transaction_line = $transaction_lines->fetch_assoc()) {
        if(($selected_transaction['ttFlags'] & TTF_SHOW_AMOUNT_AS_TIME) == 0) {
            if($transaction_line['tlfaHasUnitAmount']) {
                $from_amount = '
                        <td class="money">0.0000</td>';
                $from_total = '
                        <td class="money">0.00</td>';
            } else {
                $from_amount = '
                        <td class="money"><a href="transactions.php?fua_transaction_id=' . $keys['transaction_id'] .
                    '&fua_line_id=' . $transaction_line['tlLineID'] . '">' .
                    money_format('%.4n', $transaction_line['tlfaUnitAmount']) . '</a></td>';
                $from_total = '
                        <td class="money">' . money_format('%.2n', $transaction_line['tlfaUnitAmount'] *
                $transaction_line['tlQuantity']) . '</td>';
            }
            $to_amount = '
                        <td class="money">' . money_format('%.4n', $transaction_line['tlUnitAmount']) . '</td>';
            $to_total = '
                        <td class="money">' . money_format('%.2n', $transaction_line['tlUnitAmount'] *
                $transaction_line['tlQuantity']) . '</td>';
        } else {
            if($transaction_line['tlfaHasUnitAmount']) {
                $from_amount = '
                        <td class="time">00:00:00</td>';
                $from_total = '
                        <td class="time">00:00:00</td>';
            } else {
                $from_amount = '
                        <td class="time"><a href="transactions.php?fua_transaction_id=' . $keys['transaction_id'] .
                    '&fua_line_id=' . $transaction_line['tlLineID'] . '">' . gmdate('H:i:s',
                    $transaction_line['tlfaUnitAmount']) . '</a></td>';
                $from_total = '
                        <td class="time">' . gmdate('H:i:s', $transaction_line['tlfaUnitAmount'] *
                $transaction_line['tlQuantity']) . '</td>';
            }
            $to_amount = '
                        <td class="time">' . gmdate('H:i:s', $transaction_line['tlUnitAmount']) . '</td>';
            $to_total = '
                        <td class="time">' . gmdate('H:i:s', $transaction_line['tlUnitAmount'] *
                $transaction_line['tlQuantity']) . '</td>';
        }
        if(!isset($selected_line) && ($transaction_line['tlLineID'] == $keys['line_id'] || $keys['line_id'] == 0)) {
            $keys['line_id'] = $transaction_line['tlLineID'];
            $selected_line = $transaction_line;
            print '
                    <tr class="selected_row">
                        <td class="item_description">' . htmlspecialchars($transaction_line['iDescription']) . '</td>
                        <td class="line_group"><select name="tlLineGroupID" onchange="this.form.submit();">
                            <option value="0"></option>';
            foreach($transaction_line_groups as $id => $tlg_total) {
                print '
                            <option value="' . $id . '"' . ($id == $transaction_line['tlLineGroupID'] ?
                    ' selected="true"' : '') . '>' . $id . '</option>';
            }
            print '
                            <option value="' . $next_group_id . '">' . $next_group_id . '</option>
                        </select></td>
                        <td class="quantity"><input type="text" name="tlQuantity" value="' .
                $transaction_line['tlQuantity'] . '" /></td>' . $from_amount . $from_total;
            if($transaction_line['tlLineGroupID'] == null) {
                if($selected_transaction['ttFlags'] & TTF_SHOW_AMOUNT_AS_TIME) {
                    print $to_amount . '
                        <td class="time_from_start">
                            <input type="text" name="tlExtAmount" value="' . gmdate('H:i:s',
                        $transaction_line['tlUnitAmount'] * $transaction_line['tlQuantity']) . '" />
                            <input type="submit" name="line_tfs" value="TFS" />
                        </td>';
                } else {
                    print '
                        <td class="money"><input type="text" name="tlUnitAmount" value="' . money_format('%.4n',
                        $transaction_line['tlUnitAmount']) . '" /></td>' . $to_total;
                } 
            } else {
                print $to_amount . $to_total;
            }
            print '
                        <td class="serial_number"><select name="tlAssemblyID" onchange="this.form.submit();">';
            foreach($assemblies as $assembly_id => $serial_number) {
                print '
                            <option value="' . $assembly_id . '"' . ($transaction_line['tlAssemblyID'] ==
                    $assembly_id ? ' selected="true"' : '') . '>' . htmlspecialchars($serial_number) .
                    '</option>';
            }
            print '
                        </select></td>
                    </tr>';
        } else {
            if($displayed % 2) {
                print '
                    <tr class="odd_row">';
            } else {
                print '
                    <tr class="even_row">';
            }
            print '
                        <td class="item_description"><a href="transactions.php?transaction_id=' .
                $keys['transaction_id'] . '&line_id=' . $transaction_line['tlLineID'] . '&line_group_id=' .
                $keys['line_group_id'] . get_params($settings, false) . '">' .
                htmlspecialchars($transaction_line['iDescription']) . '</a></td>
                        <td class="line_group">' . $transaction_line['tlLineGroupID'] . '</td>
                        <td class="quantity">' . $transaction_line['tlQuantity'] . '</td>' . $from_amount .
                $from_total . $to_amount . $to_total . '
                        <td class="serial_number">' .
                htmlspecialchars($assemblies[$transaction_line['tlAssemblyID']]) . '</td>
                    </tr>';
        }
        if(++$displayed >= LINE_PAGE_SIZE) {
            break;
        }
    }
    print '
                    <tr class="footer">
                        <td class="item_description"><input type="text" name="new_item_description_filter" value="' .
        $settings['new_item_description_filter'] . '" /></td>
                        <td colspan="8"><input type="submit" name="add_transaction_item" value="Add Item" /></td>
                    </tr>';
} else {
    print '
                    <tr class="footer">
                        <td><input type="text" disabled="true" /></td>
                        <td colspan="8"><input type="submit" value="Add Item" disabled="true" /></td>
                    </tr>';
}
print '
                </table>
                <table id="transaction_line_attributes">
                    <caption>Transaction Line Attributes</caption>
                    <tr class="field_header">
                        <th>Name</th>
                        <th>Value</th>
                    </tr>';
if(isset($selected_transaction)) {
    if(isset($selected_line)) {
        print '
                    <tr>
                        <td>From URL</td>
                        <td class="text_attribute">
                            <input type="text" name="tlFromURL" value="' . $selected_line['tlFromURL'] . '" />
                        </td>
                    </tr>
                </table>
                <input type="hidden" name="line_id" value="' . $keys['line_id'] . '" />
                <input type="hidden" name="p_tlExtAmount" value="' . ($selected_line['tlUnitAmount'] *
            $selected_line['tlQuantity']) . '" />
                <input type="hidden" name="p_tlUnitAmount" value="' . $selected_line['tlUnitAmount'] . '" />
                <input type="hidden" name="p_tlQuantity" value="' . $selected_line['tlQuantity'] . '" />
                <input type="hidden" name="p_tlLineGroupID" value="' . ($selected_line['tlLineGroupID'] == null ?
            '0' : $selected_line['tlLineGroupID']) . '" />
                <input type="hidden" name="p_tlAssemblyID" value="' . $selected_line['tlAssemblyID'] . '" />
                <input type="hidden" name="p_tlFromURL" value="' . $selected_line['tlFromURL'] . '" />';
    } else {
        print '
                </table>
            </div>';
    }
    print '
                <input type="hidden" name="p_page_number" value="' . $settings['page_number'] . '" />
                <input type="hidden" name="p_line_page_number" value="' . $settings['line_page_number'] . '" />
                <input type="hidden" name="transaction_id" value="' . $keys['transaction_id'] . '" />
                <input type="hidden" name="line_group_id" value="' . $keys['line_group_id'] . '" />
                <input type="hidden" name="p_tDate" value="' . $selected_transaction['TransactionDate'] . '" />
                <input type="hidden" name="p_tTransactionTypeID" value="' .
        $selected_transaction['tTransactionTypeID'] . '" />
                <input type="hidden" name="p_tFromPersonID" value="' . $selected_transaction['tFromPersonID'] . '" />
                <input type="hidden" name="p_tToPersonID" value="' . $selected_transaction['tToPersonID'] . '" />
                <input type="hidden" name="p_tShipping" value="' . $selected_transaction['tShipping'] . '" />
                <input type="hidden" name="p_tTax" value="' . $selected_transaction['tTax'] . '" />
                <input type="hidden" name="p_new_item_description_filter" value="' . 
        $settings['new_item_description_filter'] . '" />
            </div>';
} else {
print '
                </table>
            </div>
            <input type="hidden" name="p_page_number" value="' . $settings['page_number'] . '" />
            <input type="hidden" name="p_line_page_number" value="' . $settings['line_page_number'] . '" />';
}
print '
        </form>
    </body>
</html>';

